@@ -30,6 +30,11 @@ func TestParse(t *testing.T) {
 						Capabilities: []string{
 							NamespaceCapabilityListJobs,
 							NamespaceCapabilityReadJob,
+							NamespaceCapabilityCSIListVolume,
+							NamespaceCapabilityCSIReadVolume,
+							NamespaceCapabilityReadJobScaling,
+							NamespaceCapabilityListScalingPolicies,
+							NamespaceCapabilityReadScalingPolicy,
 						},
 					},
 				},
@@ -46,6 +51,9 @@ func TestParse(t *testing.T) {
 			namespace "secret" {
 				capabilities = ["deny", "read-logs"]
 			}
+			namespace "autoscaler" {
+				policy = "scale"
+			}
 			agent {
 				policy = "read"
 			}
@@ -58,6 +66,9 @@ func TestParse(t *testing.T) {
 			quota {
 				policy = "read"
 			}
+			plugin {
+				policy = "read"
+			}
 			`,
 			"",
 			&Policy{
@@ -68,6 +79,11 @@ func TestParse(t *testing.T) {
 						Capabilities: []string{
 							NamespaceCapabilityListJobs,
 							NamespaceCapabilityReadJob,
+							NamespaceCapabilityCSIListVolume,
+							NamespaceCapabilityCSIReadVolume,
+							NamespaceCapabilityReadJobScaling,
+							NamespaceCapabilityListScalingPolicies,
+							NamespaceCapabilityReadScalingPolicy,
 						},
 					},
 					{
@@ -76,12 +92,20 @@ func TestParse(t *testing.T) {
 						Capabilities: []string{
 							NamespaceCapabilityListJobs,
 							NamespaceCapabilityReadJob,
+							NamespaceCapabilityCSIListVolume,
+							NamespaceCapabilityCSIReadVolume,
+							NamespaceCapabilityReadJobScaling,
+							NamespaceCapabilityListScalingPolicies,
+							NamespaceCapabilityReadScalingPolicy,
+							NamespaceCapabilityScaleJob,
 							NamespaceCapabilitySubmitJob,
 							NamespaceCapabilityDispatchJob,
 							NamespaceCapabilityReadLogs,
 							NamespaceCapabilityReadFS,
 							NamespaceCapabilityAllocExec,
 							NamespaceCapabilityAllocLifecycle,
+							NamespaceCapabilityCSIMountVolume,
+							NamespaceCapabilityCSIWriteVolume,
 						},
 					},
 					{
@@ -91,6 +115,16 @@ func TestParse(t *testing.T) {
 							NamespaceCapabilityReadLogs,
 						},
 					},
+					{
+						Name:   "autoscaler",
+						Policy: PolicyScale,
+						Capabilities: []string{
+							NamespaceCapabilityListScalingPolicies,
+							NamespaceCapabilityReadScalingPolicy,
+							NamespaceCapabilityReadJobScaling,
+							NamespaceCapabilityScaleJob,
+						},
+					},
 				},
 				Agent: &AgentPolicy{
 					Policy: PolicyRead,
@@ -104,6 +138,9 @@ func TestParse(t *testing.T) {
 				Quota: &QuotaPolicy{
 					Policy: PolicyRead,
 				},
+				Plugin: &PluginPolicy{
+					Policy: PolicyRead,
+				},
 			},
 		},
 		{
@@ -246,6 +283,28 @@ func TestParse(t *testing.T) {
 			"Invalid host volume name",
 			nil,
 		},
+		{
+			`
+			plugin {
+				policy = "list"
+			}
+			`,
+			"",
+			&Policy{
+				Plugin: &PluginPolicy{
+					Policy: PolicyList,
+				},
+			},
+		},
+		{
+			`
+			plugin {
+				policy = "reader"
+			}
+			`,
+			"Invalid plugin policy",
+			nil,
+		},
 	}
 
 	for idx, tc := range tcases {
@@ -268,3 +327,16 @@ func TestParse(t *testing.T) {
 		})
 	}
 }
+
+func TestParse_BadInput(t *testing.T) {
+	inputs := []string{
+		`namespace "\500" {}`,
+	}
+
+	for i, c := range inputs {
+		t.Run(fmt.Sprintf("%d: %v", i, c), func(t *testing.T) {
+			_, err := Parse(c)
+			assert.Error(t, err)
+		})
+	}
+}
