@@ -1,14 +1,15 @@
 package api
 
 import (
+	"context"
+	"fmt"
+	"os"
 	"reflect"
 	"sort"
 	"testing"
 
 	"time"
 
-	"github.com/hashicorp/nomad/helper"
-	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/stretchr/testify/require"
 )
 
@@ -35,9 +36,9 @@ func TestAllocations_List(t *testing.T) {
 	return
 
 	//job := &Job{
-	//ID:   helper.StringToPtr("job1"),
-	//Name: helper.StringToPtr("Job #1"),
-	//Type: helper.StringToPtr(JobTypeService),
+	//ID:   stringToPtr("job1"),
+	//Name: stringToPtr("Job #1"),
+	//Type: stringToPtr(JobTypeService),
 	//}
 	//eval, _, err := c.Jobs().Register(job, nil)
 	//if err != nil {
@@ -82,9 +83,9 @@ func TestAllocations_PrefixList(t *testing.T) {
 	return
 
 	//job := &Job{
-	//ID:   helper.StringToPtr("job1"),
-	//Name: helper.StringToPtr("Job #1"),
-	//Type: helper.StringToPtr(JobTypeService),
+	//ID:   stringToPtr("job1"),
+	//Name: stringToPtr("Job #1"),
+	//Type: stringToPtr(JobTypeService),
 	//}
 
 	//eval, _, err := c.Jobs().Register(job, nil)
@@ -130,13 +131,13 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 	t.Parallel()
 	// Create a job, task group and alloc
 	job := &Job{
-		Name:      helper.StringToPtr("foo"),
-		Namespace: helper.StringToPtr(DefaultNamespace),
-		ID:        helper.StringToPtr("bar"),
-		ParentID:  helper.StringToPtr("lol"),
+		Name:      stringToPtr("foo"),
+		Namespace: stringToPtr(DefaultNamespace),
+		ID:        stringToPtr("bar"),
+		ParentID:  stringToPtr("lol"),
 		TaskGroups: []*TaskGroup{
 			{
-				Name: helper.StringToPtr("bar"),
+				Name: stringToPtr("bar"),
 				Tasks: []*Task{
 					{
 						Name: "task1",
@@ -148,11 +149,11 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 	job.Canonicalize()
 
 	alloc := &Allocation{
-		ID:        uuid.Generate(),
+		ID:        generateUUID(),
 		Namespace: DefaultNamespace,
-		EvalID:    uuid.Generate(),
+		EvalID:    generateUUID(),
 		Name:      "foo-bar[1]",
-		NodeID:    uuid.Generate(),
+		NodeID:    generateUUID(),
 		TaskGroup: *job.TaskGroups[0].Name,
 		JobID:     *job.ID,
 		Job:       job,
@@ -176,8 +177,8 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 		{
 			desc: "no reschedule events",
 			reschedulePolicy: &ReschedulePolicy{
-				Attempts: helper.IntToPtr(3),
-				Interval: helper.TimeToPtr(15 * time.Minute),
+				Attempts: intToPtr(3),
+				Interval: timeToPtr(15 * time.Minute),
 			},
 			expAttempted: 0,
 			expTotal:     3,
@@ -185,8 +186,8 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 		{
 			desc: "all reschedule events within interval",
 			reschedulePolicy: &ReschedulePolicy{
-				Attempts: helper.IntToPtr(3),
-				Interval: helper.TimeToPtr(15 * time.Minute),
+				Attempts: intToPtr(3),
+				Interval: timeToPtr(15 * time.Minute),
 			},
 			time: time.Now(),
 			rescheduleTracker: &RescheduleTracker{
@@ -202,8 +203,8 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 		{
 			desc: "some reschedule events outside interval",
 			reschedulePolicy: &ReschedulePolicy{
-				Attempts: helper.IntToPtr(3),
-				Interval: helper.TimeToPtr(15 * time.Minute),
+				Attempts: intToPtr(3),
+				Interval: timeToPtr(15 * time.Minute),
 			},
 			time: time.Now(),
 			rescheduleTracker: &RescheduleTracker{
@@ -240,9 +241,56 @@ func TestAllocations_RescheduleInfo(t *testing.T) {
 
 }
 
+// TestAllocations_ExecErrors ensures errors are properly formatted
+func TestAllocations_ExecErrors(t *testing.T) {
+	c, s := makeClient(t, nil, nil)
+	defer s.Stop()
+	a := c.Allocations()
+
+	job := &Job{
+		Name:      stringToPtr("foo"),
+		Namespace: stringToPtr(DefaultNamespace),
+		ID:        stringToPtr("bar"),
+		ParentID:  stringToPtr("lol"),
+		TaskGroups: []*TaskGroup{
+			{
+				Name: stringToPtr("bar"),
+				Tasks: []*Task{
+					{
+						Name: "task1",
+					},
+				},
+			},
+		},
+	}
+	job.Canonicalize()
+
+	allocID := generateUUID()
+
+	alloc := &Allocation{
+		ID:        allocID,
+		Namespace: DefaultNamespace,
+		EvalID:    generateUUID(),
+		Name:      "foo-bar[1]",
+		NodeID:    generateUUID(),
+		TaskGroup: *job.TaskGroups[0].Name,
+		JobID:     *job.ID,
+		Job:       job,
+	}
+	// Querying when no allocs exist returns nothing
+	sizeCh := make(chan TerminalSize, 1)
+
+	// make a request that will result in an error
+	// ensure the error is what we expect
+	exitCode, err := a.Exec(context.Background(), alloc, "bar", false, []string{"command"}, os.Stdin, os.Stdout, os.Stderr, sizeCh, nil)
+
+	require.Equal(t, exitCode, -2)
+	require.Equal(t, err.Error(), fmt.Sprintf("Unknown allocation \"%s\"", allocID))
+}
+
 func TestAllocations_ShouldMigrate(t *testing.T) {
 	t.Parallel()
-	require.True(t, DesiredTransition{Migrate: helper.BoolToPtr(true)}.ShouldMigrate())
+	require.True(t, DesiredTransition{Migrate: boolToPtr(true)}.ShouldMigrate())
 	require.False(t, DesiredTransition{}.ShouldMigrate())
-	require.False(t, DesiredTransition{Migrate: helper.BoolToPtr(false)}.ShouldMigrate())
+	require.False(t, DesiredTransition{Migrate: boolToPtr(false)}.ShouldMigrate())
 }
