@@ -9,9 +9,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/hashicorp/nomad/helper/uuid"
-	"github.com/hashicorp/nomad/nomad/structs"
-	"github.com/hashicorp/nomad/testutil"
+	"github.com/hashicorp/nomad/api/internal/testutil"
 	"github.com/stretchr/testify/require"
 )
 
@@ -184,8 +182,8 @@ func TestNodes_ToggleDrain(t *testing.T) {
 	// Check again
 	out, _, err = nodes.Info(nodeID, nil)
 	require.Nil(err)
-	if out.SchedulingEligibility != structs.NodeSchedulingIneligible {
-		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, structs.NodeSchedulingIneligible)
+	if out.SchedulingEligibility != NodeSchedulingIneligible {
+		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, NodeSchedulingIneligible)
 	}
 
 	// Toggle off again
@@ -202,7 +200,7 @@ func TestNodes_ToggleDrain(t *testing.T) {
 	if out.DrainStrategy != nil {
 		t.Fatalf("drain strategy should be unset")
 	}
-	if out.SchedulingEligibility != structs.NodeSchedulingEligible {
+	if out.SchedulingEligibility != NodeSchedulingEligible {
 		t.Fatalf("should be eligible")
 	}
 }
@@ -236,7 +234,7 @@ func TestNodes_ToggleEligibility(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %s", err)
 	}
-	if out.SchedulingEligibility != structs.NodeSchedulingEligible {
+	if out.SchedulingEligibility != NodeSchedulingEligible {
 		t.Fatalf("node should be eligible")
 	}
 
@@ -252,8 +250,8 @@ func TestNodes_ToggleEligibility(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %s", err)
 	}
-	if out.SchedulingEligibility != structs.NodeSchedulingIneligible {
-		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, structs.NodeSchedulingIneligible)
+	if out.SchedulingEligibility != NodeSchedulingIneligible {
+		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, NodeSchedulingIneligible)
 	}
 
 	// Toggle on
@@ -268,8 +266,8 @@ func TestNodes_ToggleEligibility(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %s", err)
 	}
-	if out.SchedulingEligibility != structs.NodeSchedulingEligible {
-		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, structs.NodeSchedulingEligible)
+	if out.SchedulingEligibility != NodeSchedulingEligible {
+		t.Fatalf("bad eligibility: %v vs %v", out.SchedulingEligibility, NodeSchedulingEligible)
 	}
 	if out.DrainStrategy != nil {
 		t.Fatalf("drain strategy should be unset")
@@ -353,30 +351,6 @@ func TestNodes_Sort(t *testing.T) {
 	}
 }
 
-func TestNodes_GC(t *testing.T) {
-	t.Parallel()
-	require := require.New(t)
-	c, s := makeClient(t, nil, nil)
-	defer s.Stop()
-	nodes := c.Nodes()
-
-	err := nodes.GC(uuid.Generate(), nil)
-	require.NotNil(err)
-	require.True(structs.IsErrUnknownNode(err))
-}
-
-func TestNodes_GcAlloc(t *testing.T) {
-	t.Parallel()
-	require := require.New(t)
-	c, s := makeClient(t, nil, nil)
-	defer s.Stop()
-	nodes := c.Nodes()
-
-	err := nodes.GcAlloc(uuid.Generate(), nil)
-	require.NotNil(err)
-	require.True(structs.IsErrUnknownAllocation(err))
-}
-
 // Unittest monitorDrainMultiplex when an error occurs
 func TestNodes_MonitorDrain_Multiplex_Bad(t *testing.T) {
 	t.Parallel()
@@ -531,3 +505,129 @@ func TestNodes_DrainStrategy_Equal(t *testing.T) {
 	o.IgnoreSystemJobs = true
 	require.True(d.Equal(o))
 }
+
+func TestNodes_Purge(t *testing.T) {
+	t.Parallel()
+	require := require.New(t)
+	c, s := makeClient(t, nil, func(c *testutil.TestServerConfig) {
+		c.DevMode = true
+	})
+	defer s.Stop()
+
+	// Purge on a nonexistent node fails.
+	_, _, err := c.Nodes().Purge("12345678-abcd-efab-cdef-123456789abc", nil)
+	if err == nil || !strings.Contains(err.Error(), "not found") {
+		t.Fatalf("expected not found error, got: %#v", err)
+	}
+
+	// Wait for node registration and get the ID so we can attempt to purge a
+	// node that exists.
+	var nodeID string
+	testutil.WaitForResult(func() (bool, error) {
+		out, _, err := c.Nodes().List(nil)
+		if err != nil {
+			return false, err
+		}
+		if n := len(out); n != 1 {
+			return false, fmt.Errorf("expected 1 node, got: %d", n)
+		}
+		nodeID = out[0].ID
+		return true, nil
+	}, func(err error) {
+		t.Fatalf("err: %s", err)
+	})
+
+	// Perform the node purge and check the response objects.
+	out, meta, err := c.Nodes().Purge(nodeID, nil)
+	require.Nil(err)
+	require.NotNil(out)
+
+	// We can't use assertQueryMeta here, as the RPC response does not populate
+	// the known leader field.
+	require.Greater(meta.LastIndex, uint64(0))
+}
+
+func TestNodeStatValueFormatting(t *testing.T) {
+	t.Parallel()
+
+	cases := []struct {
+		expected string
+		value    StatValue
+	}{
+		{
+			"true",
+			StatValue{BoolVal: boolToPtr(true)},
+		},
+		{
+			"false",
+			StatValue{BoolVal: boolToPtr(false)},
+		},
+		{
+			"myvalue",
+			StatValue{StringVal: stringToPtr("myvalue")},
+		},
+		{
+			"2.718",
+			StatValue{
+				FloatNumeratorVal: float64ToPtr(2.718),
+			},
+		},
+		{
+			"2.718 / 3.14",
+			StatValue{
+				FloatNumeratorVal:   float64ToPtr(2.718),
+				FloatDenominatorVal: float64ToPtr(3.14),
+			},
+		},
+		{
+			"2.718 MHz",
+			StatValue{
+				FloatNumeratorVal: float64ToPtr(2.718),
+				Unit:              "MHz",
+			},
+		},
+		{
+			"2.718 / 3.14 MHz",
+			StatValue{
+				FloatNumeratorVal:   float64ToPtr(2.718),
+				FloatDenominatorVal: float64ToPtr(3.14),
+				Unit:                "MHz",
+			},
+		},
+		{
+			"2",
+			StatValue{
+				IntNumeratorVal: int64ToPtr(2),
+			},
+		},
+		{
+			"2 / 3",
+			StatValue{
+				IntNumeratorVal:   int64ToPtr(2),
+				IntDenominatorVal: int64ToPtr(3),
+			},
+		},
+		{
+			"2 MHz",
+			StatValue{
+				IntNumeratorVal: int64ToPtr(2),
+				Unit:            "MHz",
+			},
+		},
+		{
+			"2 / 3 MHz",
+			StatValue{
+				IntNumeratorVal:   int64ToPtr(2),
+				IntDenominatorVal: int64ToPtr(3),
+				Unit:              "MHz",
+			},
+		},
+	}
+
+	for i, c := range cases {
+		t.Run(fmt.Sprintf("case %d %v", i, c.expected), func(t *testing.T) {
+			formatted := c.value.String()
+			require.Equal(t, c.expected, formatted)
+		})
+	}
+}
