@@ -217,6 +217,19 @@ func NewDefaultReschedulePolicy(jobType string) *ReschedulePolicy {
 			MaxDelay:      timeToPtr(0),
 			Unlimited:     boolToPtr(false),
 		}
+
+	default:
+		// GH-7203: it is possible an unknown job type is passed to this
+		// function and we need to ensure a non-nil object is returned so that
+		// the canonicalization runs without panicking.
+		dp = &ReschedulePolicy{
+			Attempts:      intToPtr(0),
+			Interval:      timeToPtr(0),
+			Delay:         timeToPtr(0),
+			DelayFunction: stringToPtr(""),
+			MaxDelay:      timeToPtr(0),
+			Unlimited:     boolToPtr(false),
+		}
 	}
 	return dp
 }
@@ -364,10 +377,12 @@ func (m *MigrateStrategy) Copy() *MigrateStrategy {
 
 // VolumeRequest is a representation of a storage volume that a TaskGroup wishes to use.
 type VolumeRequest struct {
-	Name     string
-	Type     string
-	Source   string
-	ReadOnly bool `mapstructure:"read_only"`
+	Name         string
+	Type         string
+	Source       string
+	ReadOnly     bool             `hcl:"read_only"`
+	MountOptions *CSIMountOptions `hcl:"mount_options"`
+	ExtraKeysHCL []string         `hcl:",unusedKeys" json:"-"`
 }
 
 const (
@@ -396,22 +411,24 @@ func (vm *VolumeMount) Canonicalize() {
 
 // TaskGroup is the unit of scheduling.
 type TaskGroup struct {
-	Name             *string
-	Count            *int
-	Constraints      []*Constraint
-	Affinities       []*Affinity
-	Tasks            []*Task
-	Spreads          []*Spread
-	Volumes          map[string]*VolumeRequest
-	RestartPolicy    *RestartPolicy
-	ReschedulePolicy *ReschedulePolicy
-	EphemeralDisk    *EphemeralDisk
-	Update           *UpdateStrategy
-	Migrate          *MigrateStrategy
-	Networks         []*NetworkResource
-	Meta             map[string]string
-	Services         []*Service
-	ShutdownDelay    *time.Duration `mapstructure:"shutdown_delay"`
+	Name                      *string
+	Count                     *int
+	Constraints               []*Constraint
+	Affinities                []*Affinity
+	Tasks                     []*Task
+	Spreads                   []*Spread
+	Volumes                   map[string]*VolumeRequest
+	RestartPolicy             *RestartPolicy
+	ReschedulePolicy          *ReschedulePolicy
+	EphemeralDisk             *EphemeralDisk
+	Update                    *UpdateStrategy
+	Migrate                   *MigrateStrategy
+	Networks                  []*NetworkResource
+	Meta                      map[string]string
+	Services                  []*Service
+	ShutdownDelay             *time.Duration `mapstructure:"shutdown_delay"`
+	StopAfterClientDisconnect *time.Duration `mapstructure:"stop_after_client_disconnect"`
+	Scaling                   *ScalingPolicy
 }
 
 // NewTaskGroup creates a new TaskGroup.
@@ -427,11 +444,16 @@ func (g *TaskGroup) Canonicalize(job *Job) {
 	if g.Name == nil {
 		g.Name = stringToPtr("")
 	}
+
 	if g.Count == nil {
-		g.Count = intToPtr(1)
+		if g.Scaling != nil && g.Scaling.Min != nil {
+			g.Count = intToPtr(int(*g.Scaling.Min))
+		} else {
+			g.Count = intToPtr(1)
+		}
 	}
-	for _, t := range g.Tasks {
-		t.Canonicalize(g, job)
+	if g.Scaling != nil {
+		g.Scaling.Canonicalize(*g.Count)
 	}
 	if g.EphemeralDisk == nil {
 		g.EphemeralDisk = DefaultEphemeralDisk()
@@ -492,23 +514,9 @@ func (g *TaskGroup) Canonicalize(job *Job) {
 	var defaultRestartPolicy *RestartPolicy
 	switch *job.Type {
 	case "service", "system":
-		// These needs to be in sync with DefaultServiceJobRestartPolicy in
-		// in nomad/structs/structs.go
-		defaultRestartPolicy = &RestartPolicy{
-			Delay:    timeToPtr(15 * time.Second),
-			Attempts: intToPtr(2),
-			Interval: timeToPtr(30 * time.Minute),
-			Mode:     stringToPtr(RestartPolicyModeFail),
-		}
+		defaultRestartPolicy = defaultServiceJobRestartPolicy()
 	default:
-		// These needs to be in sync with DefaultBatchJobRestartPolicy in
-		// in nomad/structs/structs.go
-		defaultRestartPolicy = &RestartPolicy{
-			Delay:    timeToPtr(15 * time.Second),
-			Attempts: intToPtr(3),
-			Interval: timeToPtr(24 * time.Hour),
-			Mode:     stringToPtr(RestartPolicyModeFail),
-		}
+		defaultRestartPolicy = defaultBatchJobRestartPolicy()
 	}
 
 	if g.RestartPolicy != nil {
@@ -516,6 +524,10 @@ func (g *TaskGroup) Canonicalize(job *Job) {
 	}
 	g.RestartPolicy = defaultRestartPolicy
 
+	for _, t := range g.Tasks {
+		t.Canonicalize(g, job)
+	}
+
 	for _, spread := range g.Spreads {
 		spread.Canonicalize()
 	}
@@ -530,6 +542,28 @@ func (g *TaskGroup) Canonicalize(job *Job) {
 	}
 }
 
+// These needs to be in sync with DefaultServiceJobRestartPolicy in
+// in nomad/structs/structs.go
+func defaultServiceJobRestartPolicy() *RestartPolicy {
+	return &RestartPolicy{
+		Delay:    timeToPtr(15 * time.Second),
+		Attempts: intToPtr(2),
+		Interval: timeToPtr(30 * time.Minute),
+		Mode:     stringToPtr(RestartPolicyModeFail),
+	}
+}
+
+// These needs to be in sync with DefaultBatchJobRestartPolicy in
+// in nomad/structs/structs.go
+func defaultBatchJobRestartPolicy() *RestartPolicy {
+	return &RestartPolicy{
+		Delay:    timeToPtr(15 * time.Second),
+		Attempts: intToPtr(3),
+		Interval: timeToPtr(24 * time.Hour),
+		Mode:     stringToPtr(RestartPolicyModeFail),
+	}
+}
+
 // Constrain is used to add a constraint to a task group.
 func (g *TaskGroup) Constrain(c *Constraint) *TaskGroup {
 	g.Constraints = append(g.Constraints, c)
@@ -596,17 +630,34 @@ type DispatchPayloadConfig struct {
 	File string
 }
 
+const (
+	TaskLifecycleHookPrestart = "prestart"
+	TaskLifecycleHookPoststart = "poststart"
+)
+
+type TaskLifecycle struct {
+	Hook    string `mapstructure:"hook"`
+	Sidecar bool   `mapstructure:"sidecar"`
+}
+
+// Determine if lifecycle has user-input values
+func (l *TaskLifecycle) Empty() bool {
+	return l == nil || (l.Hook == "")
+}
+
 // Task is a single process in a task group.
 type Task struct {
 	Name            string
 	Driver          string
 	User            string
+	Lifecycle       *TaskLifecycle
 	Config          map[string]interface{}
 	Constraints     []*Constraint
 	Affinities      []*Affinity
 	Env             map[string]string
 	Services        []*Service
 	Resources       *Resources
+	RestartPolicy   *RestartPolicy
 	Meta            map[string]string
 	KillTimeout     *time.Duration `mapstructure:"kill_timeout"`
 	LogConfig       *LogConfig     `mapstructure:"logs"`
@@ -615,6 +666,7 @@ type Task struct {
 	Templates       []*Template
 	DispatchPayload *DispatchPayloadConfig
 	VolumeMounts    []*VolumeMount
+	CSIPluginConfig *TaskCSIPluginConfig `mapstructure:"csi_plugin" json:",omitempty"`
 	Leader          bool
 	ShutdownDelay   time.Duration `mapstructure:"shutdown_delay"`
 	KillSignal      string        `mapstructure:"kill_signal"`
@@ -626,6 +678,7 @@ func (t *Task) Canonicalize(tg *TaskGroup, job *Job) {
 		t.Resources = &Resources{}
 	}
 	t.Resources.Canonicalize()
+
 	if t.KillTimeout == nil {
 		t.KillTimeout = timeToPtr(5 * time.Second)
 	}
@@ -652,6 +705,20 @@ func (t *Task) Canonicalize(tg *TaskGroup, job *Job) {
 	for _, vm := range t.VolumeMounts {
 		vm.Canonicalize()
 	}
+	if t.Lifecycle.Empty() {
+		t.Lifecycle = nil
+	}
+	if t.CSIPluginConfig != nil {
+		t.CSIPluginConfig.Canonicalize()
+	}
+	if t.RestartPolicy == nil {
+		t.RestartPolicy = tg.RestartPolicy
+	} else {
+		tgrp := &RestartPolicy{}
+		*tgrp = *tg.RestartPolicy
+		tgrp.Merge(t.RestartPolicy)
+		t.RestartPolicy = tgrp
+	}
 }
 
 // TaskArtifact is used to download artifacts before running a task.
@@ -737,13 +804,16 @@ func (tmpl *Template) Canonicalize() {
 	if tmpl.Envvars == nil {
 		tmpl.Envvars = boolToPtr(false)
 	}
+
+	//COMPAT(0.12) VaultGrace is deprecated and unused as of Vault 0.5
 	if tmpl.VaultGrace == nil {
-		tmpl.VaultGrace = timeToPtr(15 * time.Second)
+		tmpl.VaultGrace = timeToPtr(0)
 	}
 }
 
 type Vault struct {
 	Policies     []string
+	Namespace    *string `mapstructure:"namespace"`
 	Env          *bool
 	ChangeMode   *string `mapstructure:"change_mode"`
 	ChangeSignal *string `mapstructure:"change_signal"`
@@ -753,6 +823,9 @@ func (v *Vault) Canonicalize() {
 	if v.Env == nil {
 		v.Env = boolToPtr(true)
 	}
+	if v.Namespace == nil {
+		v.Namespace = stringToPtr("")
+	}
 	if v.ChangeMode == nil {
 		v.ChangeMode = stringToPtr("restart")
 	}
@@ -853,6 +926,7 @@ type TaskEvent struct {
 	Time           int64
 	DisplayMessage string
 	Details        map[string]string
+	Message        string
 	// DEPRECATION NOTICE: The following fields are all deprecated. see TaskEvent struct in structs.go for details.
 	FailsTask        bool
 	RestartReason    string
@@ -861,7 +935,6 @@ type TaskEvent struct {
 	DriverMessage    string
 	ExitCode         int
 	Signal           int
-	Message          string
 	KillReason       string
 	KillTimeout      time.Duration
 	KillError        string
@@ -876,3 +949,48 @@ type TaskEvent struct {
 	TaskSignal       string
 	GenericSource    string
 }
+
+// CSIPluginType is an enum string that encapsulates the valid options for a
+// CSIPlugin stanza's Type. These modes will allow the plugin to be used in
+// different ways by the client.
+type CSIPluginType string
+
+const (
+	// CSIPluginTypeNode indicates that Nomad should only use the plugin for
+	// performing Node RPCs against the provided plugin.
+	CSIPluginTypeNode CSIPluginType = "node"
+
+	// CSIPluginTypeController indicates that Nomad should only use the plugin for
+	// performing Controller RPCs against the provided plugin.
+	CSIPluginTypeController CSIPluginType = "controller"
+
+	// CSIPluginTypeMonolith indicates that Nomad can use the provided plugin for
+	// both controller and node rpcs.
+	CSIPluginTypeMonolith CSIPluginType = "monolith"
+)
+
+// TaskCSIPluginConfig contains the data that is required to setup a task as a
+// CSI plugin. This will be used by the csi_plugin_supervisor_hook to configure
+// mounts for the plugin and initiate the connection to the plugin catalog.
+type TaskCSIPluginConfig struct {
+	// ID is the identifier of the plugin.
+	// Ideally this should be the FQDN of the plugin.
+	ID string `mapstructure:"id"`
+
+	// CSIPluginType instructs Nomad on how to handle processing a plugin
+	Type CSIPluginType `mapstructure:"type"`
+
+	// MountDir is the destination that nomad should mount in its CSI
+	// directory for the plugin. It will then expect a file called CSISocketName
+	// to be created by the plugin, and will provide references into
+	// "MountDir/CSIIntermediaryDirname/VolumeName/AllocID for mounts.
+	//
+	// Default is /csi.
+	MountDir string `mapstructure:"mount_dir"`
+}
+
+func (t *TaskCSIPluginConfig) Canonicalize() {
+	if t.MountDir == "" {
+		t.MountDir = "/csi"
+	}
+}
