@@ -13,17 +13,18 @@ import (
 	"testing"
 
 	consulapi "github.com/hashicorp/consul/api"
-	consultest "github.com/hashicorp/consul/testutil"
 	"github.com/hashicorp/nomad/client/allocdir"
 	"github.com/hashicorp/nomad/client/allocrunner/interfaces"
 	"github.com/hashicorp/nomad/client/taskenv"
 	"github.com/hashicorp/nomad/client/testutil"
 	agentconsul "github.com/hashicorp/nomad/command/agent/consul"
+	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/helper/args"
 	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/mock"
 	"github.com/hashicorp/nomad/nomad/structs"
+	"github.com/hashicorp/nomad/nomad/structs/config"
 	"github.com/stretchr/testify/require"
 	"golang.org/x/sys/unix"
 )
@@ -53,9 +54,9 @@ func TestEnvoyBootstrapHook_maybeLoadSIToken(t *testing.T) {
 
 	t.Run("file does not exist", func(t *testing.T) {
 		h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{logger: testlog.HCLogger(t)})
-		config, err := h.maybeLoadSIToken("task1", "/does/not/exist")
+		cfg, err := h.maybeLoadSIToken("task1", "/does/not/exist")
 		require.NoError(t, err) // absence of token is not an error
-		require.Equal(t, "", config)
+		require.Equal(t, "", cfg)
 	})
 
 	t.Run("load token from file", func(t *testing.T) {
@@ -64,9 +65,9 @@ func TestEnvoyBootstrapHook_maybeLoadSIToken(t *testing.T) {
 		defer cleanupDir(t, f)
 
 		h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{logger: testlog.HCLogger(t)})
-		config, err := h.maybeLoadSIToken("task1", f)
+		cfg, err := h.maybeLoadSIToken("task1", f)
 		require.NoError(t, err)
-		require.Equal(t, token, config)
+		require.Equal(t, token, cfg)
 	})
 
 	t.Run("file is unreadable", func(t *testing.T) {
@@ -75,13 +76,37 @@ func TestEnvoyBootstrapHook_maybeLoadSIToken(t *testing.T) {
 		defer cleanupDir(t, f)
 
 		h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{logger: testlog.HCLogger(t)})
-		config, err := h.maybeLoadSIToken("task1", f)
+		cfg, err := h.maybeLoadSIToken("task1", f)
 		require.Error(t, err)
 		require.False(t, os.IsNotExist(err))
-		require.Equal(t, "", config)
+		require.Equal(t, "", cfg)
 	})
 }
 
+func TestEnvoyBootstrapHook_decodeTriState(t *testing.T) {
+	t.Parallel()
+
+	require.Equal(t, "", decodeTriState(nil))
+	require.Equal(t, "true", decodeTriState(helper.BoolToPtr(true)))
+	require.Equal(t, "false", decodeTriState(helper.BoolToPtr(false)))
+}
+
+var (
+	consulPlainConfig = consulTransportConfig{
+		HTTPAddr: "2.2.2.2",
+	}
+
+	consulTLSConfig = consulTransportConfig{
+		HTTPAddr:  "2.2.2.2",            // arg
+		Auth:      "user:password",      // env
+		SSL:       "true",               // env
+		VerifySSL: "true",               // env
+		CAFile:    "/etc/tls/ca-file",   // arg
+		CertFile:  "/etc/tls/cert-file", // arg
+		KeyFile:   "/etc/tls/key-file",  // arg
+	}
+)
+
 func TestEnvoyBootstrapHook_envoyBootstrapArgs(t *testing.T) {
 	t.Parallel()
 
@@ -89,17 +114,17 @@ func TestEnvoyBootstrapHook_envoyBootstrapArgs(t *testing.T) {
 		ebArgs := envoyBootstrapArgs{
 			sidecarFor:     "s1",
 			grpcAddr:       "1.1.1.1",
-			consulHTTPAddr: "2.2.2.2",
+			consulConfig:   consulPlainConfig,
 			envoyAdminBind: "localhost:3333",
 		}
-		args := ebArgs.args()
+		result := ebArgs.args()
 		require.Equal(t, []string{"connect", "envoy",
 			"-grpc-addr", "1.1.1.1",
 			"-http-addr", "2.2.2.2",
 			"-admin-bind", "localhost:3333",
 			"-bootstrap",
 			"-sidecar-for", "s1",
-		}, args)
+		}, result)
 	})
 
 	t.Run("including SI token", func(t *testing.T) {
@@ -107,11 +132,11 @@ func TestEnvoyBootstrapHook_envoyBootstrapArgs(t *testing.T) {
 		ebArgs := envoyBootstrapArgs{
 			sidecarFor:     "s1",
 			grpcAddr:       "1.1.1.1",
-			consulHTTPAddr: "2.2.2.2",
+			consulConfig:   consulPlainConfig,
 			envoyAdminBind: "localhost:3333",
 			siToken:        token,
 		}
-		args := ebArgs.args()
+		result := ebArgs.args()
 		require.Equal(t, []string{"connect", "envoy",
 			"-grpc-addr", "1.1.1.1",
 			"-http-addr", "2.2.2.2",
@@ -119,12 +144,96 @@ func TestEnvoyBootstrapHook_envoyBootstrapArgs(t *testing.T) {
 			"-bootstrap",
 			"-sidecar-for", "s1",
 			"-token", token,
-		}, args)
+		}, result)
+	})
+
+	t.Run("including certificates", func(t *testing.T) {
+		ebArgs := envoyBootstrapArgs{
+			sidecarFor:     "s1",
+			grpcAddr:       "1.1.1.1",
+			consulConfig:   consulTLSConfig,
+			envoyAdminBind: "localhost:3333",
+		}
+		result := ebArgs.args()
+		require.Equal(t, []string{"connect", "envoy",
+			"-grpc-addr", "1.1.1.1",
+			"-http-addr", "2.2.2.2",
+			"-admin-bind", "localhost:3333",
+			"-bootstrap",
+			"-sidecar-for", "s1",
+			"-ca-file", "/etc/tls/ca-file",
+			"-client-cert", "/etc/tls/cert-file",
+			"-client-key", "/etc/tls/key-file",
+		}, result)
+	})
+
+	t.Run("ingress gateway", func(t *testing.T) {
+		ebArgs := envoyBootstrapArgs{
+			consulConfig:   consulPlainConfig,
+			grpcAddr:       "1.1.1.1",
+			envoyAdminBind: "localhost:3333",
+			gateway:        "my-ingress-gateway",
+		}
+		result := ebArgs.args()
+		require.Equal(t, []string{"connect", "envoy",
+			"-grpc-addr", "1.1.1.1",
+			"-http-addr", "2.2.2.2",
+			"-admin-bind", "localhost:3333",
+			"-bootstrap",
+			"-gateway", "my-ingress-gateway",
+		}, result)
 	})
 }
 
-// dig through envoy config to look for consul token
+func TestEnvoyBootstrapHook_envoyBootstrapEnv(t *testing.T) {
+	t.Parallel()
+
+	environment := []string{"foo=bar", "baz=1"}
+
+	t.Run("plain consul config", func(t *testing.T) {
+		require.Equal(t, []string{
+			"foo=bar", "baz=1",
+		}, envoyBootstrapArgs{
+			sidecarFor:     "s1",
+			grpcAddr:       "1.1.1.1",
+			consulConfig:   consulPlainConfig,
+			envoyAdminBind: "localhost:3333",
+		}.env(environment))
+	})
+
+	t.Run("tls consul config", func(t *testing.T) {
+		require.Equal(t, []string{
+			"foo=bar", "baz=1",
+			"CONSUL_HTTP_AUTH=user:password",
+			"CONSUL_HTTP_SSL=true",
+			"CONSUL_HTTP_SSL_VERIFY=true",
+		}, envoyBootstrapArgs{
+			sidecarFor:     "s1",
+			grpcAddr:       "1.1.1.1",
+			consulConfig:   consulTLSConfig,
+			envoyAdminBind: "localhost:3333",
+		}.env(environment))
+	})
+}
+
+// envoyConfig is used to unmarshal an envoy bootstrap configuration file, so that
+// we can inspect the contents in tests.
 type envoyConfig struct {
+	Admin struct {
+		Address struct {
+			SocketAddress struct {
+				Address string `json:"address"`
+				Port    int    `json:"port_value"`
+			} `json:"socket_address"`
+		} `json:"address"`
+	} `json:"admin"`
+	Node struct {
+		Cluster  string `json:"cluster"`
+		Metadata struct {
+			Namespace string `json:"namespace"`
+			Version   string `json:"envoy_version"`
+		}
+	}
 	DynamicResources struct {
 		ADSConfig struct {
 			GRPCServices struct {
@@ -143,19 +252,10 @@ func TestEnvoyBootstrapHook_with_SI_token(t *testing.T) {
 	t.Parallel()
 	testutil.RequireConsul(t)
 
-	testconsul, err := consultest.NewTestServerConfig(func(c *consultest.TestServerConfig) {
-		// If -v wasn't specified squelch consul logging
-		if !testing.Verbose() {
-			c.Stdout = ioutil.Discard
-			c.Stderr = ioutil.Discard
-		}
-	})
-	if err != nil {
-		t.Fatalf("error starting test consul server: %v", err)
-	}
-	defer testconsul.Stop()
+	testConsul := getTestConsul(t)
+	defer testConsul.Stop()
 
-	alloc := mock.Alloc()
+	alloc := mock.ConnectAlloc()
 	alloc.AllocatedResources.Shared.Networks = []*structs.NetworkResource{
 		{
 			Mode: "bridge",
@@ -192,7 +292,7 @@ func TestEnvoyBootstrapHook_with_SI_token(t *testing.T) {
 
 	// Register Group Services
 	consulConfig := consulapi.DefaultConfig()
-	consulConfig.Address = testconsul.HTTPAddr
+	consulConfig.Address = testConsul.HTTPAddr
 	consulAPIClient, err := consulapi.NewClient(consulConfig)
 	require.NoError(t, err)
 
@@ -202,14 +302,13 @@ func TestEnvoyBootstrapHook_with_SI_token(t *testing.T) {
 	require.NoError(t, consulClient.RegisterWorkload(agentconsul.BuildAllocServices(mock.Node(), alloc, agentconsul.NoopRestarter())))
 
 	// Run Connect bootstrap Hook
-	h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{
-		alloc:          alloc,
-		consulHTTPAddr: testconsul.HTTPAddr,
-		logger:         logger,
-	})
+	h := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(alloc, &config.ConsulConfig{
+		Addr: consulConfig.Address,
+	}, logger))
 	req := &interfaces.TaskPrestartRequest{
 		Task:    sidecarTask,
 		TaskDir: allocDir.NewTaskDir(sidecarTask.Name),
+		TaskEnv: taskenv.NewEmptyTaskEnv(),
 	}
 	require.NoError(t, req.TaskDir.Build(false, nil))
 
@@ -246,26 +345,17 @@ func TestEnvoyBootstrapHook_with_SI_token(t *testing.T) {
 	require.Equal(t, token, value)
 }
 
-// TestTaskRunner_EnvoyBootstrapHook_Prestart asserts the EnvoyBootstrapHook
+// TestTaskRunner_EnvoyBootstrapHook_sidecar_ok asserts the EnvoyBootstrapHook
 // creates Envoy's bootstrap.json configuration based on Connect proxy sidecars
 // registered for the task.
-func TestTaskRunner_EnvoyBootstrapHook_Ok(t *testing.T) {
+func TestTaskRunner_EnvoyBootstrapHook_sidecar_ok(t *testing.T) {
 	t.Parallel()
 	testutil.RequireConsul(t)
 
-	testconsul, err := consultest.NewTestServerConfig(func(c *consultest.TestServerConfig) {
-		// If -v wasn't specified squelch consul logging
-		if !testing.Verbose() {
-			c.Stdout = ioutil.Discard
-			c.Stderr = ioutil.Discard
-		}
-	})
-	if err != nil {
-		t.Fatalf("error starting test consul server: %v", err)
-	}
-	defer testconsul.Stop()
+	testConsul := getTestConsul(t)
+	defer testConsul.Stop()
 
-	alloc := mock.Alloc()
+	alloc := mock.ConnectAlloc()
 	alloc.AllocatedResources.Shared.Networks = []*structs.NetworkResource{
 		{
 			Mode: "bridge",
@@ -291,7 +381,7 @@ func TestTaskRunner_EnvoyBootstrapHook_Ok(t *testing.T) {
 	}
 	sidecarTask := &structs.Task{
 		Name: "sidecar",
-		Kind: "connect-proxy:foo",
+		Kind: structs.NewTaskKind(structs.ConnectProxyPrefix, "foo"),
 	}
 	tg.Tasks = append(tg.Tasks, sidecarTask)
 
@@ -302,7 +392,7 @@ func TestTaskRunner_EnvoyBootstrapHook_Ok(t *testing.T) {
 
 	// Register Group Services
 	consulConfig := consulapi.DefaultConfig()
-	consulConfig.Address = testconsul.HTTPAddr
+	consulConfig.Address = testConsul.HTTPAddr
 	consulAPIClient, err := consulapi.NewClient(consulConfig)
 	require.NoError(t, err)
 
@@ -312,14 +402,13 @@ func TestTaskRunner_EnvoyBootstrapHook_Ok(t *testing.T) {
 	require.NoError(t, consulClient.RegisterWorkload(agentconsul.BuildAllocServices(mock.Node(), alloc, agentconsul.NoopRestarter())))
 
 	// Run Connect bootstrap Hook
-	h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{
-		alloc:          alloc,
-		consulHTTPAddr: testconsul.HTTPAddr,
-		logger:         logger,
-	})
+	h := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(alloc, &config.ConsulConfig{
+		Addr: consulConfig.Address,
+	}, logger))
 	req := &interfaces.TaskPrestartRequest{
 		Task:    sidecarTask,
 		TaskDir: allocDir.NewTaskDir(sidecarTask.Name),
+		TaskEnv: taskenv.NewEmptyTaskEnv(),
 	}
 	require.NoError(t, req.TaskDir.Build(false, nil))
 
@@ -353,8 +442,85 @@ func TestTaskRunner_EnvoyBootstrapHook_Ok(t *testing.T) {
 	require.Equal(t, "", value)
 }
 
+func TestTaskRunner_EnvoyBootstrapHook_gateway_ok(t *testing.T) {
+	t.Parallel()
+	logger := testlog.HCLogger(t)
+
+	testConsul := getTestConsul(t)
+	defer testConsul.Stop()
+
+	// Setup an Allocation
+	alloc := mock.ConnectIngressGatewayAlloc("bridge")
+	allocDir, cleanupDir := allocdir.TestAllocDir(t, logger, "EnvoyBootstrapIngressGateway")
+	defer cleanupDir()
+
+	// Get a Consul client
+	consulConfig := consulapi.DefaultConfig()
+	consulConfig.Address = testConsul.HTTPAddr
+	consulAPIClient, err := consulapi.NewClient(consulConfig)
+	require.NoError(t, err)
+
+	// Register Group Services
+	serviceClient := agentconsul.NewServiceClient(consulAPIClient.Agent(), logger, true)
+	go serviceClient.Run()
+	defer serviceClient.Shutdown()
+	require.NoError(t, serviceClient.RegisterWorkload(agentconsul.BuildAllocServices(mock.Node(), alloc, agentconsul.NoopRestarter())))
+
+	// Register Configuration Entry
+	ceClient := consulAPIClient.ConfigEntries()
+	set, _, err := ceClient.Set(&consulapi.IngressGatewayConfigEntry{
+		Kind: consulapi.IngressGateway,
+		Name: "gateway-service", // matches job
+		Listeners: []consulapi.IngressListener{{
+			Port:     2000,
+			Protocol: "tcp",
+			Services: []consulapi.IngressService{{
+				Name: "service1",
+			}},
+		}},
+	}, nil)
+	require.NoError(t, err)
+	require.True(t, set)
+
+	// Run Connect bootstrap hook
+	h := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(alloc, &config.ConsulConfig{
+		Addr: consulConfig.Address,
+	}, logger))
+
+	req := &interfaces.TaskPrestartRequest{
+		Task:    alloc.Job.TaskGroups[0].Tasks[0],
+		TaskDir: allocDir.NewTaskDir(alloc.Job.TaskGroups[0].Tasks[0].Name),
+		TaskEnv: taskenv.NewEmptyTaskEnv(),
+	}
+	require.NoError(t, req.TaskDir.Build(false, nil))
+
+	var resp interfaces.TaskPrestartResponse
+
+	// Run the hook
+	require.NoError(t, h.Prestart(context.Background(), req, &resp))
+
+	// Assert the hook is done
+	require.True(t, resp.Done)
+	require.NotNil(t, resp.Env)
+
+	// Read the Envoy Config file
+	env := map[string]string{
+		taskenv.SecretsDir: req.TaskDir.SecretsDir,
+	}
+	f, err := os.Open(args.ReplaceEnv(structs.EnvoyBootstrapPath, env))
+	require.NoError(t, err)
+	defer f.Close()
+
+	var out envoyConfig
+	require.NoError(t, json.NewDecoder(f).Decode(&out))
+
+	// the only interesting thing on bootstrap is the presence of the cluster,
+	// everything is configured at runtime through xDS
+	require.Equal(t, "my-ingress-service", out.Node.Cluster)
+}
+
 // TestTaskRunner_EnvoyBootstrapHook_Noop asserts that the Envoy bootstrap hook
-// is a noop for non-Connect proxy sidecar tasks.
+// is a noop for non-Connect proxy sidecar / gateway tasks.
 func TestTaskRunner_EnvoyBootstrapHook_Noop(t *testing.T) {
 	t.Parallel()
 	logger := testlog.HCLogger(t)
@@ -367,11 +533,9 @@ func TestTaskRunner_EnvoyBootstrapHook_Noop(t *testing.T) {
 
 	// Run Envoy bootstrap Hook. Use invalid Consul address as it should
 	// not get hit.
-	h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{
-		alloc:          alloc,
-		consulHTTPAddr: "http://127.0.0.2:1",
-		logger:         logger,
-	})
+	h := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(alloc, &config.ConsulConfig{
+		Addr: "http://127.0.0.2:1",
+	}, logger))
 	req := &interfaces.TaskPrestartRequest{
 		Task:    task,
 		TaskDir: allocDir.NewTaskDir(task.Name),
@@ -399,19 +563,10 @@ func TestTaskRunner_EnvoyBootstrapHook_RecoverableError(t *testing.T) {
 	t.Parallel()
 	testutil.RequireConsul(t)
 
-	testconsul, err := consultest.NewTestServerConfig(func(c *consultest.TestServerConfig) {
-		// If -v wasn't specified squelch consul logging
-		if !testing.Verbose() {
-			c.Stdout = ioutil.Discard
-			c.Stderr = ioutil.Discard
-		}
-	})
-	if err != nil {
-		t.Fatalf("error starting test consul server: %v", err)
-	}
-	defer testconsul.Stop()
+	testConsul := getTestConsul(t)
+	defer testConsul.Stop()
 
-	alloc := mock.Alloc()
+	alloc := mock.ConnectAlloc()
 	alloc.AllocatedResources.Shared.Networks = []*structs.NetworkResource{
 		{
 			Mode: "bridge",
@@ -451,21 +606,20 @@ func TestTaskRunner_EnvoyBootstrapHook_RecoverableError(t *testing.T) {
 	// not running.
 
 	// Run Connect bootstrap Hook
-	h := newEnvoyBootstrapHook(&envoyBootstrapHookConfig{
-		alloc:          alloc,
-		consulHTTPAddr: testconsul.HTTPAddr,
-		logger:         logger,
-	})
+	h := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(alloc, &config.ConsulConfig{
+		Addr: testConsul.HTTPAddr,
+	}, logger))
 	req := &interfaces.TaskPrestartRequest{
 		Task:    sidecarTask,
 		TaskDir: allocDir.NewTaskDir(sidecarTask.Name),
+		TaskEnv: taskenv.NewEmptyTaskEnv(),
 	}
 	require.NoError(t, req.TaskDir.Build(false, nil))
 
 	resp := &interfaces.TaskPrestartResponse{}
 
 	// Run the hook
-	err = h.Prestart(context.Background(), req, resp)
+	err := h.Prestart(context.Background(), req, resp)
 	require.EqualError(t, err, "error creating bootstrap configuration for Connect proxy sidecar: exit status 1")
 	require.True(t, structs.IsRecoverable(err))
 
@@ -477,3 +631,64 @@ func TestTaskRunner_EnvoyBootstrapHook_RecoverableError(t *testing.T) {
 	require.Error(t, err)
 	require.True(t, os.IsNotExist(err))
 }
+
+func TestTaskRunner_EnvoyBootstrapHook_extractNameAndKind(t *testing.T) {
+	t.Run("connect sidecar", func(t *testing.T) {
+		kind, name, err := (*envoyBootstrapHook)(nil).extractNameAndKind(
+			structs.NewTaskKind(structs.ConnectProxyPrefix, "foo"),
+		)
+		require.Nil(t, err)
+		require.Equal(t, "connect-proxy", kind)
+		require.Equal(t, "foo", name)
+	})
+
+	t.Run("connect gateway", func(t *testing.T) {
+		kind, name, err := (*envoyBootstrapHook)(nil).extractNameAndKind(
+			structs.NewTaskKind(structs.ConnectIngressPrefix, "foo"),
+		)
+		require.Nil(t, err)
+		require.Equal(t, "connect-ingress", kind)
+		require.Equal(t, "foo", name)
+	})
+
+	t.Run("connect native", func(t *testing.T) {
+		_, _, err := (*envoyBootstrapHook)(nil).extractNameAndKind(
+			structs.NewTaskKind(structs.ConnectNativePrefix, "foo"),
+		)
+		require.EqualError(t, err, "envoy must be used as connect sidecar or gateway")
+	})
+
+	t.Run("normal task", func(t *testing.T) {
+		_, _, err := (*envoyBootstrapHook)(nil).extractNameAndKind(
+			structs.TaskKind(""),
+		)
+		require.EqualError(t, err, "envoy must be used as connect sidecar or gateway")
+	})
+}
+
+func TestTaskRunner_EnvoyBootstrapHook_grpcAddress(t *testing.T) {
+	bridgeH := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(
+		mock.ConnectIngressGatewayAlloc("bridge"),
+		new(config.ConsulConfig),
+		testlog.HCLogger(t),
+	))
+
+	hostH := newEnvoyBootstrapHook(newEnvoyBootstrapHookConfig(
+		mock.ConnectIngressGatewayAlloc("host"),
+		new(config.ConsulConfig),
+		testlog.HCLogger(t),
+	))
+
+	t.Run("environment", func(t *testing.T) {
+		env := map[string]string{
+			grpcConsulVariable: "1.2.3.4:9000",
+		}
+		require.Equal(t, "1.2.3.4:9000", bridgeH.grpcAddress(env))
+		require.Equal(t, "1.2.3.4:9000", hostH.grpcAddress(env))
+	})
+
+	t.Run("defaults", func(t *testing.T) {
+		require.Equal(t, "unix://alloc/tmp/consul_grpc.sock", bridgeH.grpcAddress(nil))
+		require.Equal(t, "127.0.0.1:8502", hostH.grpcAddress(nil))
+	})
+}
