@@ -11,7 +11,7 @@ import (
 	metrics "github.com/armon/go-metrics"
 	log "github.com/hashicorp/go-hclog"
 	multierror "github.com/hashicorp/go-multierror"
-	"github.com/hashicorp/hcl2/hcldec"
+	"github.com/hashicorp/hcl/v2/hcldec"
 	"github.com/hashicorp/nomad/client/allocdir"
 	"github.com/hashicorp/nomad/client/allocrunner/interfaces"
 	"github.com/hashicorp/nomad/client/allocrunner/taskrunner/restarts"
@@ -19,7 +19,9 @@ import (
 	"github.com/hashicorp/nomad/client/config"
 	"github.com/hashicorp/nomad/client/consul"
 	"github.com/hashicorp/nomad/client/devicemanager"
+	"github.com/hashicorp/nomad/client/dynamicplugins"
 	cinterfaces "github.com/hashicorp/nomad/client/interfaces"
+	"github.com/hashicorp/nomad/client/pluginmanager/csimanager"
 	"github.com/hashicorp/nomad/client/pluginmanager/drivermanager"
 	cstate "github.com/hashicorp/nomad/client/state"
 	cstructs "github.com/hashicorp/nomad/client/structs"
@@ -186,6 +188,9 @@ type TaskRunner struct {
 	// deviceStatsReporter is used to lookup resource usage for alloc devices
 	deviceStatsReporter cinterfaces.DeviceStatsReporter
 
+	// csiManager is used to manage the mounting of CSI volumes into tasks
+	csiManager csimanager.Manager
+
 	// devicemanager is used to mount devices as well as lookup device
 	// statistics
 	devicemanager devicemanager.Manager
@@ -194,6 +199,9 @@ type TaskRunner struct {
 	// handlers
 	driverManager drivermanager.Manager
 
+	// dynamicRegistry is where dynamic plugins should be registered.
+	dynamicRegistry dynamicplugins.Registry
+
 	// maxEvents is the capacity of the TaskEvents on the TaskState.
 	// Defaults to defaultMaxEvents but overrideable for testing.
 	maxEvents int
@@ -202,6 +210,9 @@ type TaskRunner struct {
 	// GetClientAllocs has been called in case of a failed restore.
 	serversContactedCh <-chan struct{}
 
+	// startConditionMetCtx is done when TR should start the task
+	startConditionMetCtx <-chan struct{}
+
 	// waitOnServers defaults to false but will be set true if a restore
 	// fails and the Run method should wait until serversContactedCh is
 	// closed.
@@ -209,6 +220,8 @@ type TaskRunner struct {
 
 	networkIsolationLock sync.Mutex
 	networkIsolationSpec *drivers.NetworkIsolationSpec
+
+	allocHookResources *cstructs.AllocHookResources
 }
 
 type Config struct {
@@ -224,6 +237,9 @@ type Config struct {
 	// ConsulSI is the client to use for managing Consul SI tokens
 	ConsulSI consul.ServiceIdentityAPI
 
+	// DynamicRegistry is where dynamic plugins should be registered.
+	DynamicRegistry dynamicplugins.Registry
+
 	// Vault is the client to use to derive and renew Vault tokens
 	Vault vaultclient.VaultClient
 
@@ -236,6 +252,9 @@ type Config struct {
 	// deviceStatsReporter is used to lookup resource usage for alloc devices
 	DeviceStatsReporter cinterfaces.DeviceStatsReporter
 
+	// CSIManager is used to manage the mounting of CSI volumes into tasks
+	CSIManager csimanager.Manager
+
 	// DeviceManager is used to mount devices as well as lookup device
 	// statistics
 	DeviceManager devicemanager.Manager
@@ -247,6 +266,9 @@ type Config struct {
 	// ServersContactedCh is closed when the first GetClientAllocs call to
 	// servers succeeds and allocs are synced.
 	ServersContactedCh chan struct{}
+
+	// startConditionMetCtx is done when TR should start the task
+	StartConditionMetCtx <-chan struct{}
 }
 
 func NewTaskRunner(config *Config) (*TaskRunner, error) {
@@ -271,32 +293,35 @@ func NewTaskRunner(config *Config) (*TaskRunner, error) {
 	}
 
 	tr := &TaskRunner{
-		alloc:               config.Alloc,
-		allocID:             config.Alloc.ID,
-		clientConfig:        config.ClientConfig,
-		task:                config.Task,
-		taskDir:             config.TaskDir,
-		taskName:            config.Task.Name,
-		taskLeader:          config.Task.Leader,
-		envBuilder:          envBuilder,
-		consulClient:        config.Consul,
-		siClient:            config.ConsulSI,
-		vaultClient:         config.Vault,
-		state:               tstate,
-		localState:          state.NewLocalState(),
-		stateDB:             config.StateDB,
-		stateUpdater:        config.StateUpdater,
-		deviceStatsReporter: config.DeviceStatsReporter,
-		killCtx:             killCtx,
-		killCtxCancel:       killCancel,
-		shutdownCtx:         trCtx,
-		shutdownCtxCancel:   trCancel,
-		triggerUpdateCh:     make(chan struct{}, triggerUpdateChCap),
-		waitCh:              make(chan struct{}),
-		devicemanager:       config.DeviceManager,
-		driverManager:       config.DriverManager,
-		maxEvents:           defaultMaxEvents,
-		serversContactedCh:  config.ServersContactedCh,
+		alloc:                config.Alloc,
+		allocID:              config.Alloc.ID,
+		clientConfig:         config.ClientConfig,
+		task:                 config.Task,
+		taskDir:              config.TaskDir,
+		taskName:             config.Task.Name,
+		taskLeader:           config.Task.Leader,
+		envBuilder:           envBuilder,
+		dynamicRegistry:      config.DynamicRegistry,
+		consulClient:         config.Consul,
+		siClient:             config.ConsulSI,
+		vaultClient:          config.Vault,
+		state:                tstate,
+		localState:           state.NewLocalState(),
+		stateDB:              config.StateDB,
+		stateUpdater:         config.StateUpdater,
+		deviceStatsReporter:  config.DeviceStatsReporter,
+		killCtx:              killCtx,
+		killCtxCancel:        killCancel,
+		shutdownCtx:          trCtx,
+		shutdownCtxCancel:    trCancel,
+		triggerUpdateCh:      make(chan struct{}, triggerUpdateChCap),
+		waitCh:               make(chan struct{}),
+		csiManager:           config.CSIManager,
+		devicemanager:        config.DeviceManager,
+		driverManager:        config.DriverManager,
+		maxEvents:            defaultMaxEvents,
+		serversContactedCh:   config.ServersContactedCh,
+		startConditionMetCtx: config.StartConditionMetCtx,
 	}
 
 	// Create the logger based on the allocation ID
@@ -315,12 +340,16 @@ func NewTaskRunner(config *Config) (*TaskRunner, error) {
 	tr.taskResources = tres
 
 	// Build the restart tracker.
-	tg := tr.alloc.Job.LookupTaskGroup(tr.alloc.TaskGroup)
-	if tg == nil {
-		tr.logger.Error("alloc missing task group")
-		return nil, fmt.Errorf("alloc missing task group")
+	rp := config.Task.RestartPolicy
+	if rp == nil {
+		tg := tr.alloc.Job.LookupTaskGroup(tr.alloc.TaskGroup)
+		if tg == nil {
+			tr.logger.Error("alloc missing task group")
+			return nil, fmt.Errorf("alloc missing task group")
+		}
+		rp = tg.RestartPolicy
 	}
-	tr.restartTracker = restarts.NewRestartTracker(tg.RestartPolicy, tr.alloc.Job.Type)
+	tr.restartTracker = restarts.NewRestartTracker(rp, tr.alloc.Job.Type, config.Task.Lifecycle)
 
 	// Get the driver
 	if err := tr.initDriver(); err != nil {
@@ -454,6 +483,14 @@ func (tr *TaskRunner) Run() {
 		}
 	}
 
+	select {
+	case <-tr.startConditionMetCtx:
+		// yay proceed
+	case <-tr.killCtx.Done():
+	case <-tr.shutdownCtx.Done():
+		return
+	}
+
 MAIN:
 	for !tr.Alloc().TerminalStatus() {
 		select {
@@ -807,6 +844,14 @@ func (tr *TaskRunner) handleKill() *drivers.ExitResult {
 	// Run the pre killing hooks
 	tr.preKill()
 
+	// Wait for task ShutdownDelay after running prekill hooks
+	// This allows for things like service de-registration to run
+	// before waiting to kill task
+	if delay := tr.Task().ShutdownDelay; delay != 0 {
+		tr.logger.Debug("waiting before killing task", "shutdown_delay", delay)
+		time.Sleep(delay)
+	}
+
 	// Tell the restart tracker that the task has been killed so it doesn't
 	// attempt to restart it.
 	tr.restartTracker.SetKilled()
@@ -891,10 +936,23 @@ func (tr *TaskRunner) buildTaskConfig() *drivers.TaskConfig {
 	alloc := tr.Alloc()
 	invocationid := uuid.Generate()[:8]
 	taskResources := tr.taskResources
+	ports := tr.Alloc().AllocatedResources.Shared.Ports
 	env := tr.envBuilder.Build()
 	tr.networkIsolationLock.Lock()
 	defer tr.networkIsolationLock.Unlock()
 
+	var dns *drivers.DNSConfig
+	if alloc.AllocatedResources != nil && len(alloc.AllocatedResources.Shared.Networks) > 0 {
+		allocDNS := alloc.AllocatedResources.Shared.Networks[0].DNS
+		if allocDNS != nil {
+			dns = &drivers.DNSConfig{
+				Servers:  allocDNS.Servers,
+				Searches: allocDNS.Searches,
+				Options:  allocDNS.Options,
+			}
+		}
+	}
+
 	return &drivers.TaskConfig{
 		ID:            fmt.Sprintf("%s/%s/%s", alloc.ID, task.Name, invocationid),
 		Name:          task.Name,
@@ -907,6 +965,7 @@ func (tr *TaskRunner) buildTaskConfig() *drivers.TaskConfig {
 				CPUShares:        taskResources.Cpu.CpuShares,
 				PercentTicks:     float64(taskResources.Cpu.CpuShares) / float64(tr.clientConfig.Node.NodeResources.Cpu.CpuShares),
 			},
+			Ports: &ports,
 		},
 		Devices:          tr.hookResources.getDevices(),
 		Mounts:           tr.hookResources.getMounts(),
@@ -918,6 +977,7 @@ func (tr *TaskRunner) buildTaskConfig() *drivers.TaskConfig {
 		StderrPath:       tr.logmonHookConfig.stderrFifo,
 		AllocID:          tr.allocID,
 		NetworkIsolation: tr.networkIsolationSpec,
+		DNS:              dns,
 	}
 }
 
@@ -1377,3 +1437,7 @@ func (tr *TaskRunner) TaskExecHandler() drivermanager.TaskExecHandler {
 func (tr *TaskRunner) DriverCapabilities() (*drivers.Capabilities, error) {
 	return tr.driver.Capabilities()
 }
+
+func (tr *TaskRunner) SetAllocHookResources(res *cstructs.AllocHookResources) {
+	tr.allocHookResources = res
+}
