@@ -1,25 +1,24 @@
 package fingerprint
 
 import (
-	"log"
 	"runtime"
 
-	cstructs "github.com/hashicorp/nomad/client/structs"
+	log "github.com/hashicorp/go-hclog"
 )
 
 // ArchFingerprint is used to fingerprint the architecture
 type ArchFingerprint struct {
 	StaticFingerprinter
-	logger *log.Logger
+	logger log.Logger
 }
 
 // NewArchFingerprint is used to create an OS fingerprint
-func NewArchFingerprint(logger *log.Logger) Fingerprint {
-	f := &ArchFingerprint{logger: logger}
+func NewArchFingerprint(logger log.Logger) Fingerprint {
+	f := &ArchFingerprint{logger: logger.Named("arch")}
 	return f
 }
 
-func (f *ArchFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp *cstructs.FingerprintResponse) error {
+func (f *ArchFingerprint) Fingerprint(req *FingerprintRequest, resp *FingerprintResponse) error {
 	resp.AddAttribute("cpu.arch", runtime.GOARCH)
 	resp.Detected = true
 	return nil
