@@ -31,6 +31,7 @@ var (
 	hostFingerprinters = map[string]Factory{
 		"arch":    NewArchFingerprint,
 		"consul":  NewConsulFingerprint,
+		"cni":     NewCNIFingerprint,
 		"cpu":     NewCPUFingerprint,
 		"host":    NewHostFingerprint,
 		"memory":  NewMemoryFingerprint,
@@ -115,6 +116,13 @@ type Fingerprint interface {
 	Periodic() (bool, time.Duration)
 }
 
+// ReloadableFingerprint can be implemented if the fingerprinter needs to be run during client reload.
+// If implemented, the client will call Reload during client reload then immediately Fingerprint
+type ReloadableFingerprint interface {
+	Fingerprint
+	Reload()
+}
+
 // StaticFingerprinter can be embedded in a struct that has a Fingerprint method
 // to make it non-periodic.
 type StaticFingerprinter struct{}
