@@ -1,26 +1,25 @@
 package fingerprint
 
 import (
-	"log"
 	"strings"
 
 	"github.com/hashicorp/consul-template/signals"
-	cstructs "github.com/hashicorp/nomad/client/structs"
+	log "github.com/hashicorp/go-hclog"
 )
 
 // SignalFingerprint is used to fingerprint the available signals
 type SignalFingerprint struct {
 	StaticFingerprinter
-	logger *log.Logger
+	logger log.Logger
 }
 
 // NewSignalFingerprint is used to create a Signal fingerprint
-func NewSignalFingerprint(logger *log.Logger) Fingerprint {
-	f := &SignalFingerprint{logger: logger}
+func NewSignalFingerprint(logger log.Logger) Fingerprint {
+	f := &SignalFingerprint{logger: logger.Named("signals")}
 	return f
 }
 
-func (f *SignalFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp *cstructs.FingerprintResponse) error {
+func (f *SignalFingerprint) Fingerprint(req *FingerprintRequest, resp *FingerprintResponse) error {
 	// Build the list of available signals
 	sigs := make([]string, 0, len(signals.SignalLookup))
 	for signal := range signals.SignalLookup {
