@@ -3,11 +3,12 @@ package fingerprint
 import (
 	"testing"
 
+	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/nomad/structs"
 )
 
 func TestSignalFingerprint(t *testing.T) {
-	fp := NewSignalFingerprint(testLogger())
+	fp := NewSignalFingerprint(testlog.HCLogger(t))
 	node := &structs.Node{
 		Attributes: make(map[string]string),
 	}
