@@ -2,11 +2,10 @@ package fingerprint
 
 import (
 	"fmt"
-	"log"
 	"os"
 	"strconv"
 
-	cstructs "github.com/hashicorp/nomad/client/structs"
+	log "github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/nomad/nomad/structs"
 )
 
@@ -16,15 +15,15 @@ const bytesPerMegabyte = 1024 * 1024
 // applications that the Nomad agent will run on this machine.
 type StorageFingerprint struct {
 	StaticFingerprinter
-	logger *log.Logger
+	logger log.Logger
 }
 
-func NewStorageFingerprint(logger *log.Logger) Fingerprint {
-	fp := &StorageFingerprint{logger: logger}
+func NewStorageFingerprint(logger log.Logger) Fingerprint {
+	fp := &StorageFingerprint{logger: logger.Named("storage")}
 	return fp
 }
 
-func (f *StorageFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp *cstructs.FingerprintResponse) error {
+func (f *StorageFingerprint) Fingerprint(req *FingerprintRequest, resp *FingerprintResponse) error {
 	cfg := req.Config
 
 	// Guard against unset AllocDir
@@ -47,9 +46,15 @@ func (f *StorageFingerprint) Fingerprint(req *cstructs.FingerprintRequest, resp
 	resp.AddAttribute("unique.storage.bytesfree", strconv.FormatUint(free, 10))
 
 	// set the disk size for the response
+	// COMPAT(0.10): Remove in 0.10
 	resp.Resources = &structs.Resources{
 		DiskMB: int(free / bytesPerMegabyte),
 	}
+	resp.NodeResources = &structs.NodeResources{
+		Disk: structs.NodeDiskResources{
+			DiskMB: int64(free / bytesPerMegabyte),
+		},
+	}
 	resp.Detected = true
 
 	return nil
