@@ -4,7 +4,7 @@ import (
 	"testing"
 
 	"github.com/hashicorp/nomad/client/config"
-	cstructs "github.com/hashicorp/nomad/client/structs"
+	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/testutil"
 )
@@ -13,7 +13,7 @@ func TestVaultFingerprint(t *testing.T) {
 	tv := testutil.NewTestVault(t)
 	defer tv.Stop()
 
-	fp := NewVaultFingerprint(testLogger())
+	fp := NewVaultFingerprint(testlog.HCLogger(t))
 	node := &structs.Node{
 		Attributes: make(map[string]string),
 	}
@@ -21,8 +21,8 @@ func TestVaultFingerprint(t *testing.T) {
 	conf := config.DefaultConfig()
 	conf.VaultConfig = tv.Config
 
-	request := &cstructs.FingerprintRequest{Config: conf, Node: node}
-	var response cstructs.FingerprintResponse
+	request := &FingerprintRequest{Config: conf, Node: node}
+	var response FingerprintResponse
 	err := fp.Fingerprint(request, &response)
 	if err != nil {
 		t.Fatalf("Failed to fingerprint: %s", err)
