@@ -2,6 +2,7 @@ package logmon
 
 import (
 	"context"
+	"time"
 
 	"github.com/hashicorp/nomad/client/logmon/proto"
 	"github.com/hashicorp/nomad/helper/pluginutils/grpcutils"
@@ -14,6 +15,8 @@ type logmonClient struct {
 	doneCtx context.Context
 }
 
+const logmonRPCTimeout = 1 * time.Minute
+
 func (c *logmonClient) Start(cfg *LogConfig) error {
 	req := &proto.StartRequest{
 		LogDir:         cfg.LogDir,
@@ -24,12 +27,18 @@ func (c *logmonClient) Start(cfg *LogConfig) error {
 		StdoutFifo:     cfg.StdoutFifo,
 		StderrFifo:     cfg.StderrFifo,
 	}
-	_, err := c.client.Start(context.Background(), req)
+	ctx, cancel := context.WithTimeout(context.Background(), logmonRPCTimeout)
+	defer cancel()
+
+	_, err := c.client.Start(ctx, req)
 	return grpcutils.HandleGrpcErr(err, c.doneCtx)
 }
 
 func (c *logmonClient) Stop() error {
 	req := &proto.StopRequest{}
-	_, err := c.client.Stop(context.Background(), req)
+	ctx, cancel := context.WithTimeout(context.Background(), logmonRPCTimeout)
+	defer cancel()
+
+	_, err := c.client.Stop(ctx, req)
 	return grpcutils.HandleGrpcErr(err, c.doneCtx)
 }
