@@ -3,7 +3,6 @@ package testutil
 import (
 	"os/exec"
 	"runtime"
-	"sync"
 	"syscall"
 	"testing"
 
@@ -57,27 +56,6 @@ func CgroupCompatible(t *testing.T) {
 	}
 }
 
-var rktExists bool
-var rktOnce sync.Once
-
-func RktCompatible(t *testing.T) {
-	if runtime.GOOS != "linux" || syscall.Geteuid() != 0 {
-		t.Skip("Must be root on Linux to run test")
-	}
-
-	// else see if rkt exists
-	rktOnce.Do(func() {
-		_, err := exec.Command("rkt", "version").CombinedOutput()
-		if err == nil {
-			rktExists = true
-		}
-	})
-
-	if !rktExists {
-		t.Skip("Must have rkt installed for rkt specific tests to run")
-	}
-}
-
 func MountCompatible(t *testing.T) {
 	if runtime.GOOS == "windows" {
 		t.Skip("Windows does not support mount")
