@@ -2,7 +2,6 @@ package command
 
 import (
 	"fmt"
-	"os"
 	"strings"
 	"testing"
 
@@ -42,13 +41,11 @@ func TestACLPolicyDeleteCommand(t *testing.T) {
 
 	// Delete the policy without a valid token fails
 	invalidToken := mock.ACLToken()
-	os.Setenv("NOMAD_TOKEN", invalidToken.SecretID)
-	code := cmd.Run([]string{"-address=" + url, policy.Name})
+	code := cmd.Run([]string{"-address=" + url, "-token=" + invalidToken.SecretID, policy.Name})
 	assert.Equal(1, code)
 
 	// Delete the policy with a valid management token
-	os.Setenv("NOMAD_TOKEN", token.SecretID)
-	code = cmd.Run([]string{"-address=" + url, policy.Name})
+	code = cmd.Run([]string{"-address=" + url, "-token=" + token.SecretID, policy.Name})
 	assert.Equal(0, code)
 
 	// Check the output
