@@ -1,7 +1,6 @@
 package command
 
 import (
-	"os"
 	"strings"
 	"testing"
 
@@ -40,13 +39,11 @@ func TestACLPolicyInfoCommand(t *testing.T) {
 
 	// Attempt to apply a policy without a valid management token
 	invalidToken := mock.ACLToken()
-	os.Setenv("NOMAD_TOKEN", invalidToken.SecretID)
-	code := cmd.Run([]string{"-address=" + url, policy.Name})
+	code := cmd.Run([]string{"-address=" + url, "-token=" + invalidToken.SecretID, policy.Name})
 	assert.Equal(1, code)
 
 	// Apply a policy with a valid management token
-	os.Setenv("NOMAD_TOKEN", token.SecretID)
-	code = cmd.Run([]string{"-address=" + url, policy.Name})
+	code = cmd.Run([]string{"-address=" + url, "-token=" + token.SecretID, policy.Name})
 	assert.Equal(0, code)
 
 	// Check the output
