@@ -6,11 +6,10 @@ import (
 	"io"
 	"os"
 	"path/filepath"
-	"reflect"
-	"strings"
 	"time"
 
 	"github.com/hashicorp/hcl"
+	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/nomad/structs/config"
 )
 
@@ -35,6 +34,7 @@ func ParseConfigFile(path string) (*Config, error) {
 	c := &Config{
 		Client:    &ClientConfig{ServerJoin: &ServerJoin{}},
 		ACL:       &ACLConfig{},
+		Audit:     &config.AuditConfig{},
 		Server:    &ServerConfig{ServerJoin: &ServerJoin{}},
 		Consul:    &config.ConsulConfig{},
 		Autopilot: &config.AutopilotConfig{},
@@ -48,7 +48,7 @@ func ParseConfigFile(path string) (*Config, error) {
 	}
 
 	// convert strings to time.Durations
-	err = durations([]td{
+	tds := []td{
 		{"gc_interval", &c.Client.GCInterval, &c.Client.GCIntervalHCL},
 		{"acl.token_ttl", &c.ACL.TokenTTL, &c.ACL.TokenTTLHCL},
 		{"acl.policy_ttl", &c.ACL.PolicyTTL, &c.ACL.PolicyTTLHCL},
@@ -61,7 +61,17 @@ func ParseConfigFile(path string) (*Config, error) {
 		{"autopilot.server_stabilization_time", &c.Autopilot.ServerStabilizationTime, &c.Autopilot.ServerStabilizationTimeHCL},
 		{"autopilot.last_contact_threshold", &c.Autopilot.LastContactThreshold, &c.Autopilot.LastContactThresholdHCL},
 		{"telemetry.collection_interval", &c.Telemetry.collectionInterval, &c.Telemetry.CollectionInterval},
-	})
+	}
+
+	// Add enterprise audit sinks for time.Duration parsing
+	for i, sink := range c.Audit.Sinks {
+		tds = append(tds, td{
+			fmt.Sprintf("audit.sink.%d", i), &sink.RotateDuration, &sink.RotateDurationHCL,
+		})
+	}
+
+	// convert strings to time.Durations
+	err = durations(tds)
 	if err != nil {
 		return nil, err
 	}
@@ -99,106 +109,61 @@ func durations(xs []td) error {
 	return nil
 }
 
-// removeEqualFold removes the first string that EqualFold matches
-func removeEqualFold(xs *[]string, search string) {
-	sl := *xs
-	for i, x := range sl {
-		if strings.EqualFold(x, search) {
-			sl = append(sl[:i], sl[i+1:]...)
-			if len(sl) == 0 {
-				*xs = nil
-			} else {
-				*xs = sl
-			}
-			return
-		}
-	}
-}
-
 func extraKeys(c *Config) error {
 	// hcl leaves behind extra keys when parsing JSON. These keys
 	// are kept on the top level, taken from slices or the keys of
 	// structs contained in slices. Clean up before looking for
 	// extra keys.
 	for range c.HTTPAPIResponseHeaders {
-		removeEqualFold(&c.ExtraKeysHCL, "http_api_response_headers")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "http_api_response_headers")
 	}
 
 	for _, p := range c.Plugins {
-		removeEqualFold(&c.ExtraKeysHCL, p.Name)
-		removeEqualFold(&c.ExtraKeysHCL, "config")
-		removeEqualFold(&c.ExtraKeysHCL, "plugin")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, p.Name)
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "config")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "plugin")
 	}
 
 	for _, k := range []string{"options", "meta", "chroot_env", "servers", "server_join"} {
-		removeEqualFold(&c.ExtraKeysHCL, k)
-		removeEqualFold(&c.ExtraKeysHCL, "client")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, k)
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "client")
 	}
 
 	// stats is an unused key, continue to silently ignore it
-	removeEqualFold(&c.Client.ExtraKeysHCL, "stats")
+	helper.RemoveEqualFold(&c.Client.ExtraKeysHCL, "stats")
 
 	// Remove HostVolume extra keys
 	for _, hv := range c.Client.HostVolumes {
-		removeEqualFold(&c.Client.ExtraKeysHCL, hv.Name)
-		removeEqualFold(&c.Client.ExtraKeysHCL, "host_volume")
+		helper.RemoveEqualFold(&c.Client.ExtraKeysHCL, hv.Name)
+		helper.RemoveEqualFold(&c.Client.ExtraKeysHCL, "host_volume")
+	}
+
+	// Remove HostNetwork extra keys
+	for _, hn := range c.Client.HostNetworks {
+		helper.RemoveEqualFold(&c.Client.ExtraKeysHCL, hn.Name)
+		helper.RemoveEqualFold(&c.Client.ExtraKeysHCL, "host_network")
+	}
+
+	// Remove AuditConfig extra keys
+	for _, f := range c.Audit.Filters {
+		helper.RemoveEqualFold(&c.Audit.ExtraKeysHCL, f.Name)
+		helper.RemoveEqualFold(&c.Audit.ExtraKeysHCL, "filter")
+	}
+
+	for _, s := range c.Audit.Sinks {
+		helper.RemoveEqualFold(&c.Audit.ExtraKeysHCL, s.Name)
+		helper.RemoveEqualFold(&c.Audit.ExtraKeysHCL, "sink")
 	}
 
 	for _, k := range []string{"enabled_schedulers", "start_join", "retry_join", "server_join"} {
-		removeEqualFold(&c.ExtraKeysHCL, k)
-		removeEqualFold(&c.ExtraKeysHCL, "server")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, k)
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "server")
 	}
 
 	for _, k := range []string{"datadog_tags"} {
-		removeEqualFold(&c.ExtraKeysHCL, k)
-		removeEqualFold(&c.ExtraKeysHCL, "telemetry")
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, k)
+		helper.RemoveEqualFold(&c.ExtraKeysHCL, "telemetry")
 	}
 
-	return extraKeysImpl([]string{}, reflect.ValueOf(*c))
-}
-
-// extraKeysImpl returns an error if any extraKeys array is not empty
-func extraKeysImpl(path []string, val reflect.Value) error {
-	stype := val.Type()
-	for i := 0; i < stype.NumField(); i++ {
-		ftype := stype.Field(i)
-		fval := val.Field(i)
-
-		name := ftype.Name
-		prop := ""
-		tagSplit(ftype, "hcl", &name, &prop)
-
-		if fval.Kind() == reflect.Ptr {
-			fval = reflect.Indirect(fval)
-		}
-
-		// struct? recurse. add the struct's key to the path
-		if fval.Kind() == reflect.Struct {
-			err := extraKeysImpl(append([]string{name}, path...), fval)
-			if err != nil {
-				return err
-			}
-		}
-
-		if "unusedKeys" == prop {
-			if ks, ok := fval.Interface().([]string); ok && len(ks) != 0 {
-				return fmt.Errorf("%s unexpected keys %s",
-					strings.Join(path, "."),
-					strings.Join(ks, ", "))
-			}
-		}
-	}
-	return nil
-}
-
-// tagSplit reads the named tag from the structfield and splits its values into strings
-func tagSplit(field reflect.StructField, tagName string, vars ...*string) {
-	tag := strings.Split(field.Tag.Get(tagName), ",")
-	end := len(tag) - 1
-	for i, s := range vars {
-		if i > end {
-			return
-		}
-		*s = tag[i]
-	}
+	return helper.UnusedKeys(c)
 }
