@@ -5,6 +5,7 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -12,9 +13,11 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"net/url"
+	"strings"
 	"testing"
 	"time"
 
+	"github.com/hashicorp/go-msgpack/codec"
 	"github.com/hashicorp/nomad/api"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/helper/testlog"
@@ -24,7 +27,6 @@ import (
 	"github.com/hashicorp/nomad/testutil"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"github.com/ugorji/go/codec"
 )
 
 // makeHTTPServer returns a test server whose logs will be written to
@@ -976,7 +978,7 @@ func TestHTTPServer_Limits_OK(t *testing.T) {
 		// Now assert each error is a clientside read deadline error
 		for i := 0; i < maxConns; i++ {
 			select {
-			case <-time.After(1 * time.Second):
+			case <-time.After(2 * time.Second):
 				t.Fatalf("timed out waiting for conn error %d", i)
 			case err := <-errCh:
 				testutil.RequireDeadlineErr(t, err)
@@ -1006,6 +1008,12 @@ func TestHTTPServer_Limits_OK(t *testing.T) {
 			}(i)
 		}
 
+		select {
+		case err := <-errCh:
+			t.Fatalf("unexpected error from connection prior to limit: %T %v", err, err)
+		case <-time.After(500 * time.Millisecond):
+		}
+
 		// Assert a new connection is dropped
 		conn, err := net.DialTimeout("tcp", addr, 1*time.Second)
 		require.NoError(t, err)
@@ -1016,7 +1024,9 @@ func TestHTTPServer_Limits_OK(t *testing.T) {
 		conn.SetReadDeadline(deadline)
 		n, err := conn.Read(buf)
 		require.Zero(t, n)
-		require.Equal(t, io.EOF, err)
+
+		// Soft-fail as following assertion helps with debugging
+		assert.Equal(t, io.EOF, err)
 
 		// Assert existing connections are ok
 		require.Len(t, errCh, 0)
@@ -1056,7 +1066,16 @@ func TestHTTPServer_Limits_OK(t *testing.T) {
 			defer s.Shutdown()
 
 			assertTimeout(t, s, tc.assertTimeout, tc.timeout)
+
 			if tc.assertLimit {
+				// There's a race between assertTimeout(false) closing
+				// its connection and the HTTP server noticing and
+				// untracking it. Since there's no way to coordiante
+				// when this occurs, sleeping is the only way to avoid
+				// asserting limits before the timed out connection is
+				// untracked.
+				time.Sleep(1 * time.Second)
+
 				assertLimit(t, s.Server.Addr, *tc.limit)
 			} else {
 				assertNoLimit(t, s.Server.Addr)
@@ -1065,6 +1084,54 @@ func TestHTTPServer_Limits_OK(t *testing.T) {
 	}
 }
 
+func Test_IsAPIClientError(t *testing.T) {
+	trueCases := []int{400, 403, 404, 499}
+	for _, c := range trueCases {
+		require.Truef(t, isAPIClientError(c), "code: %v", c)
+	}
+
+	falseCases := []int{100, 300, 500, 501, 505}
+	for _, c := range falseCases {
+		require.Falsef(t, isAPIClientError(c), "code: %v", c)
+	}
+}
+
+func Test_decodeBody(t *testing.T) {
+
+	testCases := []struct {
+		inputReq      *http.Request
+		inputOut      interface{}
+		expectedOut   interface{}
+		expectedError error
+		name          string
+	}{
+		{
+			inputReq:      &http.Request{Body: http.NoBody},
+			expectedError: errors.New("Request body is empty"),
+			name:          "empty input request body",
+		},
+		{
+			inputReq: &http.Request{Body: ioutil.NopCloser(strings.NewReader(`{"foo":"bar"}`))},
+			inputOut: &struct {
+				Foo string `json:"foo"`
+			}{},
+			expectedOut: &struct {
+				Foo string `json:"foo"`
+			}{Foo: "bar"},
+			expectedError: nil,
+			name:          "populated request body and correct out",
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			actualError := decodeBody(tc.inputReq, tc.inputOut)
+			assert.Equal(t, tc.expectedError, actualError, tc.name)
+			assert.Equal(t, tc.expectedOut, tc.inputOut, tc.name)
+		})
+	}
+}
+
 func httpTest(t testing.TB, cb func(c *Config), f func(srv *TestAgent)) {
 	s := makeHTTPServer(t, cb)
 	defer s.Shutdown()
