@@ -88,3 +88,58 @@ func TestInitCommand_defaultJob(t *testing.T) {
 		t.Error("default job contains tab character - please convert to spaces")
 	}
 }
+
+func TestInitCommand_customFilename(t *testing.T) {
+	t.Parallel()
+	ui := new(cli.MockUi)
+	cmd := &JobInitCommand{Meta: Meta{Ui: ui}}
+	filename := "custom.nomad"
+
+	// Ensure we change the cwd back
+	origDir, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("err: %s", err)
+	}
+	defer os.Chdir(origDir)
+
+	// Create a temp dir and change into it
+	dir, err := ioutil.TempDir("", "nomad")
+	if err != nil {
+		t.Fatalf("err: %s", err)
+	}
+	defer os.RemoveAll(dir)
+	if err := os.Chdir(dir); err != nil {
+		t.Fatalf("err: %s", err)
+	}
+
+	// Works if the file doesn't exist
+	if code := cmd.Run([]string{filename}); code != 0 {
+		t.Fatalf("expect exit code 0, got: %d", code)
+	}
+	content, err := ioutil.ReadFile(filename)
+	if err != nil {
+		t.Fatalf("err: %s", err)
+	}
+	defaultJob, _ := Asset("command/assets/example.nomad")
+	if string(content) != string(defaultJob) {
+		t.Fatalf("unexpected file content\n\n%s", string(content))
+	}
+
+	// Works with -short flag
+	os.Remove(filename)
+	if code := cmd.Run([]string{"-short", filename}); code != 0 {
+		require.Zero(t, code, "unexpected exit code: %d", code)
+	}
+	content, err = ioutil.ReadFile(filename)
+	require.NoError(t, err)
+	shortJob, _ := Asset("command/assets/example-short.nomad")
+	require.Equal(t, string(content), string(shortJob))
+
+	// Fails if the file exists
+	if code := cmd.Run([]string{filename}); code != 1 {
+		t.Fatalf("expect exit code 1, got: %d", code)
+	}
+	if out := ui.ErrorWriter.String(); !strings.Contains(out, "exists") {
+		t.Fatalf("expect file exists error, got: %s", out)
+	}
+}
