@@ -17,7 +17,7 @@ const (
 nomad job run -check-index %d %s
 
 When running the job with the check-index flag, the job will only be run if the
-server side version matches the job modify index returned. If the index has
+job modify index given matches the server-side version. If the index has
 changed, another user has modified the job and the plan's results are
 potentially invalid.`
 
@@ -49,6 +49,7 @@ Alias: nomad plan
   submitting the job using "nomad run -check-index", which will check that the job
   was not modified between the plan and run command before invoking the
   scheduler. This ensures the job has not been modified since the plan.
+  Multiregion jobs do not return a job modify index.
 
   A structured diff between the local and remote job is displayed to
   give insight into what the scheduler will attempt to do and why.
@@ -153,6 +154,10 @@ func (c *JobPlanCommand) Run(args []string) int {
 		opts.PolicyOverride = true
 	}
 
+	if job.IsMultiregion() {
+		return c.multiregionPlan(client, job, opts, diff, verbose)
+	}
+
 	// Submit the job
 	resp, _, err := client.Jobs().PlanOpts(job, opts, nil)
 	if err != nil {
@@ -160,6 +165,46 @@ func (c *JobPlanCommand) Run(args []string) int {
 		return 255
 	}
 
+	exitCode := c.outputPlannedJob(job, resp, diff, verbose)
+	c.Ui.Output(c.Colorize().Color(formatJobModifyIndex(resp.JobModifyIndex, path)))
+	return exitCode
+}
+
+func (c *JobPlanCommand) multiregionPlan(client *api.Client, job *api.Job, opts *api.PlanOptions, diff, verbose bool) int {
+
+	var exitCode int
+	plans := map[string]*api.JobPlanResponse{}
+
+	// collect all the plans first so that we can report all errors
+	for _, region := range job.Multiregion.Regions {
+		regionName := region.Name
+		client.SetRegion(regionName)
+
+		// Submit the job for this region
+		resp, _, err := client.Jobs().PlanOpts(job, opts, nil)
+		if err != nil {
+			c.Ui.Error(fmt.Sprintf("Error during plan for region %q: %s", regionName, err))
+			exitCode = 255
+		}
+		plans[regionName] = resp
+	}
+
+	if exitCode > 0 {
+		return exitCode
+	}
+
+	for regionName, resp := range plans {
+		c.Ui.Output(c.Colorize().Color(fmt.Sprintf("[bold]Region: %q[reset]", regionName)))
+		regionExitCode := c.outputPlannedJob(job, resp, diff, verbose)
+		if regionExitCode > exitCode {
+			exitCode = regionExitCode
+		}
+	}
+	return exitCode
+}
+
+func (c *JobPlanCommand) outputPlannedJob(job *api.Job, resp *api.JobPlanResponse, diff, verbose bool) int {
+
 	// Print the diff if not disabled
 	if diff {
 		c.Ui.Output(fmt.Sprintf("%s\n",
@@ -182,8 +227,6 @@ func (c *JobPlanCommand) Run(args []string) int {
 		c.addPreemptions(resp)
 	}
 
-	// Print the job index info
-	c.Ui.Output(c.Colorize().Color(formatJobModifyIndex(resp.JobModifyIndex, path)))
 	return getExitCode(resp)
 }
 
