@@ -26,6 +26,7 @@ func TestOperatorAutopilotSetConfigCommand(t *testing.T) {
 		"-address=" + addr,
 		"-cleanup-dead-servers=false",
 		"-max-trailing-logs=99",
+		"-min-quorum=3",
 		"-last-contact-threshold=123ms",
 		"-server-stabilization-time=123ms",
 		"-enable-redundancy-zones=true",
@@ -46,6 +47,7 @@ func TestOperatorAutopilotSetConfigCommand(t *testing.T) {
 
 	require.False(conf.CleanupDeadServers)
 	require.EqualValues(99, conf.MaxTrailingLogs)
+	require.EqualValues(3, conf.MinQuorum)
 	require.EqualValues(123*time.Millisecond, conf.LastContactThreshold)
 	require.EqualValues(123*time.Millisecond, conf.ServerStabilizationTime)
 	require.True(conf.EnableRedundancyZones)
