@@ -21,10 +21,11 @@ func TestConfig_ParseHCL(t *testing.T) {
 				image = "redis:3.2"
 			}`,
 			&TaskConfig{
-				Image:        "redis:3.2",
-				Devices:      []DockerDevice{},
-				Mounts:       []DockerMount{},
-				CPUCFSPeriod: 100000,
+				Image:            "redis:3.2",
+				Devices:          []DockerDevice{},
+				Mounts:           []DockerMount{},
+				CPUCFSPeriod:     100000,
+				ImagePullTimeout: "5m",
 			},
 		},
 	}
@@ -53,40 +54,44 @@ func TestConfig_ParseJSON(t *testing.T) {
 			name:  "nil values for blocks are safe",
 			input: `{"Config": {"image": "bash:3", "mounts": null}}`,
 			expected: TaskConfig{
-				Image:        "bash:3",
-				Mounts:       []DockerMount{},
-				Devices:      []DockerDevice{},
-				CPUCFSPeriod: 100000,
+				Image:            "bash:3",
+				Mounts:           []DockerMount{},
+				Devices:          []DockerDevice{},
+				CPUCFSPeriod:     100000,
+				ImagePullTimeout: "5m",
 			},
 		},
 		{
 			name:  "nil values for 'volumes' field are safe",
 			input: `{"Config": {"image": "bash:3", "volumes": null}}`,
 			expected: TaskConfig{
-				Image:        "bash:3",
-				Mounts:       []DockerMount{},
-				Devices:      []DockerDevice{},
-				CPUCFSPeriod: 100000,
+				Image:            "bash:3",
+				Mounts:           []DockerMount{},
+				Devices:          []DockerDevice{},
+				CPUCFSPeriod:     100000,
+				ImagePullTimeout: "5m",
 			},
 		},
 		{
 			name:  "nil values for 'args' field are safe",
 			input: `{"Config": {"image": "bash:3", "args": null}}`,
 			expected: TaskConfig{
-				Image:        "bash:3",
-				Mounts:       []DockerMount{},
-				Devices:      []DockerDevice{},
-				CPUCFSPeriod: 100000,
+				Image:            "bash:3",
+				Mounts:           []DockerMount{},
+				Devices:          []DockerDevice{},
+				CPUCFSPeriod:     100000,
+				ImagePullTimeout: "5m",
 			},
 		},
 		{
 			name:  "nil values for string fields are safe",
 			input: `{"Config": {"image": "bash:3", "command": null}}`,
 			expected: TaskConfig{
-				Image:        "bash:3",
-				Mounts:       []DockerMount{},
-				Devices:      []DockerDevice{},
-				CPUCFSPeriod: 100000,
+				Image:            "bash:3",
+				Mounts:           []DockerMount{},
+				Devices:          []DockerDevice{},
+				CPUCFSPeriod:     100000,
+				ImagePullTimeout: "5m",
 			},
 		},
 	}
@@ -178,6 +183,7 @@ func TestConfig_ParseAllHCL(t *testing.T) {
 	cfgStr := `
 config {
   image = "redis:3.2"
+  image_pull_timeout = "15m"
   advertise_ipv6_address = true
   args = ["command_arg1", "command_arg2"]
   auth {
@@ -222,6 +228,7 @@ config {
     }
   }
   mac_address = "02:42:ac:11:00:02"
+  memory_hard_limit = 512
   mounts = [
     {
       type = "bind"
@@ -263,13 +270,15 @@ config {
   network_aliases = ["redis"]
   network_mode = "host"
   pids_limit = 2000
-  pid_mode = "host"
+	pid_mode = "host"
+	ports = ["http", "https"]
   port_map {
     http = 80
     redis = 6379
   }
   privileged = true
   readonly_rootfs = true
+  runtime = "runc"
   security_opt = [
     "credentialspec=file://gmsaUser.json"
   ],
@@ -299,6 +308,7 @@ config {
 
 	expected := &TaskConfig{
 		Image:             "redis:3.2",
+		ImagePullTimeout:  "15m",
 		AdvertiseIPv6Addr: true,
 		Args:              []string{"command_arg1", "command_arg2"},
 		Auth: DockerAuth{
@@ -348,7 +358,8 @@ config {
 				"max-file": "3",
 				"max-size": "10m",
 			}},
-		MacAddress: "02:42:ac:11:00:02",
+		MacAddress:      "02:42:ac:11:00:02",
+		MemoryHardLimit: 512,
 		Mounts: []DockerMount{
 			{
 				Type:     "bind",
@@ -392,12 +403,14 @@ config {
 		NetworkMode:    "host",
 		PidsLimit:      2000,
 		PidMode:        "host",
+		Ports:          []string{"http", "https"},
 		PortMap: map[string]int{
 			"http":  80,
 			"redis": 6379,
 		},
 		Privileged:     true,
 		ReadonlyRootfs: true,
+		Runtime:        "runc",
 		SecurityOpt: []string{
 			"credentialspec=file://gmsaUser.json",
 		},
@@ -522,7 +535,33 @@ func TestConfig_InternalCapabilities(t *testing.T) {
 			require.Equal(t, c.expected, d.InternalCapabilities())
 		})
 	}
+}
 
+func TestConfig_DriverConfig_InfraImagePullTimeout(t *testing.T) {
+	cases := []struct {
+		name     string
+		config   string
+		expected string
+	}{
+		{
+			name:     "default",
+			config:   `{}`,
+			expected: "5m",
+		},
+		{
+			name:     "set explicitly",
+			config:   `{ infra_image_pull_timeout = "1m" }`,
+			expected: "1m",
+		},
+	}
+
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			var tc DriverConfig
+			hclutils.NewConfigParser(configSpec).ParseHCL(t, "config "+c.config, &tc)
+			require.Equal(t, c.expected, tc.InfraImagePullTimeout)
+		})
+	}
 }
 
 func TestConfig_DriverConfig_PullActivityTimeout(t *testing.T) {
@@ -551,3 +590,33 @@ func TestConfig_DriverConfig_PullActivityTimeout(t *testing.T) {
 		})
 	}
 }
+
+func TestConfig_DriverConfig_AllowRuntimes(t *testing.T) {
+	cases := []struct {
+		name     string
+		config   string
+		expected map[string]struct{}
+	}{
+		{
+			name:     "pure default",
+			config:   `{}`,
+			expected: map[string]struct{}{"runc": struct{}{}, "nvidia": struct{}{}},
+		},
+		{
+			name:     "custom",
+			config:   `{ allow_runtimes = ["runc", "firecracker"]}`,
+			expected: map[string]struct{}{"runc": struct{}{}, "firecracker": struct{}{}},
+		},
+	}
+
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			var tc map[string]interface{}
+			hclutils.NewConfigParser(configSpec).ParseHCL(t, "config "+c.config, &tc)
+
+			dh := dockerDriverHarness(t, tc)
+			d := dh.Impl().(*Driver)
+			require.Equal(t, c.expected, d.config.allowRuntimes)
+		})
+	}
+}
