@@ -56,10 +56,10 @@ func TestDockerDriver_PidsLimit(t *testing.T) {
 	defer freeport.Return(ports)
 	cfg.PidsLimit = 1
 	cfg.Command = "/bin/sh"
-	cfg.Args = []string{"-c", "sleep 2 & sleep 2"}
+	cfg.Args = []string{"-c", "sleep 5 & sleep 5 & sleep 5"}
 	require.NoError(task.EncodeConcreteDriverConfig(cfg))
 
-	_, driver, _, cleanup := dockerSetup(t, task)
+	_, driver, _, cleanup := dockerSetup(t, task, nil)
 	defer cleanup()
 
 	driver.WaitUntilStarted(task.ID, time.Duration(tu.TestMultiplier()*5)*time.Second)
