@@ -55,6 +55,8 @@ func parseGroups(result *api.Job, list *ast.ObjectList) error {
 			"network",
 			"service",
 			"volume",
+			"scaling",
+			"stop_after_client_disconnect",
 		}
 		if err := helper.CheckHCLKeys(listVal, valid); err != nil {
 			return multierror.Prefix(err, fmt.Sprintf("'%s' ->", n))
@@ -78,6 +80,7 @@ func parseGroups(result *api.Job, list *ast.ObjectList) error {
 		delete(m, "network")
 		delete(m, "service")
 		delete(m, "volume")
+		delete(m, "scaling")
 
 		// Build the group with the basic decode
 		var g api.TaskGroup
@@ -181,6 +184,13 @@ func parseGroups(result *api.Job, list *ast.ObjectList) error {
 			}
 		}
 
+		// Parse scaling policy
+		if o := listVal.Filter("scaling"); len(o.Items) > 0 {
+			if err := parseScalingPolicy(&g.Scaling, o); err != nil {
+				return multierror.Prefix(err, "scaling ->")
+			}
+		}
+
 		// Parse tasks
 		if o := listVal.Filter("task"); len(o.Items) > 0 {
 			if err := parseTasks(&g.Tasks, o); err != nil {
@@ -295,41 +305,84 @@ func parseRestartPolicy(final **api.RestartPolicy, list *ast.ObjectList) error {
 }
 
 func parseVolumes(out *map[string]*api.VolumeRequest, list *ast.ObjectList) error {
-	volumes := make(map[string]*api.VolumeRequest, len(list.Items))
+	hcl.DecodeObject(out, list)
 
-	for _, item := range list.Items {
-		n := item.Keys[0].Token.Value().(string)
-		valid := []string{
-			"type",
-			"read_only",
-			"hidden",
-			"source",
-		}
-		if err := helper.CheckHCLKeys(item.Val, valid); err != nil {
-			return err
-		}
-
-		var m map[string]interface{}
-		if err := hcl.DecodeObject(&m, item.Val); err != nil {
-			return err
-		}
-
-		var result api.VolumeRequest
-		dec, err := mapstructure.NewDecoder(&mapstructure.DecoderConfig{
-			WeaklyTypedInput: true,
-			Result:           &result,
-		})
+	for k, v := range *out {
+		err := helper.UnusedKeys(v)
 		if err != nil {
 			return err
 		}
-		if err := dec.Decode(m); err != nil {
-			return err
-		}
-
-		result.Name = n
-		volumes[n] = &result
+		// This is supported by `hcl:",key"`, but that only works if we start at the
+		// parent ast.ObjectItem
+		v.Name = k
 	}
 
-	*out = volumes
+	return nil
+}
+
+func parseScalingPolicy(out **api.ScalingPolicy, list *ast.ObjectList) error {
+	list = list.Elem()
+	if len(list.Items) > 1 {
+		return fmt.Errorf("only one 'scaling' block allowed")
+	}
+
+	// Get our resource object
+	o := list.Items[0]
+
+	// We need this later
+	var listVal *ast.ObjectList
+	if ot, ok := o.Val.(*ast.ObjectType); ok {
+		listVal = ot.List
+	} else {
+		return fmt.Errorf("should be an object")
+	}
+
+	valid := []string{
+		"min",
+		"max",
+		"policy",
+		"enabled",
+	}
+	if err := helper.CheckHCLKeys(o.Val, valid); err != nil {
+		return err
+	}
+
+	var m map[string]interface{}
+	if err := hcl.DecodeObject(&m, o.Val); err != nil {
+		return err
+	}
+	delete(m, "policy")
+
+	var result api.ScalingPolicy
+	dec, err := mapstructure.NewDecoder(&mapstructure.DecoderConfig{
+		WeaklyTypedInput: true,
+		Result:           &result,
+	})
+	if err != nil {
+		return err
+	}
+	if err := dec.Decode(m); err != nil {
+		return err
+	}
+	if result.Max == nil {
+		return fmt.Errorf("missing 'max'")
+	}
+
+	// If we have policy, then parse that
+	if o := listVal.Filter("policy"); len(o.Items) > 0 {
+		if len(o.Elem().Items) > 1 {
+			return fmt.Errorf("only one 'policy' block allowed per 'scaling' block")
+		}
+		p := o.Elem().Items[0]
+		var m map[string]interface{}
+		if err := hcl.DecodeObject(&m, p.Val); err != nil {
+			return err
+		}
+		if err := mapstructure.WeakDecode(m, &result.Policy); err != nil {
+			return err
+		}
+	}
+
+	*out = &result
 	return nil
 }
