@@ -38,6 +38,7 @@ func parseJob(result *api.Job, list *ast.ObjectList) error {
 	delete(m, "update")
 	delete(m, "vault")
 	delete(m, "spread")
+	delete(m, "multiregion")
 
 	// Set the ID and name to the object key
 	result.ID = helper.StringToPtr(obj.Keys[0].Token.Value().(string))
@@ -80,6 +81,7 @@ func parseJob(result *api.Job, list *ast.ObjectList) error {
 		"vault",
 		"vault_token",
 		"consul_token",
+		"multiregion",
 	}
 	if err := helper.CheckHCLKeys(listVal, valid); err != nil {
 		return multierror.Prefix(err, "job:")
@@ -141,6 +143,15 @@ func parseJob(result *api.Job, list *ast.ObjectList) error {
 		}
 	}
 
+	// If we have a multiregion block, then parse that
+	if o := listVal.Filter("multiregion"); len(o.Items) > 0 {
+		var mr api.Multiregion
+		if err := parseMultiregion(&mr, o); err != nil {
+			return multierror.Prefix(err, "multiregion ->")
+		}
+		result.Multiregion = &mr
+	}
+
 	// Parse out meta fields. These are in HCL as a list so we need
 	// to iterate over them and merge them.
 	if metaO := listVal.Filter("meta"); len(metaO.Items) > 0 {
