@@ -117,11 +117,32 @@ func TestParse(t *testing.T) {
 								Operand: "=",
 							},
 						},
-
 						Volumes: map[string]*api.VolumeRequest{
 							"foo": {
-								Name: "foo",
-								Type: "host",
+								Name:         "foo",
+								Type:         "host",
+								Source:       "/path",
+								ExtraKeysHCL: nil,
+							},
+							"bar": {
+								Name:   "bar",
+								Type:   "csi",
+								Source: "bar-vol",
+								MountOptions: &api.CSIMountOptions{
+									FSType: "ext4",
+								},
+								ExtraKeysHCL: nil,
+							},
+							"baz": {
+								Name:   "baz",
+								Type:   "csi",
+								Source: "bar-vol",
+								MountOptions: &api.CSIMountOptions{
+									MountFlags: []string{
+										"ro",
+									},
+								},
+								ExtraKeysHCL: nil,
 							},
 						},
 						Affinities: []*api.Affinity{
@@ -163,6 +184,7 @@ func TestParse(t *testing.T) {
 								},
 							},
 						},
+						StopAfterClientDisconnect: helper.TimeToPtr(120 * time.Second),
 						ReschedulePolicy: &api.ReschedulePolicy{
 							Interval: helper.TimeToPtr(12 * time.Hour),
 							Attempts: helper.IntToPtr(5),
@@ -215,6 +237,9 @@ func TestParse(t *testing.T) {
 										Weight:  helper.Int8ToPtr(25),
 									},
 								},
+								RestartPolicy: &api.RestartPolicy{
+									Attempts: helper.IntToPtr(10),
+								},
 								Services: []*api.Service{
 									{
 										Tags:       []string{"foo", "bar"},
@@ -307,6 +332,7 @@ func TestParse(t *testing.T) {
 									},
 								},
 								Vault: &api.Vault{
+									Namespace:  helper.StringToPtr("ns1"),
 									Policies:   []string{"foo", "bar"},
 									Env:        helper.BoolToPtr(true),
 									ChangeMode: helper.StringToPtr(structs.VaultChangeModeRestart),
@@ -339,6 +365,10 @@ func TestParse(t *testing.T) {
 								Name:   "storagelocker",
 								Driver: "docker",
 								User:   "",
+								Lifecycle: &api.TaskLifecycle{
+									Hook:    "prestart",
+									Sidecar: true,
+								},
 								Config: map[string]interface{}{
 									"image": "hashicorp/storagelocker",
 								},
@@ -565,6 +595,30 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"csi-plugin.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("binstore-storagelocker"),
+				Name: helper.StringToPtr("binstore-storagelocker"),
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name: helper.StringToPtr("binsl"),
+						Tasks: []*api.Task{
+							{
+								Name:   "binstore",
+								Driver: "docker",
+								CSIPluginConfig: &api.TaskCSIPluginConfig{
+									ID:       "org.hashicorp.csi",
+									Type:     api.CSIPluginTypeMonolith,
+									MountDir: "/csi/test",
+								},
+							},
+						},
+					},
+				},
+			},
+			false,
+		},
 		{
 			"service-check-initial-status.hcl",
 			&api.Job{
@@ -605,6 +659,68 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"service-check-pass-fail.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("check_pass_fail"),
+				Name: helper.StringToPtr("check_pass_fail"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name:  helper.StringToPtr("group"),
+					Count: helper.IntToPtr(1),
+					Tasks: []*api.Task{{
+						Name: "task",
+						Services: []*api.Service{{
+							Name:      "service",
+							PortLabel: "http",
+							Checks: []api.ServiceCheck{{
+								Name:                   "check-name",
+								Type:                   "http",
+								Path:                   "/",
+								Interval:               10 * time.Second,
+								Timeout:                2 * time.Second,
+								InitialStatus:          capi.HealthPassing,
+								Method:                 "POST",
+								SuccessBeforePassing:   3,
+								FailuresBeforeCritical: 4,
+							}},
+						}},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"service-check-pass-fail.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("check_pass_fail"),
+				Name: helper.StringToPtr("check_pass_fail"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name:  helper.StringToPtr("group"),
+					Count: helper.IntToPtr(1),
+					Tasks: []*api.Task{{
+						Name: "task",
+						Services: []*api.Service{{
+							Name:      "service",
+							PortLabel: "http",
+							Checks: []api.ServiceCheck{{
+								Name:                   "check-name",
+								Type:                   "http",
+								Path:                   "/",
+								Interval:               10 * time.Second,
+								Timeout:                2 * time.Second,
+								InitialStatus:          capi.HealthPassing,
+								Method:                 "POST",
+								SuccessBeforePassing:   3,
+								FailuresBeforeCritical: 4,
+							}},
+						}},
+					}},
+				}},
+			},
+			false,
+		},
 		{
 			"service-check-bad-header.hcl",
 			nil,
@@ -823,6 +939,25 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"service-enable-tag-override.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("service_eto"),
+				Name: helper.StringToPtr("service_eto"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Tasks: []*api.Task{{
+						Name: "task",
+						Services: []*api.Service{{
+							Name:              "example",
+							EnableTagOverride: true,
+						}},
+					}},
+				}},
+			},
+			false,
+		},
 		{
 			"reschedule-job.hcl",
 			&api.Job{
@@ -941,11 +1076,16 @@ func TestParse(t *testing.T) {
 								Mode: "bridge",
 								ReservedPorts: []api.Port{
 									{
-										Label: "http",
-										Value: 80,
-										To:    8080,
+										Label:       "http",
+										Value:       80,
+										To:          8080,
+										HostNetwork: "public",
 									},
 								},
+								DNS: &api.DNSConfig{
+									Servers: []string{"8.8.8.8"},
+									Options: []string{"ndots:2", "edns0"},
+								},
 							},
 						},
 						Services: []*api.Service{
@@ -958,6 +1098,7 @@ func TestParse(t *testing.T) {
 									SidecarService: &api.ConsulSidecarService{
 										Tags: []string{"side1", "side2"},
 										Proxy: &api.ConsulProxy{
+											LocalServicePort: 8080,
 											Upstreams: []*api.ConsulUpstream{
 												{
 													DestinationName: "other-service",
@@ -1046,6 +1187,333 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"tg-service-proxy-expose.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("group_service_proxy_expose"),
+				Name: helper.StringToPtr("group_service_proxy_expose"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "example",
+						Connect: &api.ConsulConnect{
+							SidecarService: &api.ConsulSidecarService{
+								Proxy: &api.ConsulProxy{
+									ExposeConfig: &api.ConsulExposeConfig{
+										Path: []*api.ConsulExposePath{{
+											Path:          "/health",
+											Protocol:      "http",
+											LocalPathPort: 2222,
+											ListenerPort:  "healthcheck",
+										}, {
+											Path:          "/metrics",
+											Protocol:      "grpc",
+											LocalPathPort: 3000,
+											ListenerPort:  "metrics",
+										}},
+									},
+								},
+							},
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-connect-sidecar_task-name.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("sidecar_task_name"),
+				Name: helper.StringToPtr("sidecar_task_name"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "example",
+						Connect: &api.ConsulConnect{
+							Native:         false,
+							SidecarService: &api.ConsulSidecarService{},
+							SidecarTask: &api.SidecarTask{
+								Name: "my-sidecar",
+							},
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-connect-proxy.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("service-connect-proxy"),
+				Name: helper.StringToPtr("service-connect-proxy"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "example",
+						Connect: &api.ConsulConnect{
+							Native: false,
+							SidecarService: &api.ConsulSidecarService{
+								Proxy: &api.ConsulProxy{
+									LocalServiceAddress: "10.0.1.2",
+									LocalServicePort:    8080,
+									ExposeConfig: &api.ConsulExposeConfig{
+										Path: []*api.ConsulExposePath{{
+											Path:          "/metrics",
+											Protocol:      "http",
+											LocalPathPort: 9001,
+											ListenerPort:  "metrics",
+										}, {
+											Path:          "/health",
+											Protocol:      "http",
+											LocalPathPort: 9002,
+											ListenerPort:  "health",
+										}},
+									},
+									Upstreams: []*api.ConsulUpstream{{
+										DestinationName: "upstream1",
+										LocalBindPort:   2001,
+									}, {
+										DestinationName: "upstream2",
+										LocalBindPort:   2002,
+									}},
+									Config: map[string]interface{}{
+										"foo": "bar",
+									},
+								},
+							},
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-connect-local-service.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("connect-proxy-local-service"),
+				Name: helper.StringToPtr("connect-proxy-local-service"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "example",
+						Connect: &api.ConsulConnect{
+							Native: false,
+							SidecarService: &api.ConsulSidecarService{
+								Proxy: &api.ConsulProxy{
+									LocalServiceAddress: "10.0.1.2",
+									LocalServicePort:    9876,
+								},
+							},
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-check-expose.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("group_service_proxy_expose"),
+				Name: helper.StringToPtr("group_service_proxy_expose"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "example",
+						Connect: &api.ConsulConnect{
+							SidecarService: &api.ConsulSidecarService{
+								Proxy: &api.ConsulProxy{},
+							},
+						},
+						Checks: []api.ServiceCheck{{
+							Name:   "example-check1",
+							Expose: true,
+						}, {
+							Name:   "example-check2",
+							Expose: false,
+						}},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-connect-native.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("connect_native_service"),
+				Name: helper.StringToPtr("connect_native_service"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name:     "example",
+						TaskName: "task1",
+						Connect: &api.ConsulConnect{
+							Native: true,
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-service-enable-tag-override.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("group_service_eto"),
+				Name: helper.StringToPtr("group_service_eto"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name:              "example",
+						EnableTagOverride: true,
+					}},
+				}},
+			},
+			false,
+		},
+
+		{
+			"tg-scaling-policy.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("elastic"),
+				Name: helper.StringToPtr("elastic"),
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name: helper.StringToPtr("group"),
+						Scaling: &api.ScalingPolicy{
+							Min: helper.Int64ToPtr(5),
+							Max: helper.Int64ToPtr(100),
+							Policy: map[string]interface{}{
+								"foo": "bar",
+								"b":   true,
+								"val": 5,
+								"f":   .1,
+							},
+							Enabled: helper.BoolToPtr(false),
+						},
+					},
+				},
+			},
+			false,
+		},
+		{
+			"tg-service-connect-gateway-ingress.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("connect_gateway_ingress"),
+				Name: helper.StringToPtr("connect_gateway_ingress"),
+				TaskGroups: []*api.TaskGroup{{
+					Name: helper.StringToPtr("group"),
+					Services: []*api.Service{{
+						Name: "ingress-gateway-service",
+						Connect: &api.ConsulConnect{
+							Gateway: &api.ConsulGateway{
+								Proxy: &api.ConsulGatewayProxy{
+									ConnectTimeout:                  helper.TimeToPtr(3 * time.Second),
+									EnvoyGatewayBindTaggedAddresses: true,
+									EnvoyGatewayBindAddresses: map[string]*api.ConsulGatewayBindAddress{
+										"listener1": {Address: "10.0.0.1", Port: 8888},
+										"listener2": {Address: "10.0.0.2", Port: 8889},
+									},
+									EnvoyGatewayNoDefaultBind: true,
+									Config:                    map[string]interface{}{"foo": "bar"},
+								},
+								Ingress: &api.ConsulIngressConfigEntry{
+									TLS: &api.ConsulGatewayTLSConfig{
+										Enabled: true,
+									},
+									Listeners: []*api.ConsulIngressListener{{
+										Port:     8001,
+										Protocol: "tcp",
+										Services: []*api.ConsulIngressService{{
+											Name: "service1",
+											Hosts: []string{
+												"127.0.0.1:8001",
+												"[::1]:8001",
+											}}, {
+											Name: "service2",
+											Hosts: []string{
+												"10.0.0.1:8001",
+											}},
+										}}, {
+										Port:     8080,
+										Protocol: "http",
+										Services: []*api.ConsulIngressService{{
+											Name: "nginx",
+											Hosts: []string{
+												"2.2.2.2:8080",
+											},
+										}},
+									},
+									},
+								},
+							},
+						},
+					}},
+				}},
+			},
+			false,
+		},
+		{
+			"tg-scaling-policy-minimal.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("elastic"),
+				Name: helper.StringToPtr("elastic"),
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name: helper.StringToPtr("group"),
+						Scaling: &api.ScalingPolicy{
+							Min:     nil,
+							Max:     helper.Int64ToPtr(10),
+							Policy:  nil,
+							Enabled: nil,
+						},
+					},
+				},
+			},
+			false,
+		},
+
+		{
+			"tg-scaling-policy-missing-max.hcl",
+			nil,
+			true,
+		},
+
+		{
+			"tg-scaling-policy-multi-policy.hcl",
+			nil,
+			true,
+		},
+
+		{
+			"multiregion.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("multiregion_job"),
+				Name: helper.StringToPtr("multiregion_job"),
+				Multiregion: &api.Multiregion{
+					Strategy: &api.MultiregionStrategy{
+						MaxParallel: helper.IntToPtr(1),
+						OnFailure:   helper.StringToPtr("fail_all"),
+					},
+					Regions: []*api.MultiregionRegion{
+						{
+							Name:        "west",
+							Count:       helper.IntToPtr(2),
+							Datacenters: []string{"west-1"},
+							Meta:        map[string]string{"region_code": "W"},
+						},
+						{
+							Name:        "east",
+							Count:       helper.IntToPtr(1),
+							Datacenters: []string{"east-1", "east-2"},
+							Meta:        map[string]string{"region_code": "E"},
+						},
+					},
+				},
+			},
+			false,
+		},
 	}
 
 	for _, tc := range cases {
