@@ -1,10 +1,10 @@
-// +build !pro,!ent
+// +build !ent
 
 package nomad
 
 import (
+	"github.com/hashicorp/go-msgpack/codec"
 	"github.com/hashicorp/raft"
-	"github.com/ugorji/go/codec"
 )
 
 // registerLogAppliers is a no-op for open-source only FSMs.
