@@ -7,7 +7,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/hashicorp/consul/testutil/retry"
+	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/hashicorp/go-hclog"
 	memdb "github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/go-version"
@@ -23,16 +23,18 @@ import (
 )
 
 func TestLeader_LeftServer(t *testing.T) {
-	s1, cleanupS1 := TestServer(t, nil)
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
+		c.BootstrapExpect = 3
+	})
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
 	servers := []*Server{s1, s2, s3}
@@ -83,16 +85,18 @@ func TestLeader_LeftServer(t *testing.T) {
 }
 
 func TestLeader_LeftLeader(t *testing.T) {
-	s1, cleanupS1 := TestServer(t, nil)
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
+		c.BootstrapExpect = 3
+	})
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
 	servers := []*Server{s1, s2, s3}
@@ -154,24 +158,29 @@ func TestLeader_MultiBootstrap(t *testing.T) {
 
 	// Ensure we don't have multiple raft peers
 	for _, s := range servers {
-		peers, _ := s.numPeers()
+		peers, err := s.numPeers()
+		if err != nil {
+			t.Fatalf("failed: %v", err)
+		}
 		if peers != 1 {
-			t.Fatalf("should only have 1 raft peer!")
+			t.Fatalf("should only have 1 raft peer! %v", peers)
 		}
 	}
 }
 
 func TestLeader_PlanQueue_Reset(t *testing.T) {
-	s1, cleanupS1 := TestServer(t, nil)
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
+		c.BootstrapExpect = 3
+	})
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
 	servers := []*Server{s1, s2, s3}
@@ -223,13 +232,13 @@ func TestLeader_EvalBroker_Reset(t *testing.T) {
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
 	servers := []*Server{s1, s2, s3}
@@ -281,13 +290,13 @@ func TestLeader_PeriodicDispatcher_Restore_Adds(t *testing.T) {
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
 	servers := []*Server{s1, s2, s3}
@@ -642,7 +651,7 @@ func TestLeader_revokeSITokenAccessorsOnRestore(t *testing.T) {
 	defer cleanupS1()
 	testutil.WaitForLeader(t, s1.RPC)
 
-	// replace consul ACLs api with a mock for tracking calls
+	// replace consul ACLs API with a mock for tracking calls in tests
 	var consulACLsAPI mockConsulACLsAPI
 	s1.consulACLs = &consulACLsAPI
 
@@ -691,29 +700,27 @@ func TestLeader_ClusterID_upgradePath(t *testing.T) {
 		cleanup func()
 	}
 
-	outdated := func(bootstrap bool) server {
+	outdated := func() server {
 		s, cleanup := TestServer(t, func(c *Config) {
 			c.NumSchedulers = 0
 			c.Build = before
-			c.DevDisableBootstrap = bootstrap
 			c.BootstrapExpect = 3
 			c.Logger.SetLevel(hclog.Trace)
 		})
 		return server{s: s, cleanup: cleanup}
 	}
 
-	upgraded := func(bootstrap bool) server {
+	upgraded := func() server {
 		s, cleanup := TestServer(t, func(c *Config) {
 			c.NumSchedulers = 0
 			c.Build = after
-			c.DevDisableBootstrap = bootstrap
-			c.BootstrapExpect = 3
+			c.BootstrapExpect = 0
 			c.Logger.SetLevel(hclog.Trace)
 		})
 		return server{s: s, cleanup: cleanup}
 	}
 
-	servers := []server{outdated(false), outdated(true), outdated(true)}
+	servers := []server{outdated(), outdated(), outdated()}
 	// fallback shutdown attempt in case testing fails
 	defer servers[0].cleanup()
 	defer servers[1].cleanup()
@@ -722,7 +729,7 @@ func TestLeader_ClusterID_upgradePath(t *testing.T) {
 	upgrade := func(i int) {
 		previous := servers[i]
 
-		servers[i] = upgraded(true)
+		servers[i] = upgraded()
 		TestJoin(t, servers[i].s, servers[(i+1)%3].s, servers[(i+2)%3].s)
 		testutil.WaitForLeader(t, servers[i].s.RPC)
 
@@ -809,7 +816,6 @@ func TestLeader_ClusterID_noUpgrade(t *testing.T) {
 		c.Logger.SetLevel(hclog.Trace)
 		c.NumSchedulers = 0
 		c.Build = minClusterIDVersion.String()
-		c.DevDisableBootstrap = true
 		c.BootstrapExpect = 3
 	})
 	defer cleanupS2()
@@ -817,7 +823,6 @@ func TestLeader_ClusterID_noUpgrade(t *testing.T) {
 		c.Logger.SetLevel(hclog.Trace)
 		c.NumSchedulers = 0
 		c.Build = minClusterIDVersion.String()
-		c.DevDisableBootstrap = true
 		c.BootstrapExpect = 3
 	})
 	defer cleanupS3()
@@ -1012,13 +1017,13 @@ func TestLeader_UpgradeRaftVersion(t *testing.T) {
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 1
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 2
 	})
 	defer cleanupS3()
@@ -1054,7 +1059,7 @@ func TestLeader_UpgradeRaftVersion(t *testing.T) {
 
 	// Replace the dead server with one running raft protocol v3
 	s4, cleanupS4 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.Datacenter = "dc1"
 		c.RaftConfig.ProtocolVersion = 3
 	})
@@ -1095,7 +1100,7 @@ func TestLeader_UpgradeRaftVersion(t *testing.T) {
 func TestLeader_Reelection(t *testing.T) {
 	raftProtocols := []int{1, 2, 3}
 	for _, p := range raftProtocols {
-		t.Run("Leader Election - Protocol version "+string(p), func(t *testing.T) {
+		t.Run(fmt.Sprintf("Leader Election - Protocol version %d", p), func(t *testing.T) {
 			leaderElectionTest(t, raft.ProtocolVersion(p))
 		})
 	}
@@ -1111,14 +1116,12 @@ func leaderElectionTest(t *testing.T, raftProtocol raft.ProtocolVersion) {
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
-		c.DevDisableBootstrap = true
 		c.RaftConfig.ProtocolVersion = raftProtocol
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
-		c.DevDisableBootstrap = true
 		c.RaftConfig.ProtocolVersion = raftProtocol
 	})
 	defer cleanupS3() // todo(shoenig) added this, should be here right??
@@ -1170,13 +1173,13 @@ func TestLeader_RollRaftServer(t *testing.T) {
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 2
 	})
 	defer cleanupS2()
 
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 2
 	})
 	defer cleanupS3()
@@ -1207,7 +1210,7 @@ func TestLeader_RollRaftServer(t *testing.T) {
 
 	// Replace the dead server with one running raft protocol v3
 	s4, cleanupS4 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 3
 	})
 	defer cleanupS4()
@@ -1230,7 +1233,7 @@ func TestLeader_RollRaftServer(t *testing.T) {
 	}
 	// Replace another dead server with one running raft protocol v3
 	s5, cleanupS5 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 3
 	})
 	defer cleanupS5()
@@ -1254,7 +1257,7 @@ func TestLeader_RollRaftServer(t *testing.T) {
 
 	// Replace the last dead server with one running raft protocol v3
 	s6, cleanupS6 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 3
 		c.RaftConfig.ProtocolVersion = 3
 	})
 	defer cleanupS6()
@@ -1322,6 +1325,37 @@ func TestLeader_TransitionsUpdateConsistencyRead(t *testing.T) {
 	require.True(t, s1.isReadyForConsistentReads())
 }
 
+// TestLeader_PausingWorkers asserts that scheduling workers are paused
+// (and unpaused) upon leader elections (and step downs).
+func TestLeader_PausingWorkers(t *testing.T) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
+		c.NumSchedulers = 12
+	})
+	defer cleanupS1()
+	testutil.WaitForLeader(t, s1.RPC)
+	require.Len(t, s1.workers, 12)
+
+	pausedWorkers := func() int {
+		c := 0
+		for _, w := range s1.workers {
+			w.pauseLock.Lock()
+			if w.paused {
+				c++
+			}
+			w.pauseLock.Unlock()
+		}
+		return c
+	}
+
+	// pause 3/4 of the workers
+	require.Equal(t, 9, pausedWorkers())
+
+	err := s1.revokeLeadership()
+	require.NoError(t, err)
+
+	require.Zero(t, pausedWorkers())
+}
+
 // Test doing an inplace upgrade on a server from raft protocol 2 to 3
 // This verifies that removing the server and adding it back with a uuid works
 // even if the server's address stays the same.
@@ -1330,24 +1364,29 @@ func TestServer_ReconcileMember(t *testing.T) {
 
 	// Create a three node cluster
 	s1, cleanupS1 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 2
 		c.RaftConfig.ProtocolVersion = 3
 	})
 	defer cleanupS1()
 
 	s2, cleanupS2 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 2
 		c.RaftConfig.ProtocolVersion = 3
 	})
 	defer cleanupS2()
 
+	TestJoin(t, s1, s2)
+	testutil.WaitForLeader(t, s1.RPC)
+
+	// test relies on s3 not being the leader, so adding it
+	// after leadership has been established to reduce
 	s3, cleanupS3 := TestServer(t, func(c *Config) {
-		c.DevDisableBootstrap = true
+		c.BootstrapExpect = 0
 		c.RaftConfig.ProtocolVersion = 2
 	})
 	defer cleanupS3()
-	TestJoin(t, s1, s2, s3)
-	testutil.WaitForLeader(t, s1.RPC)
+
+	TestJoin(t, s1, s3)
 
 	// Create a memberlist object for s3, with raft protocol upgraded to 3
 	upgradedS3Member := serf.Member{
@@ -1367,24 +1406,34 @@ func TestServer_ReconcileMember(t *testing.T) {
 	upgradedS3Member.Tags["mvn"] = "1"
 	upgradedS3Member.Tags["raft_vsn"] = "3"
 
-	// Find the leader so that we can call reconcile member on it
-	var leader *Server
-	for _, s := range []*Server{s1, s2, s3} {
-		if s.IsLeader() {
-			leader = s
+	findLeader := func(t *testing.T) *Server {
+		t.Helper()
+		for _, s := range []*Server{s1, s2, s3} {
+			if s.IsLeader() {
+				t.Logf("found leader: %v %v", s.config.NodeID, s.config.RPCAddr)
+				return s
+			}
 		}
+
+		t.Fatalf("no leader found")
+		return nil
 	}
-	leader.reconcileMember(upgradedS3Member)
+
+	// Find the leader so that we can call reconcile member on it
+	leader := findLeader(t)
+	if err := leader.reconcileMember(upgradedS3Member); err != nil {
+		t.Fatalf("failed to reconcile member: %v", err)
+	}
+
 	// This should remove s3 from the config and potentially cause a leader election
 	testutil.WaitForLeader(t, s1.RPC)
 
 	// Figure out the new leader and call reconcile again, this should add s3 with the new ID format
-	for _, s := range []*Server{s1, s2, s3} {
-		if s.IsLeader() {
-			leader = s
-		}
+	leader = findLeader(t)
+	if err := leader.reconcileMember(upgradedS3Member); err != nil {
+		t.Fatalf("failed to reconcile member: %v", err)
 	}
-	leader.reconcileMember(upgradedS3Member)
+
 	testutil.WaitForLeader(t, s1.RPC)
 	future := s2.raft.GetConfiguration()
 	if err := future.Error(); err != nil {
@@ -1404,7 +1453,7 @@ func TestServer_ReconcileMember(t *testing.T) {
 		t.Fatalf("got %d server addresses want %d", got, want)
 	}
 	if got, want := ids, 3; got != want {
-		t.Fatalf("got %d server ids want %d", got, want)
+		t.Fatalf("got %d server ids want %d: %#v", got, want, future.Configuration().Servers)
 	}
 }
 
