@@ -35,6 +35,7 @@ import (
 	"github.com/hashicorp/nomad/nomad/state"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/nomad/structs/config"
+	"github.com/hashicorp/nomad/nomad/volumewatcher"
 	"github.com/hashicorp/nomad/scheduler"
 	"github.com/hashicorp/raft"
 	raftboltdb "github.com/hashicorp/raft-boltdb"
@@ -101,13 +102,20 @@ type Server struct {
 
 	// The raft instance is used among Nomad nodes within the
 	// region to protect operations that require strong consistency
-	leaderCh      <-chan bool
 	raft          *raft.Raft
 	raftLayer     *RaftLayer
 	raftStore     *raftboltdb.BoltStore
 	raftInmem     *raft.InmemStore
 	raftTransport *raft.NetworkTransport
 
+	// reassertLeaderCh is used to signal that the leader loop must
+	// re-establish leadership.
+	//
+	// This might be relevant in snapshot restores, where leader in-memory
+	// state changed significantly such that leader state (e.g. periodic
+	// jobs, eval brokers) need to be recomputed.
+	reassertLeaderCh chan chan error
+
 	// autopilot is the Autopilot instance for this server.
 	autopilot *autopilot.Autopilot
 
@@ -187,6 +195,9 @@ type Server struct {
 	// nodeDrainer is used to drain allocations from nodes.
 	nodeDrainer *drainer.NodeDrainer
 
+	// volumeWatcher is used to release volume claims
+	volumeWatcher *volumewatcher.Watcher
+
 	// evalBroker is used to manage the in-progress evaluations
 	// that are waiting to be brokered to a sub-scheduler
 	evalBroker *EvalBroker
@@ -205,6 +216,9 @@ type Server struct {
 	// consulCatalog is used for discovering other Nomad Servers via Consul
 	consulCatalog consul.CatalogAPI
 
+	// consulConfigEntries is used for managing Consul Configuration Entries.
+	consulConfigEntries ConsulConfigsAPI
+
 	// consulACLs is used for managing Consul Service Identity tokens.
 	consulACLs ConsulACLsAPI
 
@@ -250,6 +264,8 @@ type endpoints struct {
 	Eval       *Eval
 	Plan       *Plan
 	Alloc      *Alloc
+	CSIVolume  *CSIVolume
+	CSIPlugin  *CSIPlugin
 	Deployment *Deployment
 	Region     *Region
 	Search     *Search
@@ -257,6 +273,7 @@ type endpoints struct {
 	System     *System
 	Operator   *Operator
 	ACL        *ACL
+	Scaling    *Scaling
 	Enterprise *EnterpriseEndpoints
 
 	// Client endpoints
@@ -264,11 +281,12 @@ type endpoints struct {
 	FileSystem        *FileSystem
 	Agent             *Agent
 	ClientAllocations *ClientAllocations
+	ClientCSI         *ClientCSI
 }
 
 // NewServer is used to construct a new Nomad server from the
 // configuration, potentially returning an error
-func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulACLs consul.ACLsAPI) (*Server, error) {
+func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulConfigEntries consul.ConfigAPI, consulACLs consul.ACLsAPI) (*Server, error) {
 	// Check the protocol version
 	if err := config.CheckVersion(); err != nil {
 		return nil, err
@@ -305,22 +323,23 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulACLs consu
 
 	// Create the server
 	s := &Server{
-		config:        config,
-		consulCatalog: consulCatalog,
-		connPool:      pool.NewPool(logger, serverRPCCache, serverMaxStreams, tlsWrap),
-		logger:        logger,
-		tlsWrap:       tlsWrap,
-		rpcServer:     rpc.NewServer(),
-		streamingRpcs: structs.NewStreamingRpcRegistry(),
-		nodeConns:     make(map[string][]*nodeConnState),
-		peers:         make(map[string][]*serverParts),
-		localPeers:    make(map[raft.ServerAddress]*serverParts),
-		reconcileCh:   make(chan serf.Member, 32),
-		eventCh:       make(chan serf.Event, 256),
-		evalBroker:    evalBroker,
-		blockedEvals:  NewBlockedEvals(evalBroker, logger),
-		rpcTLS:        incomingTLS,
-		aclCache:      aclCache,
+		config:           config,
+		consulCatalog:    consulCatalog,
+		connPool:         pool.NewPool(logger, serverRPCCache, serverMaxStreams, tlsWrap),
+		logger:           logger,
+		tlsWrap:          tlsWrap,
+		rpcServer:        rpc.NewServer(),
+		streamingRpcs:    structs.NewStreamingRpcRegistry(),
+		nodeConns:        make(map[string][]*nodeConnState),
+		peers:            make(map[string][]*serverParts),
+		localPeers:       make(map[raft.ServerAddress]*serverParts),
+		reassertLeaderCh: make(chan chan error),
+		reconcileCh:      make(chan serf.Member, 32),
+		eventCh:          make(chan serf.Event, 256),
+		evalBroker:       evalBroker,
+		blockedEvals:     NewBlockedEvals(evalBroker, logger),
+		rpcTLS:           incomingTLS,
+		aclCache:         aclCache,
 	}
 
 	s.shutdownCtx, s.shutdownCancel = context.WithCancel(context.Background())
@@ -346,7 +365,7 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulACLs consu
 	s.statsFetcher = NewStatsFetcher(s.logger, s.connPool, s.config.Region)
 
 	// Setup Consul (more)
-	s.setupConsul(consulACLs)
+	s.setupConsul(consulConfigEntries, consulACLs)
 
 	// Setup Vault
 	if err := s.setupVaultClient(); err != nil {
@@ -396,6 +415,12 @@ func NewServer(config *Config, consulCatalog consul.CatalogAPI, consulACLs consu
 		return nil, fmt.Errorf("failed to create deployment watcher: %v", err)
 	}
 
+	// Setup the volume watcher
+	if err := s.setupVolumeWatcher(); err != nil {
+		s.logger.Error("failed to create volume watcher", "error", err)
+		return nil, fmt.Errorf("failed to create volume watcher: %v", err)
+	}
+
 	// Setup the node drainer.
 	s.setupNodeDrainer()
 
@@ -640,6 +665,9 @@ func (s *Server) Shutdown() error {
 	// Stop the Consul ACLs token revocations
 	s.consulACLs.Stop()
 
+	// Stop being able to set Configuration Entries
+	s.consulConfigEntries.Stop()
+
 	return nil
 }
 
@@ -823,7 +851,7 @@ func (s *Server) setupBootstrapHandler() error {
 
 		// (ab)use serf.go's behavior of setting BootstrapExpect to
 		// zero if we have bootstrapped.  If we have bootstrapped
-		bootstrapExpect := atomic.LoadInt32(&s.config.BootstrapExpect)
+		bootstrapExpect := s.config.BootstrapExpect
 		if bootstrapExpect == 0 {
 			// This Nomad Server has been bootstrapped.  Rely on
 			// the peersTimeout firing as a guard to prevent
@@ -849,7 +877,7 @@ func (s *Server) setupBootstrapHandler() error {
 			// quorum has been reached, we do not need to poll
 			// Consul.  Let the normal timeout-based strategy
 			// take over.
-			if raftPeers >= int(bootstrapExpect) {
+			if raftPeers >= bootstrapExpect {
 				peersTimeout.Reset(peersPollInterval + lib.RandomStagger(peersPollInterval/peersPollJitterFactor))
 				return nil
 			}
@@ -983,9 +1011,21 @@ func (s *Server) setupDeploymentWatcher() error {
 
 	// Create the deployment watcher
 	s.deploymentWatcher = deploymentwatcher.NewDeploymentsWatcher(
-		s.logger, raftShim,
+		s.logger,
+		raftShim,
+		s.staticEndpoints.Deployment,
+		s.staticEndpoints.Job,
 		deploymentwatcher.LimitStateQueriesPerSecond,
-		deploymentwatcher.CrossDeploymentUpdateBatchDuration)
+		deploymentwatcher.CrossDeploymentUpdateBatchDuration,
+	)
+
+	return nil
+}
+
+// setupVolumeWatcher creates a volume watcher that sends CSI RPCs
+func (s *Server) setupVolumeWatcher() error {
+	s.volumeWatcher = volumewatcher.NewVolumesWatcher(
+		s.logger, s.staticEndpoints.CSIVolume, s.getLeaderAcl())
 
 	return nil
 }
@@ -1008,13 +1048,15 @@ func (s *Server) setupNodeDrainer() {
 }
 
 // setupConsul is used to setup Server specific consul components.
-func (s *Server) setupConsul(consulACLs consul.ACLsAPI) {
+func (s *Server) setupConsul(consulConfigEntries consul.ConfigAPI, consulACLs consul.ACLsAPI) {
+	s.consulConfigEntries = NewConsulConfigsAPI(consulConfigEntries, s.logger)
 	s.consulACLs = NewConsulACLsAPI(consulACLs, s.logger, s.purgeSITokenAccessors)
 }
 
 // setupVaultClient is used to set up the Vault API client.
 func (s *Server) setupVaultClient() error {
-	v, err := NewVaultClient(s.config.VaultConfig, s.logger, s.purgeVaultAccessors)
+	delegate := s.entVaultDelegate()
+	v, err := NewVaultClient(s.config.VaultConfig, s.logger, s.purgeVaultAccessors, delegate)
 	if err != nil {
 		return err
 	}
@@ -1093,11 +1135,16 @@ func (s *Server) setupRpcServer(server *rpc.Server, ctx *RPCContext) {
 		s.staticEndpoints.Eval = &Eval{srv: s, logger: s.logger.Named("eval")}
 		s.staticEndpoints.Job = NewJobEndpoints(s)
 		s.staticEndpoints.Node = &Node{srv: s, logger: s.logger.Named("client")} // Add but don't register
+		s.staticEndpoints.CSIVolume = &CSIVolume{srv: s, logger: s.logger.Named("csi_volume")}
+		s.staticEndpoints.CSIPlugin = &CSIPlugin{srv: s, logger: s.logger.Named("csi_plugin")}
 		s.staticEndpoints.Deployment = &Deployment{srv: s, logger: s.logger.Named("deployment")}
 		s.staticEndpoints.Operator = &Operator{srv: s, logger: s.logger.Named("operator")}
+		s.staticEndpoints.Operator.register()
+
 		s.staticEndpoints.Periodic = &Periodic{srv: s, logger: s.logger.Named("periodic")}
 		s.staticEndpoints.Plan = &Plan{srv: s, logger: s.logger.Named("plan")}
 		s.staticEndpoints.Region = &Region{srv: s, logger: s.logger.Named("region")}
+		s.staticEndpoints.Scaling = &Scaling{srv: s, logger: s.logger.Named("scaling")}
 		s.staticEndpoints.Status = &Status{srv: s, logger: s.logger.Named("status")}
 		s.staticEndpoints.System = &System{srv: s, logger: s.logger.Named("system")}
 		s.staticEndpoints.Search = &Search{srv: s, logger: s.logger.Named("search")}
@@ -1107,6 +1154,7 @@ func (s *Server) setupRpcServer(server *rpc.Server, ctx *RPCContext) {
 		s.staticEndpoints.ClientStats = &ClientStats{srv: s, logger: s.logger.Named("client_stats")}
 		s.staticEndpoints.ClientAllocations = &ClientAllocations{srv: s, logger: s.logger.Named("client_allocs")}
 		s.staticEndpoints.ClientAllocations.register()
+		s.staticEndpoints.ClientCSI = &ClientCSI{srv: s, logger: s.logger.Named("client_csi")}
 
 		// Streaming endpoints
 		s.staticEndpoints.FileSystem = &FileSystem{srv: s, logger: s.logger.Named("client_fs")}
@@ -1121,17 +1169,21 @@ func (s *Server) setupRpcServer(server *rpc.Server, ctx *RPCContext) {
 	server.Register(s.staticEndpoints.Alloc)
 	server.Register(s.staticEndpoints.Eval)
 	server.Register(s.staticEndpoints.Job)
+	server.Register(s.staticEndpoints.CSIVolume)
+	server.Register(s.staticEndpoints.CSIPlugin)
 	server.Register(s.staticEndpoints.Deployment)
 	server.Register(s.staticEndpoints.Operator)
 	server.Register(s.staticEndpoints.Periodic)
 	server.Register(s.staticEndpoints.Plan)
 	server.Register(s.staticEndpoints.Region)
+	server.Register(s.staticEndpoints.Scaling)
 	server.Register(s.staticEndpoints.Status)
 	server.Register(s.staticEndpoints.System)
 	server.Register(s.staticEndpoints.Search)
 	s.staticEndpoints.Enterprise.Register(server)
 	server.Register(s.staticEndpoints.ClientStats)
 	server.Register(s.staticEndpoints.ClientAllocations)
+	server.Register(s.staticEndpoints.ClientCSI)
 	server.Register(s.staticEndpoints.FileSystem)
 	server.Register(s.staticEndpoints.Agent)
 
@@ -1173,8 +1225,7 @@ func (s *Server) setupRaft() error {
 	s.raftTransport = trans
 
 	// Make sure we set the Logger.
-	logger := s.logger.StandardLoggerIntercept(&log.StandardLoggerOptions{InferLevels: true})
-	s.config.RaftConfig.Logger = logger
+	s.config.RaftConfig.Logger = s.logger.Named("raft")
 	s.config.RaftConfig.LogOutput = nil
 
 	// Our version of Raft protocol 2 requires the LocalID to match the network
@@ -1277,9 +1328,9 @@ func (s *Server) setupRaft() error {
 		}
 	}
 
-	// If we are in bootstrap or dev mode and the state is clean then we can
+	// If we are a single server cluster and the state is clean then we can
 	// bootstrap now.
-	if s.config.Bootstrap || s.config.DevMode {
+	if s.isSingleServerCluster() {
 		hasState, err := raft.HasExistingState(log, stable, snap)
 		if err != nil {
 			return err
@@ -1300,11 +1351,6 @@ func (s *Server) setupRaft() error {
 		}
 	}
 
-	// Setup the leader channel; that keeps the latest leadership alone
-	leaderCh := make(chan bool, 1)
-	s.config.RaftConfig.NotifyCh = leaderCh
-	s.leaderCh = dropButLastChannel(leaderCh, s.shutdownCh)
-
 	// Setup the Raft store
 	s.raft, err = raft.NewRaft(s.config.RaftConfig, s.fsm, log, stable, snap, trans)
 	if err != nil {
@@ -1327,10 +1373,10 @@ func (s *Server) setupSerf(conf *serf.Config, ch chan serf.Event, path string) (
 	conf.Tags["id"] = s.config.NodeID
 	conf.Tags["rpc_addr"] = s.clientRpcAdvertise.(*net.TCPAddr).IP.String()         // Address that clients will use to RPC to servers
 	conf.Tags["port"] = fmt.Sprintf("%d", s.serverRpcAdvertise.(*net.TCPAddr).Port) // Port servers use to RPC to one and another
-	if s.config.Bootstrap || (s.config.DevMode && !s.config.DevDisableBootstrap) {
+	if s.isSingleServerCluster() {
 		conf.Tags["bootstrap"] = "1"
 	}
-	bootstrapExpect := atomic.LoadInt32(&s.config.BootstrapExpect)
+	bootstrapExpect := s.config.BootstrapExpect
 	if bootstrapExpect != 0 {
 		conf.Tags["expect"] = fmt.Sprintf("%d", bootstrapExpect)
 	}
@@ -1531,7 +1577,7 @@ func (s *Server) Stats() map[string]map[string]string {
 			"server":        "true",
 			"leader":        fmt.Sprintf("%v", s.IsLeader()),
 			"leader_addr":   string(s.raft.Leader()),
-			"bootstrap":     fmt.Sprintf("%v", s.config.Bootstrap),
+			"bootstrap":     fmt.Sprintf("%v", s.isSingleServerCluster()),
 			"known_regions": toString(uint64(len(s.peers))),
 		},
 		"raft":    s.raft.Stats(),
@@ -1599,7 +1645,7 @@ func (s *Server) ClusterID() (string, error) {
 
 	// try to load the cluster ID from state store
 	fsmState := s.fsm.State()
-	existingMeta, err := fsmState.ClusterMetadata()
+	existingMeta, err := fsmState.ClusterMetadata(nil)
 	if err != nil {
 		s.logger.Named("core").Error("failed to get cluster ID", "error", err)
 		return "", err
@@ -1624,6 +1670,10 @@ func (s *Server) ClusterID() (string, error) {
 	return generatedID, nil
 }
 
+func (s *Server) isSingleServerCluster() bool {
+	return s.config.BootstrapExpect == 1
+}
+
 // peersInfoContent is used to help operators understand what happened to the
 // peers.json file. This is written to a file called peers.info in the same
 // location.
