@@ -5,6 +5,7 @@ import (
 	"sync"
 
 	memdb "github.com/hashicorp/go-memdb"
+
 	"github.com/hashicorp/nomad/nomad/structs"
 )
 
@@ -47,6 +48,10 @@ func init() {
 		autopilotConfigTableSchema,
 		schedulerConfigTableSchema,
 		clusterMetaTableSchema,
+		csiVolumeTableSchema,
+		csiPluginTableSchema,
+		scalingPolicyTableSchema,
+		scalingEventTableSchema,
 	}...)
 }
 
@@ -677,3 +682,201 @@ func clusterMetaTableSchema() *memdb.TableSchema {
 		},
 	}
 }
+
+// CSIVolumes are identified by id globally, and searchable by driver
+func csiVolumeTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: "csi_volumes",
+		Indexes: map[string]*memdb.IndexSchema{
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+				Indexer: &memdb.CompoundIndex{
+					Indexes: []memdb.Indexer{
+						&memdb.StringFieldIndex{
+							Field: "Namespace",
+						},
+						&memdb.StringFieldIndex{
+							Field: "ID",
+						},
+					},
+				},
+			},
+			"plugin_id": {
+				Name:         "plugin_id",
+				AllowMissing: false,
+				Unique:       false,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "PluginID",
+				},
+			},
+		},
+	}
+}
+
+// CSIPlugins are identified by id globally, and searchable by driver
+func csiPluginTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: "csi_plugins",
+		Indexes: map[string]*memdb.IndexSchema{
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+				Indexer: &memdb.StringFieldIndex{
+					Field: "ID",
+				},
+			},
+		},
+	}
+}
+
+// StringFieldIndex is used to extract a field from an object
+// using reflection and builds an index on that field.
+type ScalingPolicyTargetFieldIndex struct {
+	Field string
+}
+
+// FromObject is used to extract an index value from an
+// object or to indicate that the index value is missing.
+func (s *ScalingPolicyTargetFieldIndex) FromObject(obj interface{}) (bool, []byte, error) {
+	policy, ok := obj.(*structs.ScalingPolicy)
+	if !ok {
+		return false, nil, fmt.Errorf("object %#v is not a ScalingPolicy", obj)
+	}
+
+	if policy.Target == nil {
+		return false, nil, nil
+	}
+
+	val, ok := policy.Target[s.Field]
+	if !ok {
+		return false, nil, nil
+	}
+
+	// Add the null character as a terminator
+	val += "\x00"
+	return true, []byte(val), nil
+}
+
+// FromArgs is used to build an exact index lookup based on arguments
+func (s *ScalingPolicyTargetFieldIndex) FromArgs(args ...interface{}) ([]byte, error) {
+	if len(args) != 1 {
+		return nil, fmt.Errorf("must provide only a single argument")
+	}
+	arg, ok := args[0].(string)
+	if !ok {
+		return nil, fmt.Errorf("argument must be a string: %#v", args[0])
+	}
+	// Add the null character as a terminator
+	arg += "\x00"
+	return []byte(arg), nil
+}
+
+// PrefixFromArgs returns a prefix that should be used for scanning based on the arguments
+func (s *ScalingPolicyTargetFieldIndex) PrefixFromArgs(args ...interface{}) ([]byte, error) {
+	val, err := s.FromArgs(args...)
+	if err != nil {
+		return nil, err
+	}
+
+	// Strip the null terminator, the rest is a prefix
+	n := len(val)
+	if n > 0 {
+		return val[:n-1], nil
+	}
+	return val, nil
+}
+
+// scalingPolicyTableSchema returns the MemDB schema for the policy table.
+func scalingPolicyTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: "scaling_policy",
+		Indexes: map[string]*memdb.IndexSchema{
+			// Primary index is used for simple direct lookup.
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+
+				// UUID is uniquely identifying
+				Indexer: &memdb.StringFieldIndex{
+					Field: "ID",
+				},
+			},
+			// Target index is used for listing by namespace or job, or looking up a specific target.
+			// A given task group can have only a single scaling policies, so this is guaranteed to be unique.
+			"target": {
+				Name:         "target",
+				AllowMissing: false,
+				Unique:       true,
+
+				// Use a compound index so the tuple of (Namespace, Job, Group) is
+				// uniquely identifying
+				Indexer: &memdb.CompoundIndex{
+					Indexes: []memdb.Indexer{
+						&ScalingPolicyTargetFieldIndex{
+							Field: "Namespace",
+						},
+
+						&ScalingPolicyTargetFieldIndex{
+							Field: "Job",
+						},
+
+						&ScalingPolicyTargetFieldIndex{
+							Field: "Group",
+						},
+					},
+				},
+			},
+			// Used to filter by enabled
+			"enabled": {
+				Name:         "enabled",
+				AllowMissing: false,
+				Unique:       false,
+				Indexer: &memdb.FieldSetIndex{
+					Field: "Enabled",
+				},
+			},
+		},
+	}
+}
+
+// scalingEventTableSchema returns the memdb schema for job scaling events
+func scalingEventTableSchema() *memdb.TableSchema {
+	return &memdb.TableSchema{
+		Name: "scaling_event",
+		Indexes: map[string]*memdb.IndexSchema{
+			"id": {
+				Name:         "id",
+				AllowMissing: false,
+				Unique:       true,
+
+				// Use a compound index so the tuple of (Namespace, JobID) is
+				// uniquely identifying
+				Indexer: &memdb.CompoundIndex{
+					Indexes: []memdb.Indexer{
+						&memdb.StringFieldIndex{
+							Field: "Namespace",
+						},
+
+						&memdb.StringFieldIndex{
+							Field: "JobID",
+						},
+					},
+				},
+			},
+
+			// TODO: need to figure out whether we want to index these or the jobs or ...
+			// "error": {
+			// 	Name:         "error",
+			// 	AllowMissing: false,
+			// 	Unique:       false,
+			// 	Indexer: &memdb.FieldSetIndex{
+			// 		Field: "Error",
+			// 	},
+			// },
+		},
+	}
+}
