@@ -1,4 +1,4 @@
-// +build !pro,!ent
+// +build !ent
 
 package state
 
@@ -7,6 +7,12 @@ import (
 	"github.com/hashicorp/nomad/nomad/structs"
 )
 
+// enterpriseInit is used to initialize the state store with enterprise
+// objects.
+func (s *StateStore) enterpriseInit() error {
+	return nil
+}
+
 // namespaceExists returns whether a namespace exists
 func (s *StateStore) namespaceExists(txn *memdb.Txn, namespace string) (bool, error) {
 	return namespace == structs.DefaultNamespace, nil
@@ -17,3 +23,7 @@ func (s *StateStore) namespaceExists(txn *memdb.Txn, namespace string) (bool, er
 func (s *StateStore) updateEntWithAlloc(index uint64, new, existing *structs.Allocation, txn *memdb.Txn) error {
 	return nil
 }
+
+func (s *StateStore) NamespaceNames() ([]string, error) {
+	return []string{structs.DefaultNamespace}, nil
+}
