@@ -26,6 +26,10 @@ type AutopilotConfig struct {
 	// be behind before being considered unhealthy.
 	MaxTrailingLogs int `hcl:"max_trailing_logs"`
 
+	// MinQuorum sets the minimum number of servers required in a cluster
+	// before autopilot can prune dead servers.
+	MinQuorum int `hcl:"min_quorum"`
+
 	// (Enterprise-only) EnableRedundancyZones specifies whether to enable redundancy zones.
 	EnableRedundancyZones *bool `hcl:"enable_redundancy_zones"`
 
@@ -73,6 +77,9 @@ func (a *AutopilotConfig) Merge(b *AutopilotConfig) *AutopilotConfig {
 	if b.MaxTrailingLogs != 0 {
 		result.MaxTrailingLogs = b.MaxTrailingLogs
 	}
+	if b.MinQuorum != 0 {
+		result.MinQuorum = b.MinQuorum
+	}
 	if b.EnableRedundancyZones != nil {
 		result.EnableRedundancyZones = b.EnableRedundancyZones
 	}
