@@ -108,7 +108,7 @@ func TestAllocsFit_PortsOvercommitted_Old(t *testing.T) {
 						Device:        "eth0",
 						IP:            "10.0.0.1",
 						MBits:         50,
-						ReservedPorts: []Port{{"main", 8000, 80}},
+						ReservedPorts: []Port{{"main", 8000, 80, ""}},
 					},
 				},
 			},
@@ -160,7 +160,7 @@ func TestAllocsFit_Old(t *testing.T) {
 					Device:        "eth0",
 					IP:            "10.0.0.1",
 					MBits:         50,
-					ReservedPorts: []Port{{"main", 80, 0}},
+					ReservedPorts: []Port{{"main", 80, 0, ""}},
 				},
 			},
 		},
@@ -176,7 +176,7 @@ func TestAllocsFit_Old(t *testing.T) {
 					Device:        "eth0",
 					IP:            "10.0.0.1",
 					MBits:         50,
-					ReservedPorts: []Port{{"main", 8000, 80}},
+					ReservedPorts: []Port{{"main", 8000, 80, ""}},
 				},
 			},
 		},
@@ -188,8 +188,8 @@ func TestAllocsFit_Old(t *testing.T) {
 	require.True(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 
 	// Should not fit second allocation
 	fit, _, used, err = AllocsFit(n, []*Allocation{a1, a1}, nil, false)
@@ -197,8 +197,8 @@ func TestAllocsFit_Old(t *testing.T) {
 	require.False(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(3000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(3072, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
 }
 
 // COMPAT(0.11): Remove in 0.11
@@ -227,7 +227,7 @@ func TestAllocsFit_TerminalAlloc_Old(t *testing.T) {
 					Device:        "eth0",
 					IP:            "10.0.0.1",
 					MBits:         50,
-					ReservedPorts: []Port{{"main", 80, 0}},
+					ReservedPorts: []Port{{"main", 80, 0, ""}},
 				},
 			},
 		},
@@ -243,7 +243,7 @@ func TestAllocsFit_TerminalAlloc_Old(t *testing.T) {
 					Device:        "eth0",
 					IP:            "10.0.0.1",
 					MBits:         50,
-					ReservedPorts: []Port{{"main", 8000, 0}},
+					ReservedPorts: []Port{{"main", 8000, 0, ""}},
 				},
 			},
 		},
@@ -255,8 +255,8 @@ func TestAllocsFit_TerminalAlloc_Old(t *testing.T) {
 	require.True(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 
 	// Should fit second allocation since it is terminal
 	a2 := a1.Copy()
@@ -266,8 +266,8 @@ func TestAllocsFit_TerminalAlloc_Old(t *testing.T) {
 	require.True(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 }
 
 func TestAllocsFit(t *testing.T) {
@@ -291,6 +291,17 @@ func TestAllocsFit(t *testing.T) {
 					MBits:  100,
 				},
 			},
+			NodeNetworks: []*NodeNetworkResource{
+				{
+					Mode:   "host",
+					Device: "eth0",
+					Addresses: []NodeNetworkAddress{
+						{
+							Address: "10.0.0.1",
+						},
+					},
+				},
+			},
 		},
 		ReservedResources: &NodeReservedResources{
 			Cpu: NodeReservedCpuResources{
@@ -318,18 +329,24 @@ func TestAllocsFit(t *testing.T) {
 					Memory: AllocatedMemoryResources{
 						MemoryMB: 1024,
 					},
-					Networks: []*NetworkResource{
-						{
-							Device:        "eth0",
-							IP:            "10.0.0.1",
-							MBits:         50,
-							ReservedPorts: []Port{{"main", 8000, 0}},
-						},
-					},
 				},
 			},
 			Shared: AllocatedSharedResources{
 				DiskMB: 5000,
+				Networks: Networks{
+					{
+						Mode: "host",
+						IP: "10.0.0.1",
+						ReservedPorts: []Port{{"main", 8000, 0, ""}},
+					},
+				},
+				Ports: AllocatedPorts{
+					{
+						Label:  "main",
+						Value:  8000,
+						HostIP: "10.0.0.1",
+					},
+				},
 			},
 		},
 	}
@@ -340,8 +357,8 @@ func TestAllocsFit(t *testing.T) {
 	require.True(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 
 	// Should not fit second allocation
 	fit, _, used, err = AllocsFit(n, []*Allocation{a1, a1}, nil, false)
@@ -349,8 +366,8 @@ func TestAllocsFit(t *testing.T) {
 	require.False(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(3000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(3072, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
 }
 
 func TestAllocsFit_TerminalAlloc(t *testing.T) {
@@ -407,7 +424,7 @@ func TestAllocsFit_TerminalAlloc(t *testing.T) {
 							Device:        "eth0",
 							IP:            "10.0.0.1",
 							MBits:         50,
-							ReservedPorts: []Port{{"main", 8000, 80}},
+							ReservedPorts: []Port{{"main", 8000, 80, ""}},
 						},
 					},
 				},
@@ -424,8 +441,8 @@ func TestAllocsFit_TerminalAlloc(t *testing.T) {
 	require.True(fit)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 
 	// Should fit second allocation since it is terminal
 	a2 := a1.Copy()
@@ -435,8 +452,8 @@ func TestAllocsFit_TerminalAlloc(t *testing.T) {
 	require.True(fit, dim)
 
 	// Sanity check the used resources
-	require.EqualValues(2000, used.Flattened.Cpu.CpuShares)
-	require.EqualValues(2048, used.Flattened.Memory.MemoryMB)
+	require.EqualValues(1000, used.Flattened.Cpu.CpuShares)
+	require.EqualValues(1024, used.Flattened.Memory.MemoryMB)
 }
 
 // Tests that AllocsFit detects device collisions
@@ -506,7 +523,7 @@ func TestAllocsFit_Devices(t *testing.T) {
 }
 
 // COMPAT(0.11): Remove in 0.11
-func TestScoreFit_Old(t *testing.T) {
+func TestScoreFitBinPack_Old(t *testing.T) {
 	node := &Node{}
 	node.Resources = &Resources{
 		CPU:      4096,
@@ -528,7 +545,7 @@ func TestScoreFit_Old(t *testing.T) {
 			},
 		},
 	}
-	score := ScoreFit(node, util)
+	score := ScoreFitBinPack(node, util)
 	if score != 18.0 {
 		t.Fatalf("bad: %v", score)
 	}
@@ -544,7 +561,7 @@ func TestScoreFit_Old(t *testing.T) {
 			},
 		},
 	}
-	score = ScoreFit(node, util)
+	score = ScoreFitBinPack(node, util)
 	if score != 0.0 {
 		t.Fatalf("bad: %v", score)
 	}
@@ -560,13 +577,13 @@ func TestScoreFit_Old(t *testing.T) {
 			},
 		},
 	}
-	score = ScoreFit(node, util)
+	score = ScoreFitBinPack(node, util)
 	if score < 10.0 || score > 16.0 {
 		t.Fatalf("bad: %v", score)
 	}
 }
 
-func TestScoreFit(t *testing.T) {
+func TestScoreFitBinPack(t *testing.T) {
 	node := &Node{}
 	node.NodeResources = &NodeResources{
 		Cpu: NodeCpuResources{
@@ -585,52 +602,53 @@ func TestScoreFit(t *testing.T) {
 		},
 	}
 
-	// Test a perfect fit
-	util := &ComparableResources{
-		Flattened: AllocatedTaskResources{
-			Cpu: AllocatedCpuResources{
-				CpuShares: 2048,
-			},
-			Memory: AllocatedMemoryResources{
-				MemoryMB: 4096,
+	cases := []struct {
+		name         string
+		flattened    AllocatedTaskResources
+		binPackScore float64
+		spreadScore  float64
+	}{
+		{
+			name: "almost filled node, but with just enough hole",
+			flattened: AllocatedTaskResources{
+				Cpu:    AllocatedCpuResources{CpuShares: 2048},
+				Memory: AllocatedMemoryResources{MemoryMB: 4096},
 			},
+			binPackScore: 18,
+			spreadScore:  0,
+		},
+		{
+			name: "unutilized node",
+			flattened: AllocatedTaskResources{
+				Cpu:    AllocatedCpuResources{CpuShares: 0},
+				Memory: AllocatedMemoryResources{MemoryMB: 0},
+			},
+			binPackScore: 0,
+			spreadScore:  18,
+		},
+		{
+			name: "mid-case scnario",
+			flattened: AllocatedTaskResources{
+				Cpu:    AllocatedCpuResources{CpuShares: 1024},
+				Memory: AllocatedMemoryResources{MemoryMB: 2048},
+			},
+			binPackScore: 13.675,
+			spreadScore:  4.325,
 		},
-	}
-	score := ScoreFit(node, util)
-	if score != 18.0 {
-		t.Fatalf("bad: %v", score)
 	}
 
-	// Test the worst fit
-	util = &ComparableResources{
-		Flattened: AllocatedTaskResources{
-			Cpu: AllocatedCpuResources{
-				CpuShares: 0,
-			},
-			Memory: AllocatedMemoryResources{
-				MemoryMB: 0,
-			},
-		},
-	}
-	score = ScoreFit(node, util)
-	if score != 0.0 {
-		t.Fatalf("bad: %v", score)
-	}
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			util := &ComparableResources{Flattened: c.flattened}
 
-	// Test a mid-case scenario
-	util = &ComparableResources{
-		Flattened: AllocatedTaskResources{
-			Cpu: AllocatedCpuResources{
-				CpuShares: 1024,
-			},
-			Memory: AllocatedMemoryResources{
-				MemoryMB: 2048,
-			},
-		},
-	}
-	score = ScoreFit(node, util)
-	if score < 10.0 || score > 16.0 {
-		t.Fatalf("bad: %v", score)
+			binPackScore := ScoreFitBinPack(node, util)
+			require.InDelta(t, c.binPackScore, binPackScore, 0.001, "binpack score")
+
+			spreadScore := ScoreFitSpread(node, util)
+			require.InDelta(t, c.spreadScore, spreadScore, 0.001, "spread score")
+
+			require.InDelta(t, 18, binPackScore+spreadScore, 0.001, "score sum")
+		})
 	}
 }
 
