@@ -1,15 +1,16 @@
 package nomad
 
 import (
+	"context"
 	"fmt"
-	"log"
 	"strings"
 	"sync"
 	"time"
 
-	"github.com/armon/go-metrics"
-
+	metrics "github.com/armon/go-metrics"
+	log "github.com/hashicorp/go-hclog"
 	memdb "github.com/hashicorp/go-memdb"
+	"github.com/hashicorp/nomad/nomad/state"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/scheduler"
 )
@@ -22,9 +23,6 @@ const (
 	// but that is much slower than backoffBaselineFast
 	backoffBaselineSlow = 500 * time.Millisecond
 
-	// backoffLimitFast is the limit of the exponential backoff
-	backoffLimitFast = time.Second
-
 	// backoffLimitSlow is the limit of the exponential backoff for
 	// the slower backoff
 	backoffLimitSlow = 10 * time.Second
@@ -55,7 +53,7 @@ const (
 // of the scheduler with the plumbing required to make it all work.
 type Worker struct {
 	srv    *Server
-	logger *log.Logger
+	logger log.Logger
 	start  time.Time
 
 	paused    bool
@@ -76,7 +74,7 @@ type Worker struct {
 func NewWorker(srv *Server) (*Worker, error) {
 	w := &Worker{
 		srv:    srv,
-		logger: srv.logger,
+		logger: srv.logger.ResetNamed("worker"),
 		start:  time.Now(),
 	}
 	w.pauseCond = sync.NewCond(&w.pauseLock)
@@ -114,21 +112,22 @@ func (w *Worker) run() {
 
 		// Check for a shutdown
 		if w.srv.IsShutdown() {
-			w.logger.Printf("[ERR] worker: nacking eval %q because the server is shutting down", eval.ID)
+			w.logger.Error("nacking eval because the server is shutting down", "eval", log.Fmt("%#v", eval))
 			w.sendAck(eval.ID, token, false)
 			return
 		}
 
 		// Wait for the raft log to catchup to the evaluation
-		if err := w.waitForIndex(waitIndex, raftSyncLimit); err != nil {
-			w.logger.Printf("[ERR] worker: error waiting for Raft index for eval %q: %v", eval.ID, err)
+		snap, err := w.snapshotMinIndex(waitIndex, raftSyncLimit)
+		if err != nil {
+			w.logger.Error("error waiting for Raft index", "error", err, "index", waitIndex)
 			w.sendAck(eval.ID, token, false)
 			continue
 		}
 
 		// Invoke the scheduler to determine placements
-		if err := w.invokeScheduler(eval, token); err != nil {
-			w.logger.Printf("[ERR] worker: error invoking the scheduler for eval %q: %v", eval.ID, err)
+		if err := w.invokeScheduler(snap, eval, token); err != nil {
+			w.logger.Error("error invoking scheduler", "error", err)
 			w.sendAck(eval.ID, token, false)
 			continue
 		}
@@ -163,7 +162,7 @@ REQ:
 	metrics.MeasureSince([]string{"nomad", "worker", "dequeue_eval"}, start)
 	if err != nil {
 		if time.Since(w.start) > dequeueErrGrace && !w.srv.IsShutdown() {
-			w.logger.Printf("[ERR] worker: failed to dequeue evaluation: %v", err)
+			w.logger.Error("failed to dequeue evaluation", "error", err)
 		}
 
 		// Adjust the backoff based on the error. If it is a scheduler version
@@ -183,7 +182,7 @@ REQ:
 
 	// Check if we got a response
 	if resp.Eval != nil {
-		w.logger.Printf("[DEBUG] worker: dequeued evaluation %s", resp.Eval.ID)
+		w.logger.Debug("dequeued evaluation", "eval_id", resp.Eval.ID)
 		return resp.Eval, resp.Token, resp.GetWaitIndex(), false
 	}
 
@@ -219,69 +218,42 @@ func (w *Worker) sendAck(evalID, token string, ack bool) {
 	// Make the RPC call
 	err := w.srv.RPC(endpoint, &req, &resp)
 	if err != nil {
-		w.logger.Printf("[ERR] worker: failed to %s evaluation '%s': %v",
-			verb, evalID, err)
+		w.logger.Error(fmt.Sprintf("failed to %s evaluation", verb), "eval_id", evalID, "error", err)
 	} else {
-		w.logger.Printf("[DEBUG] worker: %s for evaluation %s", verb, evalID)
+		w.logger.Debug(fmt.Sprintf("%s evaluation", verb), "eval_id", evalID)
 	}
 }
 
-// waitForIndex ensures that the local state is at least as fresh
-// as the given index. This is used before starting an evaluation,
-// but also potentially mid-stream. If a Plan fails because of stale
-// state (attempt to allocate to a failed/dead node), we may need
-// to sync our state again and do the planning with more recent data.
-func (w *Worker) waitForIndex(index uint64, timeout time.Duration) error {
-	// XXX: Potential optimization is to set up a watch on the state stores
-	// index table and only unblock via a trigger rather than timing out and
-	// checking.
-
+// snapshotMinIndex times calls to StateStore.SnapshotAfter which may block.
+func (w *Worker) snapshotMinIndex(waitIndex uint64, timeout time.Duration) (*state.StateSnapshot, error) {
 	start := time.Now()
-	defer metrics.MeasureSince([]string{"nomad", "worker", "wait_for_index"}, start)
-CHECK:
-	// Get the states current index
-	snapshotIndex, err := w.srv.fsm.State().LatestIndex()
-	if err != nil {
-		return fmt.Errorf("failed to determine state store's index: %v", err)
+	ctx, cancel := context.WithTimeout(w.srv.shutdownCtx, timeout)
+	snap, err := w.srv.fsm.State().SnapshotMinIndex(ctx, waitIndex)
+	cancel()
+	metrics.MeasureSince([]string{"nomad", "worker", "wait_for_index"}, start)
+
+	// Wrap error to ensure callers don't disregard timeouts.
+	if err == context.DeadlineExceeded {
+		err = fmt.Errorf("timed out after %s waiting for index=%d", timeout, waitIndex)
 	}
 
-	// We only need the FSM state to be as recent as the given index
-	if index <= snapshotIndex {
-		w.backoffReset()
-		return nil
-	}
-
-	// Check if we've reached our limit
-	if time.Now().Sub(start) > timeout {
-		return fmt.Errorf("sync wait timeout reached")
-	}
-
-	// Exponential back off if we haven't yet reached it
-	if w.backoffErr(backoffBaselineFast, backoffLimitFast) {
-		return fmt.Errorf("shutdown while waiting for state sync")
-	}
-	goto CHECK
+	return snap, err
 }
 
 // invokeScheduler is used to invoke the business logic of the scheduler
-func (w *Worker) invokeScheduler(eval *structs.Evaluation, token string) error {
+func (w *Worker) invokeScheduler(snap *state.StateSnapshot, eval *structs.Evaluation, token string) error {
 	defer metrics.MeasureSince([]string{"nomad", "worker", "invoke_scheduler", eval.Type}, time.Now())
 	// Store the evaluation token
 	w.evalToken = token
 
-	// Snapshot the current state
-	snap, err := w.srv.fsm.State().Snapshot()
-	if err != nil {
-		return fmt.Errorf("failed to snapshot state: %v", err)
-	}
-
 	// Store the snapshot's index
+	var err error
 	w.snapshotIndex, err = snap.LatestIndex()
 	if err != nil {
 		return fmt.Errorf("failed to determine snapshot's index: %v", err)
 	}
 
-	// Create the scheduler, or use the special system scheduler
+	// Create the scheduler, or use the special core scheduler
 	var sched scheduler.Scheduler
 	if eval.Type == structs.JobTypeCore {
 		sched = NewCoreScheduler(w.srv, snap)
@@ -312,6 +284,16 @@ func (w *Worker) SubmitPlan(plan *structs.Plan) (*structs.PlanResult, scheduler.
 	// Add the evaluation token to the plan
 	plan.EvalToken = w.evalToken
 
+	// Add SnapshotIndex to ensure leader's StateStore processes the Plan
+	// at or after the index it was created.
+	plan.SnapshotIndex = w.snapshotIndex
+
+	// Normalize stopped and preempted allocs before RPC
+	normalizePlan := ServersMeetMinimumVersion(w.srv.Members(), MinVersionPlanNormalization, true)
+	if normalizePlan {
+		plan.NormalizeAllocations()
+	}
+
 	// Setup the request
 	req := structs.PlanRequest{
 		Plan: plan,
@@ -324,14 +306,13 @@ func (w *Worker) SubmitPlan(plan *structs.Plan) (*structs.PlanResult, scheduler.
 SUBMIT:
 	// Make the RPC call
 	if err := w.srv.RPC("Plan.Submit", &req, &resp); err != nil {
-		w.logger.Printf("[ERR] worker: failed to submit plan for evaluation %s: %v",
-			plan.EvalID, err)
+		w.logger.Error("failed to submit plan for evaluation", "eval_id", plan.EvalID, "error", err)
 		if w.shouldResubmit(err) && !w.backoffErr(backoffBaselineSlow, backoffLimitSlow) {
 			goto SUBMIT
 		}
 		return nil, nil, err
 	} else {
-		w.logger.Printf("[DEBUG] worker: submitted plan at index %d for evaluation %s", resp.Index, plan.EvalID)
+		w.logger.Debug("submitted plan for evaluation", "eval_id", plan.EvalID)
 		w.backoffReset()
 	}
 
@@ -342,23 +323,19 @@ SUBMIT:
 	}
 
 	// Check if a state update is required. This could be required if we
-	// planning based on stale data, which is causing issues. For example, a
+	// planned based on stale data, which is causing issues. For example, a
 	// node failure since the time we've started planning or conflicting task
 	// allocations.
 	var state scheduler.State
 	if result.RefreshIndex != 0 {
 		// Wait for the raft log to catchup to the evaluation
-		w.logger.Printf("[DEBUG] worker: refreshing state to index %d for %q", result.RefreshIndex, plan.EvalID)
-		if err := w.waitForIndex(result.RefreshIndex, raftSyncLimit); err != nil {
+		w.logger.Debug("refreshing state", "refresh_index", result.RefreshIndex, "eval_id", plan.EvalID)
+
+		var err error
+		state, err = w.snapshotMinIndex(result.RefreshIndex, raftSyncLimit)
+		if err != nil {
 			return nil, nil, err
 		}
-
-		// Snapshot the current state
-		snap, err := w.srv.fsm.State().Snapshot()
-		if err != nil {
-			return nil, nil, fmt.Errorf("failed to snapshot state: %v", err)
-		}
-		state = snap
 	}
 
 	// Return the result and potential state update
@@ -376,6 +353,7 @@ func (w *Worker) UpdateEval(eval *structs.Evaluation) error {
 
 	// Store the snapshot index in the eval
 	eval.SnapshotIndex = w.snapshotIndex
+	eval.UpdateModifyTime()
 
 	// Setup the request
 	req := structs.EvalUpdateRequest{
@@ -390,14 +368,13 @@ func (w *Worker) UpdateEval(eval *structs.Evaluation) error {
 SUBMIT:
 	// Make the RPC call
 	if err := w.srv.RPC("Eval.Update", &req, &resp); err != nil {
-		w.logger.Printf("[ERR] worker: failed to update evaluation %#v: %v",
-			eval, err)
+		w.logger.Error("failed to update evaluation", "eval", log.Fmt("%#v", eval), "error", err)
 		if w.shouldResubmit(err) && !w.backoffErr(backoffBaselineSlow, backoffLimitSlow) {
 			goto SUBMIT
 		}
 		return err
 	} else {
-		w.logger.Printf("[DEBUG] worker: updated evaluation %#v", eval)
+		w.logger.Debug("updated evaluation", "eval", log.Fmt("%#v", eval))
 		w.backoffReset()
 	}
 	return nil
@@ -415,6 +392,10 @@ func (w *Worker) CreateEval(eval *structs.Evaluation) error {
 	// Store the snapshot index in the eval
 	eval.SnapshotIndex = w.snapshotIndex
 
+	now := time.Now().UTC().UnixNano()
+	eval.CreateTime = now
+	eval.ModifyTime = now
+
 	// Setup the request
 	req := structs.EvalUpdateRequest{
 		Evals:     []*structs.Evaluation{eval},
@@ -428,14 +409,13 @@ func (w *Worker) CreateEval(eval *structs.Evaluation) error {
 SUBMIT:
 	// Make the RPC call
 	if err := w.srv.RPC("Eval.Create", &req, &resp); err != nil {
-		w.logger.Printf("[ERR] worker: failed to create evaluation %#v: %v",
-			eval, err)
+		w.logger.Error("failed to create evaluation", "eval", log.Fmt("%#v", eval), "error", err)
 		if w.shouldResubmit(err) && !w.backoffErr(backoffBaselineSlow, backoffLimitSlow) {
 			goto SUBMIT
 		}
 		return err
 	} else {
-		w.logger.Printf("[DEBUG] worker: created evaluation %#v", eval)
+		w.logger.Debug("created evaluation", "eval", log.Fmt("%#v", eval))
 		w.backoffReset()
 	}
 	return nil
@@ -476,6 +456,7 @@ func (w *Worker) ReblockEval(eval *structs.Evaluation) error {
 
 	// Store the snapshot index in the eval
 	eval.SnapshotIndex = w.snapshotIndex
+	eval.UpdateModifyTime()
 
 	// Setup the request
 	req := structs.EvalUpdateRequest{
@@ -490,14 +471,13 @@ func (w *Worker) ReblockEval(eval *structs.Evaluation) error {
 SUBMIT:
 	// Make the RPC call
 	if err := w.srv.RPC("Eval.Reblock", &req, &resp); err != nil {
-		w.logger.Printf("[ERR] worker: failed to reblock evaluation %#v: %v",
-			eval, err)
+		w.logger.Error("failed to reblock evaluation", "eval", log.Fmt("%#v", eval), "error", err)
 		if w.shouldResubmit(err) && !w.backoffErr(backoffBaselineSlow, backoffLimitSlow) {
 			goto SUBMIT
 		}
 		return err
 	} else {
-		w.logger.Printf("[DEBUG] worker: reblocked evaluation %#v", eval)
+		w.logger.Debug("reblocked evaluation", "eval", log.Fmt("%#v", eval))
 		w.backoffReset()
 	}
 	return nil
