@@ -2,7 +2,7 @@
 
 set -o errexit
 
-VERSION="1.6.1"
+VERSION="1.6.4"
 DOWNLOAD=https://releases.hashicorp.com/consul/${VERSION}/consul_${VERSION}_linux_amd64.zip
 
 function install_consul() {
@@ -12,7 +12,7 @@ function install_consul() {
 		fi
 	fi
 
-	wget -q -O /tmp/consul.zip ${DOWNLOAD}
+	curl -sSL --fail -o /tmp/consul.zip ${DOWNLOAD}
 
 	unzip -d /tmp /tmp/consul.zip
 	mv /tmp/consul /usr/bin/consul
