@@ -14,13 +14,19 @@ HOME_DIR=ubuntu
 # Wait for network
 sleep 15
 
-IP_ADDRESS=$(curl http://instance-data/latest/meta-data/local-ipv4)
-# IP_ADDRESS="$(/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}')"
 DOCKER_BRIDGE_IP_ADDRESS=(`ifconfig docker0 2>/dev/null|awk '/inet addr:/ {print $2}'|sed 's/addr://'`)
 CLOUD=$1
 RETRY_JOIN=$2
 NOMAD_BINARY=$3
 
+# Get IP from metadata service
+if [ "$CLOUD" = "gce" ]; then
+  IP_ADDRESS=$(curl -H "Metadata-Flavor: Google" http://metadata/computeMetadata/v1/instance/network-interfaces/0/ip)
+else
+  IP_ADDRESS=$(curl http://instance-data/latest/meta-data/local-ipv4)
+fi
+# IP_ADDRESS="$(/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}')"
+
 # Consul
 sed -i "s/IP_ADDRESS/$IP_ADDRESS/g" $CONFIGDIR/consul_client.json
 sed -i "s/RETRY_JOIN/$RETRY_JOIN/g" $CONFIGDIR/consul_client.json
