@@ -1,9 +1,10 @@
 import ApplicationAdapter from './application';
 
-export default ApplicationAdapter.extend({
-  pathForType: () => 'agent/members',
+export default class AgentAdapter extends ApplicationAdapter {
+  pathForType = () => 'agent/members';
+
   urlForFindRecord() {
     const [, ...args] = arguments;
     return this.urlForFindAll(...args);
-  },
-});
+  }
+}
