@@ -4,33 +4,38 @@ import RESTAdapter from 'ember-data/adapters/rest';
 import codesForError from '../utils/codes-for-error';
 import removeRecord from '../utils/remove-record';
 import { default as NoLeaderError, NO_LEADER } from '../utils/no-leader-error';
+import classic from 'ember-classic-decorator';
 
 export const namespace = 'v1';
 
-export default RESTAdapter.extend({
-  namespace,
+@classic
+export default class ApplicationAdapter extends RESTAdapter {
+  namespace = namespace;
 
-  system: service(),
-  token: service(),
+  @service system;
+  @service token;
 
-  headers: computed('token.secret', function() {
+  @computed('token.secret')
+  get headers() {
     const token = this.get('token.secret');
     if (token) {
       return {
         'X-Nomad-Token': token,
       };
     }
-  }),
+
+    return undefined;
+  }
 
   handleResponse(status, headers, payload) {
     if (status === 500 && payload === NO_LEADER) {
       return new NoLeaderError();
     }
-    return this._super(...arguments);
-  },
+    return super.handleResponse(...arguments);
+  }
 
   findAll() {
-    return this._super(...arguments).catch(error => {
+    return super.findAll(...arguments).catch(error => {
       const errorCodes = codesForError(error);
 
       const isNotImplemented = errorCodes.includes('501');
@@ -42,23 +47,26 @@ export default RESTAdapter.extend({
       // Rethrow to be handled downstream
       throw error;
     });
-  },
+  }
 
-  ajaxOptions(url, type, options = {}) {
+  ajaxOptions(url, verb, options = {}) {
     options.data || (options.data = {});
     if (this.get('system.shouldIncludeRegion')) {
+      // Region should only ever be a query param. The default ajaxOptions
+      // behavior is to include data attributes in the requestBody for PUT
+      // and POST requests. This works around that.
       const region = this.get('system.activeRegion');
       if (region) {
-        options.data.region = region;
+        url = associateRegion(url, region);
       }
     }
-    return this._super(url, type, options);
-  },
+    return super.ajaxOptions(url, verb, options);
+  }
 
   // In order to remove stale records from the store, findHasMany has to unload
   // all records related to the request in question.
   findHasMany(store, snapshot, link, relationship) {
-    return this._super(...arguments).then(payload => {
+    return super.findHasMany(...arguments).then(payload => {
       const relationshipType = relationship.type;
       const inverse = snapshot.record.inverseFor(relationship.key);
       if (inverse) {
@@ -71,7 +79,7 @@ export default RESTAdapter.extend({
       }
       return payload;
     });
-  },
+  }
 
   // Single record requests deviate from REST practice by using
   // the singular form of the resource name.
@@ -81,35 +89,40 @@ export default RESTAdapter.extend({
   //
   // This is the original implementation of _buildURL
   // without the pluralization of modelName
-  urlForFindRecord: urlForRecord,
-  urlForUpdateRecord: urlForRecord,
-});
+  urlForFindRecord(id, modelName) {
+    let path;
+    let url = [];
+    let host = get(this, 'host');
+    let prefix = this.urlPrefix();
 
-function urlForRecord(id, modelName) {
-  let path;
-  let url = [];
-  let host = get(this, 'host');
-  let prefix = this.urlPrefix();
-
-  if (modelName) {
-    path = modelName.camelize();
-    if (path) {
-      url.push(path);
+    if (modelName) {
+      path = modelName.camelize();
+      if (path) {
+        url.push(path);
+      }
     }
+
+    if (id) {
+      url.push(encodeURIComponent(id));
+    }
+
+    if (prefix) {
+      url.unshift(prefix);
+    }
+
+    url = url.join('/');
+    if (!host && url && url.charAt(0) !== '/') {
+      url = '/' + url;
+    }
+
+    return url;
   }
 
-  if (id) {
-    url.push(encodeURIComponent(id));
+  urlForUpdateRecord() {
+    return this.urlForFindRecord(...arguments);
   }
+}
 
-  if (prefix) {
-    url.unshift(prefix);
-  }
-
-  url = url.join('/');
-  if (!host && url && url.charAt(0) !== '/') {
-    url = '/' + url;
-  }
-
-  return url;
+function associateRegion(url, region) {
+  return url.indexOf('?') !== -1 ? `${url}&region=${region}` : `${url}?region=${region}`;
 }
