@@ -1,12 +1,15 @@
 import { inject as service } from '@ember/service';
 import Component from '@ember/component';
 import { computed } from '@ember/object';
+import classic from 'ember-classic-decorator';
 
-export default Component.extend({
-  system: service(),
-  router: service(),
+@classic
+export default class GutterMenu extends Component {
+  @service system;
+  @service router;
 
-  sortedNamespaces: computed('system.namespaces.@each.name', function() {
+  @computed('system.namespaces.@each.name')
+  get sortedNamespaces() {
     const namespaces = this.get('system.namespaces').toArray() || [];
 
     return namespaces.sort((a, b) => {
@@ -30,15 +33,21 @@ export default Component.extend({
 
       return 0;
     });
-  }),
+  }
 
-  onHamburgerClick() {},
+  onHamburgerClick() {}
 
   gotoJobsForNamespace(namespace) {
     if (!namespace || !namespace.get('id')) return;
 
-    this.router.transitionTo('jobs', {
+    // Jobs and CSI Volumes are both namespace-sensitive. Changing namespaces is
+    // an intent to reset context, but where to reset to depends on where the namespace
+    // is being switched from. Jobs take precedence, but if the namespace is switched from
+    // a storage-related page, context should be reset to volumes.
+    const destination = this.router.currentRouteName.startsWith('csi.') ? 'csi.volumes' : 'jobs';
+
+    this.router.transitionTo(destination, {
       queryParams: { namespace: namespace.get('id') },
     });
-  },
-});
+  }
+}
