@@ -1,10 +1,14 @@
 import { computed } from '@ember/object';
 import { alias } from '@ember/object/computed';
 import PeriodicChildJobPage from './periodic-child';
+import classic from 'ember-classic-decorator';
 
-export default PeriodicChildJobPage.extend({
-  payload: alias('job.decodedPayload'),
-  payloadJSON: computed('payload', function() {
+@classic
+export default class ParameterizedChild extends PeriodicChildJobPage {
+  @alias('job.decodedPayload') payload;
+
+  @computed('payload')
+  get payloadJSON() {
     let json;
     try {
       json = JSON.parse(this.payload);
@@ -12,5 +16,5 @@ export default PeriodicChildJobPage.extend({
       // Swallow error and fall back to plain text rendering
     }
     return json;
-  }),
-});
+  }
+}
