@@ -1,27 +1,34 @@
 import Component from '@ember/component';
 import { task } from 'ember-concurrency';
+import { ForbiddenError } from '@ember-data/adapter/error';
 import messageFromAdapterError from 'nomad-ui/utils/message-from-adapter-error';
+import { tagName } from '@ember-decorators/component';
+import classic from 'ember-classic-decorator';
 
-export default Component.extend({
-  job: null,
-  tagName: '',
+@classic
+@tagName('')
+export default class LatestDeployment extends Component {
+  job = null;
 
-  handleError() {},
+  handleError() {}
 
-  isShowingDeploymentDetails: false,
+  isShowingDeploymentDetails = false;
 
-  promote: task(function*() {
+  @task(function*() {
     try {
       yield this.get('job.latestDeployment.content').promote();
     } catch (err) {
       let message = messageFromAdapterError(err);
-      if (!message || message === 'Forbidden') {
+
+      if (err instanceof ForbiddenError) {
         message = 'Your ACL token does not grant permission to promote deployments.';
       }
+
       this.handleError({
         title: 'Could Not Promote Deployment',
         description: message,
       });
     }
-  }),
-});
+  })
+  promote;
+}
