@@ -1,17 +1,21 @@
 import Component from '@ember/component';
 import { computed } from '@ember/object';
+import { classNames } from '@ember-decorators/component';
+import classic from 'ember-classic-decorator';
 
-export default Component.extend({
-  job: null,
-  classNames: ['boxed-section'],
+@classic
+@classNames('boxed-section')
+export default class Summary extends Component {
+  job = null;
 
-  isExpanded: computed(function() {
+  @computed
+  get isExpanded() {
     const storageValue = window.localStorage.nomadExpandJobSummary;
     return storageValue != null ? JSON.parse(storageValue) : true;
-  }),
+  }
 
   persist(item, isOpen) {
     window.localStorage.nomadExpandJobSummary = isOpen;
     this.notifyPropertyChange('isExpanded');
-  },
-});
+  }
+}
