@@ -1,8 +1,8 @@
 import moment from 'moment';
 import Helper from '@ember/component/helper';
 
-export function formatMonthTs([date]) {
-  const format = 'MMM DD HH:mm:ss ZZ';
+export function formatMonthTs([date], options = {}) {
+  const format = options.short ? 'MMM D' : 'MMM DD HH:mm:ss ZZ';
   return moment(date).format(format);
 }
 
