@@ -3,8 +3,8 @@ import attr from 'ember-data/attr';
 import { fragmentArray } from 'ember-data-model-fragments/attributes';
 import { hasMany } from 'ember-data/relationships';
 
-export default Model.extend({
-  diff: attr(),
-  failedTGAllocs: fragmentArray('placement-failure', { defaultValue: () => [] }),
-  preemptions: hasMany('allocation'),
-});
+export default class JobPlan extends Model {
+  @attr() diff;
+  @fragmentArray('placement-failure', { defaultValue: () => [] }) failedTGAllocs;
+  @hasMany('allocation') preemptions;
+}
