@@ -1,6 +1,6 @@
 import Fragment from 'ember-data-model-fragments/fragment';
 import { fragmentArray } from 'ember-data-model-fragments/attributes';
 
-export default Fragment.extend({
-  upstreams: fragmentArray('sidecar-proxy-upstream'),
-});
+export default class SidecarProxy extends Fragment {
+  @fragmentArray('sidecar-proxy-upstream') upstreams;
+}
