@@ -1,6 +1,6 @@
 import Fragment from 'ember-data-model-fragments/fragment';
 import { fragment } from 'ember-data-model-fragments/attributes';
 
-export default Fragment.extend({
-  proxy: fragment('sidecar-proxy'),
-});
+export default class SidecarService extends Fragment {
+  @fragment('sidecar-proxy') proxy;
+}
