@@ -1,12 +1,18 @@
 import EmberRouter from '@ember/routing/router';
 import config from './config/environment';
 
-const Router = EmberRouter.extend({
-  location: config.locationType,
-  rootURL: config.rootURL,
-});
+export default class Router extends EmberRouter {
+  location = config.locationType;
+  rootURL = config.rootURL;
+}
 
 Router.map(function() {
+  this.route('exec', { path: '/exec/:job_name' }, function() {
+    this.route('task-group', { path: '/:task_group_name' }, function() {
+      this.route('task', { path: '/:task_name' });
+    });
+  });
+
   this.route('jobs', function() {
     this.route('run');
     this.route('job', { path: '/:job_name' }, function() {
@@ -20,15 +26,34 @@ Router.map(function() {
   });
 
   this.route('clients', function() {
-    this.route('client', { path: '/:node_id' });
+    this.route('client', { path: '/:node_id' }, function() {
+      this.route('monitor');
+    });
   });
 
   this.route('servers', function() {
-    this.route('server', { path: '/:agent_id' });
+    this.route('server', { path: '/:agent_id' }, function() {
+      this.route('monitor');
+    });
+  });
+
+  this.route('csi', function() {
+    this.route('volumes', function() {
+      this.route('volume', { path: '/:volume_name' });
+    });
+
+    this.route('plugins', function() {
+      this.route('plugin', { path: '/:plugin_name' }, function() {
+        this.route('allocations');
+      });
+    });
   });
 
   this.route('allocations', function() {
     this.route('allocation', { path: '/:allocation_id' }, function() {
+      this.route('fs-root', { path: '/fs' });
+      this.route('fs', { path: '/fs/*path' });
+
       this.route('task', { path: '/:name' }, function() {
         this.route('logs');
         this.route('fs-root', { path: '/fs' });
@@ -43,5 +68,3 @@ Router.map(function() {
 
   this.route('not-found', { path: '/*' });
 });
-
-export default Router;
