@@ -1,8 +1,8 @@
 import Route from '@ember/routing/route';
 
-export default Route.extend({
+export default class LogsRoute extends Route {
   model() {
-    const task = this._super(...arguments);
+    const task = super.model(...arguments);
     return task && task.get('allocation.node').then(() => task);
-  },
-});
+  }
+}
