@@ -3,16 +3,18 @@ import Route from '@ember/routing/route';
 import RSVP from 'rsvp';
 import notifyError from 'nomad-ui/utils/notify-error';
 import { jobCrumbs } from 'nomad-ui/utils/breadcrumb-utils';
+import classic from 'ember-classic-decorator';
 
-export default Route.extend({
-  store: service(),
-  token: service(),
+@classic
+export default class JobRoute extends Route {
+  @service store;
+  @service token;
 
-  breadcrumbs: jobCrumbs,
+  breadcrumbs = jobCrumbs;
 
   serialize(model) {
     return { job_name: model.get('plainId') };
-  },
+  }
 
   model(params, transition) {
     const namespace = transition.to.queryParams.namespace || this.get('system.activeNamespace.id');
@@ -24,5 +26,5 @@ export default Route.extend({
         return RSVP.all([job.get('allocations'), job.get('evaluations')]).then(() => job);
       })
       .catch(notifyError(this));
-  },
-});
+  }
+}
