@@ -3,19 +3,19 @@ import { collect } from '@ember/object/computed';
 import { watchRelationship } from 'nomad-ui/utils/properties/watch';
 import WithWatchers from 'nomad-ui/mixins/with-watchers';
 
-export default Route.extend(WithWatchers, {
+export default class AllocationsRoute extends Route.extend(WithWatchers) {
   model() {
     const job = this.modelFor('jobs.job');
     return job && job.get('allocations').then(() => job);
-  },
+  }
 
   startWatchers(controller, model) {
     if (model) {
       controller.set('watchAllocations', this.watchAllocations.perform(model));
     }
-  },
+  }
 
-  watchAllocations: watchRelationship('allocations'),
+  @watchRelationship('allocations') watchAllocations;
 
-  watchers: collect('watchAllocations'),
-});
+  @collect('watchAllocations') watchers;
+}
