@@ -1,10 +1,12 @@
 import ApplicationSerializer from './application';
 
-export default ApplicationSerializer.extend({
-  attrs: {
+export default class ResourcesSerializer extends ApplicationSerializer {
+  attrs = {
     cpu: 'CPU',
     memory: 'MemoryMB',
     disk: 'DiskMB',
     iops: 'IOPS',
-  },
-});
+  };
+
+  arrayNullOverrides = ['Ports'];
+}
