@@ -1,7 +1,10 @@
 import { copy } from 'ember-copy';
 import ApplicationSerializer from './application';
 
-export default ApplicationSerializer.extend({
+export default class TaskGroup extends ApplicationSerializer {
+  arrayNullOverrides = ['Services'];
+  mapToArray = ['Volumes'];
+
   normalize(typeHash, hash) {
     // Provide EphemeralDisk to each task
     hash.Tasks.forEach(task => {
@@ -9,8 +12,7 @@ export default ApplicationSerializer.extend({
     });
 
     hash.ReservedEphemeralDisk = hash.EphemeralDisk.SizeMB;
-    hash.Services = hash.Services || [];
 
-    return this._super(typeHash, hash);
-  },
-});
+    return super.normalize(typeHash, hash);
+  }
+}
