@@ -2,6 +2,6 @@ import AdapterError from '@ember-data/adapter/error';
 
 export const NO_LEADER = 'No cluster leader';
 
-export default AdapterError.extend({
-  message: NO_LEADER,
-});
+export default class NoLeaderError extends AdapterError {
+  message = NO_LEADER;
+}
