@@ -5,6 +5,8 @@ self.deprecationWorkflow.config = {
     { handler: 'throw', matchId: 'ember-inflector.globals' },
     { handler: 'throw', matchId: 'ember-runtime.deprecate-copy-copyable' },
     { handler: 'throw', matchId: 'ember-console.deprecate-logger' },
-    { handler: 'log', matchId: 'ember-test-helpers.rendering-context.jquery-element' },
+    { handler: 'throw', matchId: 'ember-test-helpers.rendering-context.jquery-element' },
+    { handler: 'throw', matchId: 'ember-cli-page-object.is-property' },
+    { handler: 'throw', matchId: 'ember-views.partial' },
   ],
 };
