@@ -23,6 +23,8 @@ export const HOSTS = provide(100, () => {
   return `${ip}:${faker.random.number({ min: 4000, max: 4999 })}`;
 });
 
+export const STORAGE_PROVIDERS = ['ebs', 'zfs', 'nfs', 'cow', 'moo'];
+
 export function generateResources(options = {}) {
   return {
     CPU: faker.helpers.randomize(CPU_RESERVATIONS),
@@ -30,6 +32,7 @@ export function generateResources(options = {}) {
     DiskMB: faker.helpers.randomize(DISK_RESERVATIONS),
     IOPS: faker.helpers.randomize(IOPS_RESERVATIONS),
     Networks: generateNetworks(options.networks),
+    Ports: generatePorts(options.networks),
   };
 }
 
@@ -68,3 +71,17 @@ export function generateNetworks(options = {}) {
         })),
     }));
 }
+
+export function generatePorts(options = {}) {
+  return Array(faker.random.number({
+    min: options.minPorts != null ? options.minPorts : 0,
+    max: options.maxPorts != null ? options.maxPorts : 2
+  }))
+    .fill(null)
+    .map(() => ({
+      Label: faker.hacker.noun(),
+      Value: faker.random.number({ min: 5000, max: 60000 }),
+      To: faker.random.number({ min: 5000, max: 60000 }),
+      HostIP: faker.random.boolean() ? faker.internet.ip() : faker.internet.ipv6(),
+    }))
+}
\ No newline at end of file
