@@ -1,16 +1,20 @@
-import { Factory, faker } from 'ember-cli-mirage';
+import { Factory } from 'ember-cli-mirage';
+
+import faker from 'nomad-ui/mirage/faker';
 
 const REF_TIME = new Date();
 
 export default Factory.extend({
   name: '',
 
-  autoRevert: () => Math.random() > 0.5,
-  promoted: () => Math.random() > 0.5,
+  autoRevert: () => faker.random.boolean(),
+  promoted: () => faker.random.boolean(),
+
+  requiresPromotion: false,
 
   requireProgressBy: () => faker.date.past(0.5 / 365, REF_TIME),
 
-  desiredTotal: faker.random.number({ min: 1, max: 10 }),
+  desiredTotal: () => faker.random.number({ min: 1, max: 10 }),
 
   desiredCanaries() {
     return faker.random.number(Math.floor(this.desiredTotal / 2));
