@@ -1,4 +1,5 @@
-import { Factory, faker, trait } from 'ember-cli-mirage';
+import { Factory, trait } from 'ember-cli-mirage';
+import faker from 'nomad-ui/mirage/faker';
 import { provide } from '../utils';
 
 const UUIDS = provide(100, faker.random.uuid.bind(faker.random));
@@ -10,11 +11,11 @@ export default Factory.extend({
   jobId: null,
   versionNumber: null,
 
-  status: faker.list.random(...DEPLOYMENT_STATUSES),
+  status: () => faker.helpers.randomize(DEPLOYMENT_STATUSES),
   statusDescription: () => faker.lorem.sentence(),
 
   notActive: trait({
-    status: faker.list.random(...DEPLOYMENT_STATUSES.without('running')),
+    status: faker.helpers.randomize(DEPLOYMENT_STATUSES.without('running')),
   }),
 
   active: trait({
@@ -27,6 +28,8 @@ export default Factory.extend({
       server.create('deployment-task-group-summary', {
         deployment,
         name: server.db.taskGroups.find(id).name,
+        desiredCanaries: 1,
+        promoted: false,
       })
     );
 
