@@ -1,7 +1,7 @@
 import { assign } from '@ember/polyfills';
 import { Factory, trait } from 'ember-cli-mirage';
 import faker from 'nomad-ui/mirage/faker';
-import { provide, provider, pickOne } from '../utils';
+import { provide, pickOne } from '../utils';
 import { DATACENTERS } from '../common';
 
 const JOB_PREFIXES = provide(5, faker.hacker.abbreviation);
@@ -18,12 +18,14 @@ export default Factory.extend({
     return this.id;
   },
 
+  version: 1,
+
   groupsCount: () => faker.random.number({ min: 1, max: 2 }),
 
   region: () => 'global',
   type: () => faker.helpers.randomize(JOB_TYPES),
   priority: () => faker.random.number(100),
-  all_at_once: faker.random.boolean,
+  allAtOnce: faker.random.boolean,
   status: () => faker.helpers.randomize(JOB_STATUSES),
   datacenters: () =>
     faker.helpers.shuffle(DATACENTERS).slice(0, faker.random.number({ min: 1, max: 4 })),
@@ -96,6 +98,9 @@ export default Factory.extend({
   // When true, no evaluations have failed placements
   noFailedPlacements: false,
 
+  // When true, all task groups get the noHostVolumes trait
+  noHostVolumes: false,
+
   // When true, allocations for this job will fail and reschedule, randomly succeeding or not
   withRescheduling: false,
 
@@ -118,31 +123,46 @@ export default Factory.extend({
       });
     }
 
-    const groups = server.createList('task-group', job.groupsCount, {
+    const groupProps = {
       job,
       createAllocations: job.createAllocations,
       withRescheduling: job.withRescheduling,
       withServices: job.withGroupServices,
       shallow: job.shallow,
-    });
+    };
+
+    if (job.groupTaskCount) {
+      groupProps.count = job.groupTaskCount;
+    }
+
+    const groups = job.noHostVolumes
+      ? server.createList('task-group', job.groupsCount, 'noHostVolumes', groupProps)
+      : server.createList('task-group', job.groupsCount, groupProps);
 
     job.update({
       taskGroupIds: groups.mapBy('id'),
-      task_group_ids: groups.mapBy('id'),
     });
 
     const hasChildren = job.periodic || (job.parameterized && !job.parentId);
     const jobSummary = server.create('job-summary', hasChildren ? 'withChildren' : 'withSummary', {
+      jobId: job.id,
       groupNames: groups.mapBy('name'),
-      job,
-      job_id: job.id,
-      JobID: job.id,
       namespace: job.namespace,
     });
 
     job.update({
       jobSummaryId: jobSummary.id,
-      job_summary_id: jobSummary.id,
+    });
+
+    const jobScale = server.create('job-scale', {
+      groupNames: groups.mapBy('name'),
+      jobId: job.id,
+      namespace: job.namespace,
+      shallow: job.shallow,
+    });
+
+    job.update({
+      jobScaleId: jobScale.id,
     });
 
     if (!job.noDeployments) {
