@@ -1,4 +1,6 @@
-import { Factory, faker } from 'ember-cli-mirage';
+import { Factory } from 'ember-cli-mirage';
+
+import faker from 'nomad-ui/mirage/faker';
 
 export default Factory.extend({
   id: i => (i === 0 ? 'default' : `namespace-${i}`),
