@@ -1,12 +1,14 @@
-import { Factory, faker } from 'ember-cli-mirage';
+import { Factory } from 'ember-cli-mirage';
+import faker from 'nomad-ui/mirage/faker';
 
 const TASK_STATUSES = ['pending', 'running', 'finished', 'failed'];
 const REF_TIME = new Date();
 
 export default Factory.extend({
   name: () => '!!!this should be set by the allocation that owns this task state!!!',
-  state: faker.list.random(...TASK_STATUSES),
-  startedAt: faker.date.past(2 / 365, REF_TIME),
+  state: () => faker.helpers.randomize(TASK_STATUSES),
+  kind: null,
+  startedAt: () => faker.date.past(2 / 365, REF_TIME),
   finishedAt() {
     if (['pending', 'running'].includes(this.state)) {
       return '0001-01-01T00:00:00Z';
@@ -17,7 +19,7 @@ export default Factory.extend({
   afterCreate(state, server) {
     const props = [
       'task-event',
-      faker.random.number({ min: 1, max: 10 }),
+      faker.random.number({ min: 1, max: 3 }),
       {
         taskStateId: state.id,
       },
