@@ -1,5 +1,5 @@
 import { Model, belongsTo } from 'ember-cli-mirage';
 
 export default Model.extend({
-  task_group: belongsTo('task-group'),
+  taskGroup: belongsTo('task-group'),
 });
