@@ -1,8 +1,9 @@
-import { currentURL, waitUntil } from '@ember/test-helpers';
+import { currentURL, waitUntil, settled } from '@ember/test-helpers';
 import { assign } from '@ember/polyfills';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
 import { formatBytes } from 'nomad-ui/helpers/format-bytes';
 import moment from 'moment';
 import ClientDetail from 'nomad-ui/tests/pages/clients/detail';
@@ -41,6 +42,11 @@ module('Acceptance | client detail', function(hooks) {
     });
   });
 
+  test('it passes an accessibility audit', async function(assert) {
+    await ClientDetail.visit({ id: node.id });
+    await a11yAudit(assert);
+  });
+
   test('/clients/:id should have a breadcrumb trail linking back to clients', async function(assert) {
     await ClientDetail.visit({ id: node.id });
 
@@ -150,6 +156,7 @@ module('Acceptance | client detail', function(hooks) {
     assert.equal(allocationRow.job, server.db.jobs.find(allocation.jobId).name, 'Job name');
     assert.ok(allocationRow.taskGroup, 'Task group name');
     assert.ok(allocationRow.jobVersion, 'Job Version');
+    assert.equal(allocationRow.volume, 'Yes', 'Volume');
     assert.equal(
       allocationRow.cpu,
       Math.floor(allocStats.resourceUsage.CpuStats.TotalTicks) / cpuUsed,
@@ -568,6 +575,8 @@ module('Acceptance | client detail', function(hooks) {
     assert.ok(ClientDetail.eligibilityToggle.isDisabled);
     server.pretender.resolve(server.pretender.requestReferences[0].request);
 
+    await settled();
+
     assert.notOk(ClientDetail.eligibilityToggle.isActive);
     assert.notOk(ClientDetail.eligibilityToggle.isDisabled);
 
@@ -900,6 +909,44 @@ module('Acceptance | client detail', function(hooks) {
     assert.ok(ClientDetail.eligibilityToggle.isDisabled);
     assert.ok(ClientDetail.drainPopover.isDisabled);
   });
+
+  test('the host volumes table lists all host volumes in alphabetical order by name', async function(assert) {
+    await ClientDetail.visit({ id: node.id });
+
+    const sortedHostVolumes = Object.keys(node.hostVolumes)
+      .map(key => node.hostVolumes[key])
+      .sortBy('Name');
+
+    assert.ok(ClientDetail.hasHostVolumes);
+    assert.equal(ClientDetail.hostVolumes.length, Object.keys(node.hostVolumes).length);
+
+    ClientDetail.hostVolumes.forEach((volume, index) => {
+      assert.equal(volume.name, sortedHostVolumes[index].Name);
+    });
+  });
+
+  test('each host volume row contains information about the host volume', async function(assert) {
+    await ClientDetail.visit({ id: node.id });
+
+    const sortedHostVolumes = Object.keys(node.hostVolumes)
+      .map(key => node.hostVolumes[key])
+      .sortBy('Name');
+
+    ClientDetail.hostVolumes[0].as(volume => {
+      const volumeRow = sortedHostVolumes[0];
+      assert.equal(volume.name, volumeRow.Name);
+      assert.equal(volume.path, volumeRow.Path);
+      assert.equal(volume.permissions, volumeRow.ReadOnly ? 'Read' : 'Read/Write');
+    });
+  });
+
+  test('the host volumes table is not shown if the client has no host volumes', async function(assert) {
+    node = server.create('node', 'noHostVolumes');
+
+    await ClientDetail.visit({ id: node.id });
+
+    assert.notOk(ClientDetail.hasHostVolumes);
+  });
 });
 
 module('Acceptance | client detail (multi-namespace)', function(hooks) {
