@@ -2,23 +2,38 @@ import { currentURL, settled } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
+import pageSizeSelect from './behaviors/page-size-select';
 import ClientsList from 'nomad-ui/tests/pages/clients/list';
 
 module('Acceptance | clients list', function(hooks) {
   setupApplicationTest(hooks);
   setupMirage(hooks);
 
+  hooks.beforeEach(function() {
+    window.localStorage.clear();
+  });
+
+  test('it passes an accessibility audit', async function(assert) {
+    const nodesCount = ClientsList.pageSize + 1;
+
+    server.createList('node', nodesCount);
+    server.createList('agent', 1);
+
+    await ClientsList.visit();
+    await a11yAudit(assert);
+  });
+
   test('/clients should list one page of clients', async function(assert) {
     // Make sure to make more nodes than 1 page to assert that pagination is working
-    const nodesCount = 10;
-    const pageSize = 8;
+    const nodesCount = ClientsList.pageSize + 1;
 
     server.createList('node', nodesCount);
     server.createList('agent', 1);
 
     await ClientsList.visit();
 
-    assert.equal(ClientsList.nodes.length, pageSize);
+    assert.equal(ClientsList.nodes.length, ClientsList.pageSize);
     assert.ok(ClientsList.hasPagination, 'Pagination found on the page');
 
     const sortedNodes = server.db.nodes.sortBy('modifyIndex').reverse();
@@ -51,6 +66,35 @@ module('Acceptance | clients list', function(hooks) {
     assert.equal(nodeRow.allocations, allocations.length, '# Allocations');
   });
 
+  test('each client record should show running allocations', async function(assert) {
+    server.createList('agent', 1);
+
+    const node = server.create('node', {
+      modifyIndex: 4,
+      status: 'ready',
+      schedulingEligibility: 'eligible',
+      drain: false,
+    });
+
+    server.create('job', { createAllocations: false });
+
+    const running = server.createList('allocation', 2, { clientStatus: 'running' });
+    server.createList('allocation', 3, { clientStatus: 'pending' });
+    server.createList('allocation', 10, { clientStatus: 'complete' });
+
+    await ClientsList.visit();
+
+    const nodeRow = ClientsList.nodes.objectAt(0);
+
+    assert.equal(nodeRow.id, node.id.split('-')[0], 'ID');
+    assert.equal(
+      nodeRow.compositeStatus.text,
+      'ready',
+      'Combined status, draining, and eligbility'
+    );
+    assert.equal(nodeRow.allocations, running.length, '# Allocations');
+  });
+
   test('client status, draining, and eligibility are collapsed into one column that stays sorted', async function(assert) {
     server.createList('agent', 1);
 
@@ -174,6 +218,17 @@ module('Acceptance | clients list', function(hooks) {
     assert.equal(currentURL(), '/settings/tokens');
   });
 
+  pageSizeSelect({
+    resourceName: 'client',
+    pageObject: ClientsList,
+    pageObjectList: ClientsList.nodes,
+    async setup() {
+      server.createList('node', ClientsList.pageSize);
+      server.createList('agent', 1);
+      await ClientsList.visit();
+    },
+  });
+
   testFacet('Class', {
     facet: ClientsList.facets.class,
     paramName: 'class',
@@ -232,6 +287,24 @@ module('Acceptance | clients list', function(hooks) {
     filter: (node, selection) => selection.includes(node.datacenter),
   });
 
+  testFacet('Volumes', {
+    facet: ClientsList.facets.volume,
+    paramName: 'volume',
+    expectedOptions(nodes) {
+      const flatten = (acc, val) => acc.concat(Object.keys(val));
+      return Array.from(new Set(nodes.mapBy('hostVolumes').reduce(flatten, [])));
+    },
+    async beforeEach() {
+      server.create('agent');
+      server.createList('node', 2, { hostVolumes: { One: { Name: 'One' } } });
+      server.createList('node', 2, { hostVolumes: { One: { Name: 'One' }, Two: { Name: 'Two' } } });
+      server.createList('node', 2, { hostVolumes: { Two: { Name: 'Two' } } });
+      await ClientsList.visit();
+    },
+    filter: (node, selection) =>
+      Object.keys(node.hostVolumes).find(volume => selection.includes(volume)),
+  });
+
   test('when the facet selections result in no matches, the empty state states why', async function(assert) {
     server.create('agent');
     server.createList('node', 2, { status: 'ready' });
