@@ -2,6 +2,7 @@ import { currentURL } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
 import setupCodeMirror from 'nomad-ui/tests/helpers/codemirror';
 import Definition from 'nomad-ui/tests/pages/jobs/job/definition';
 
@@ -19,6 +20,10 @@ module('Acceptance | job definition', function(hooks) {
     await Definition.visit({ id: job.id });
   });
 
+  test('it passes an accessibility audit', async function(assert) {
+    await a11yAudit(assert, 'scrollable-region-focusable');
+  });
+
   test('visiting /jobs/:job_id/definition', async function(assert) {
     assert.equal(currentURL(), `/jobs/${job.id}/definition`);
     assert.equal(document.title, `Job ${job.name} definition - Nomad`);
