@@ -2,6 +2,7 @@ import { currentURL } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
 import Evaluations from 'nomad-ui/tests/pages/jobs/job/evaluations';
 
 let job;
@@ -18,6 +19,10 @@ module('Acceptance | job evaluations', function(hooks) {
     await Evaluations.visit({ id: job.id });
   });
 
+  test('it passes an accessibility audit', async function(assert) {
+    await a11yAudit(assert);
+  });
+
   test('lists all evaluations for the job', async function(assert) {
     assert.equal(Evaluations.evaluations.length, evaluations.length, 'All evaluations are listed');
 
