@@ -2,6 +2,7 @@ import { currentURL } from '@ember/test-helpers';
 import { module, test } from 'qunit';
 import { setupApplicationTest } from 'ember-qunit';
 import { setupMirage } from 'ember-cli-mirage/test-support';
+import a11yAudit from 'nomad-ui/tests/helpers/a11y-audit';
 import Versions from 'nomad-ui/tests/pages/jobs/job/versions';
 import moment from 'moment';
 
@@ -19,6 +20,10 @@ module('Acceptance | job versions', function(hooks) {
     await Versions.visit({ id: job.id });
   });
 
+  test('it passes an accessibility audit', async function(assert) {
+    await a11yAudit(assert);
+  });
+
   test('/jobs/:id/versions should list all job versions', async function(assert) {
     assert.ok(Versions.versions.length, versions.length, 'Each version gets a row in the timeline');
     assert.equal(document.title, `Job ${job.name} versions - Nomad`);
